#include "inssel.h"


#ifdef MONO_ARCH_NEED_GOT_VAR

static void mini_emit_aotconst (MonoCompile *cfg, int dreg, MonoJumpInfoType patch_type, gpointer cons);

#endif

/* 
 * Emit code which checks whenever the interface id of @klass is smaller than
 * than the value given by max_iid_reg.
*/
static void
mini_emit_max_iid_check (MonoCompile *s, int max_iid_reg, MonoClass *klass,
						 MonoInst *false_target)
{
	if (s->compile_aot) {
		int iid_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_AOTCONST (s, iid_reg, klass, MONO_PATCH_INFO_IID);
		if (false_target)
			MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BLT_UN, max_iid_reg, iid_reg, false_target);
		else
			MONO_EMIT_NEW_COMPARE_EXC (s, LT_UN, max_iid_reg, iid_reg, "InvalidCastException");
	} else {
		if (false_target)
			MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BLT_UN, max_iid_reg, klass->interface_id, false_target);
		else
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT_UN, max_iid_reg, klass->interface_id, "InvalidCastException");
	}
}

/* Same as above, but obtains max_iid from a vtable */
static void
mini_emit_max_iid_check_vtable (MonoCompile *s, int vtable_reg, MonoClass *klass,
								 MonoInst *false_target)
{
	int max_iid_reg = mono_regstate_next_int (s->rs);
		
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU2_MEMBASE, max_iid_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, max_interface_id));
	mini_emit_max_iid_check (s, max_iid_reg, klass, false_target);
}

/* Same as above, but obtains max_iid from a klass */
static void
mini_emit_max_iid_check_class (MonoCompile *s, int klass_reg, MonoClass *klass,
								 MonoInst *false_target)
{
	int max_iid_reg = mono_regstate_next_int (s->rs);

 	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU2_MEMBASE, max_iid_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, max_interface_id));		
	mini_emit_max_iid_check (s, max_iid_reg, klass, false_target);
}

/* Emit code which loads interface_offsets [klass->interface_id]
 * The array is stored in memory before vtable.
 */
static void
mini_emit_load_intf_reg_vtable (MonoCompile *s, int intf_reg, int vtable_reg, MonoClass *klass)
{
	if (s->compile_aot) {
		int ioffset_reg = mono_regstate_next_int (s->rs);
		int iid_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_AOTCONST (s, iid_reg, klass, MONO_PATCH_INFO_ADJUSTED_IID);
		MONO_EMIT_NEW_BIALU (s, OP_PADD, ioffset_reg, iid_reg, vtable_reg);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, intf_reg, ioffset_reg, 0);
	} else {
		MONO_EMIT_NEW_LOAD_MEMBASE (s, intf_reg, vtable_reg, -((klass->interface_id + 1) * SIZEOF_VOID_P));
	}
}

/* 
 * Emit code which loads into "intf_bit_reg" a nonzero value if the MonoClass
 * stored in "klass_reg" implements the interface "klass".
 */
static void
mini_emit_load_intf_bit_reg_class (MonoCompile *s, int intf_bit_reg, int klass_reg, MonoClass *klass)
{
	int ibitmap_reg = mono_regstate_next_int (s->rs);
	int ibitmap_byte_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_LOAD_MEMBASE (s, ibitmap_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, interface_bitmap));

	if (s->compile_aot) {
		int iid_reg = mono_regstate_next_int (s->rs);
		int shifted_iid_reg = mono_regstate_next_int (s->rs);
		int ibitmap_byte_address_reg = mono_regstate_next_int (s->rs);
		int masked_iid_reg = mono_regstate_next_int (s->rs);
		int iid_one_bit_reg = mono_regstate_next_int (s->rs);
		int iid_bit_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_AOTCONST (s, iid_reg, klass, MONO_PATCH_INFO_IID);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SHR_IMM, shifted_iid_reg, iid_reg, 3);
		MONO_EMIT_NEW_BIALU (s, CEE_ADD, ibitmap_byte_address_reg, ibitmap_reg, shifted_iid_reg);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, ibitmap_byte_reg, ibitmap_byte_address_reg, 0);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_AND_IMM, masked_iid_reg, iid_reg, 7);
		MONO_EMIT_NEW_ICONST (s, iid_one_bit_reg, 1);
		MONO_EMIT_NEW_BIALU (s, CEE_SHL, iid_bit_reg, iid_one_bit_reg, masked_iid_reg);
		MONO_EMIT_NEW_BIALU (s, CEE_AND, intf_bit_reg, ibitmap_byte_reg, iid_bit_reg);
	} else {
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI1_MEMBASE, ibitmap_byte_reg, ibitmap_reg, klass->interface_id >> 3);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_AND_IMM, intf_bit_reg, ibitmap_byte_reg, 1 << (klass->interface_id & 7));
	}
}
/* 
 * Emit code which loads into "intf_bit_reg" a nonzero value if the MonoVTable
 * stored in "vtable_reg" implements the interface "klass".
 */
static void
mini_emit_load_intf_bit_reg_vtable (MonoCompile *s, int intf_bit_reg, int vtable_reg, MonoClass *klass)
{
	int ibitmap_reg = mono_regstate_next_int (s->rs);
	int ibitmap_byte_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_LOAD_MEMBASE (s, ibitmap_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, interface_bitmap));

	if (s->compile_aot) {
		int iid_reg = mono_regstate_next_int (s->rs);
		int shifted_iid_reg = mono_regstate_next_int (s->rs);
		int ibitmap_byte_address_reg = mono_regstate_next_int (s->rs);
		int masked_iid_reg = mono_regstate_next_int (s->rs);
		int iid_one_bit_reg = mono_regstate_next_int (s->rs);
		int iid_bit_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_AOTCONST (s, iid_reg, klass, MONO_PATCH_INFO_IID);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SHR_IMM, shifted_iid_reg, iid_reg, 3);
		MONO_EMIT_NEW_BIALU (s, CEE_ADD, ibitmap_byte_address_reg, ibitmap_reg, shifted_iid_reg);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, ibitmap_byte_reg, ibitmap_byte_address_reg, 0);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_AND_IMM, masked_iid_reg, iid_reg, 7);
		MONO_EMIT_NEW_ICONST (s, iid_one_bit_reg, 1);
		MONO_EMIT_NEW_BIALU (s, CEE_SHL, iid_bit_reg, iid_one_bit_reg, masked_iid_reg);
		MONO_EMIT_NEW_BIALU (s, CEE_AND, intf_bit_reg, ibitmap_byte_reg, iid_bit_reg);
	} else {
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI1_MEMBASE, ibitmap_byte_reg, ibitmap_reg, klass->interface_id >> 3);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_AND_IMM, intf_bit_reg, ibitmap_byte_reg, 1 << (klass->interface_id & 7));
	}
}

#ifdef MONO_ARCH_HAVE_IMT
static void
emit_imt_argument (MonoCompile *cfg, MonoCallInst *call, gboolean have_imt_arg, int imt_reg) {
#ifdef MONO_ARCH_IMT_REG
	int method_reg = mono_regstate_next_int (cfg->rs);

	if (cfg->compile_aot) {
		g_assert (!have_imt_arg);
		MONO_EMIT_NEW_AOTCONST (cfg, method_reg, call->method, MONO_PATCH_INFO_METHODCONST);
	} else {
		if (have_imt_arg) {
			MONO_EMIT_NEW_UNALU (cfg, OP_MOVE, method_reg, imt_reg);
		} else {
			MonoInst *inst;
			MONO_INST_NEW (cfg, inst, OP_PCONST);
			inst->inst_p0 = call->method;
			inst->dreg = method_reg;
			mono_bblock_add_inst (cfg->cbb, inst);
		}
	}

	mono_call_inst_add_outarg_reg (cfg, call, method_reg, MONO_ARCH_IMT_REG, FALSE);
#else
	mono_arch_emit_imt_argument (cfg, call);
#endif
}
#endif

static void 
mini_emit_virtual_call_internal (MonoCompile *cfg, void *st, MonoInst *tree, int novirtop, int virtop,
		gboolean have_imt_arg, int imt_reg)
{
	MonoMethod *method = ((MonoCallInst*)tree)->method;
	int vtable_reg, slot_reg, this_reg;
	MBState *state = st;
	int slot;

	this_reg = state->left->reg1;
	mono_arch_emit_this_vret_args (cfg, (MonoCallInst*)tree, this_reg, state->left->tree->type, novirtop == OP_VCALL? state->right->reg1: -1);

	if (!(method->flags & METHOD_ATTRIBUTE_VIRTUAL) || 
	     ((method->flags & METHOD_ATTRIBUTE_FINAL) && 
	     method->wrapper_type != MONO_WRAPPER_REMOTING_INVOKE_WITH_CHECK)) {
		/* 
		 * the method is not virtual, we just need to ensure this is not null
		 * and then we can call the method directly.
		 */
		if (method->klass->marshalbyref || method->klass == mono_defaults.object_class) {
			method = ((MonoCallInst*)tree)->method = mono_marshal_get_remoting_invoke_with_check (method);
		}

		if (!method->string_ctor)
			MONO_EMIT_NEW_UNALU (cfg, OP_CHECK_THIS, -1, this_reg);

		tree->dreg = state->reg1;
		tree->opcode = novirtop;
		mono_bblock_add_inst (cfg->cbb, tree);
		return;
	}

#ifdef MONO_ARCH_HAVE_CREATE_DELEGATE_TRAMPOLINE
	if ((method->klass->parent == mono_defaults.multicastdelegate_class) && (!strcmp (method->name, "Invoke"))) {
		/* Make a call to delegate->invoke_impl */
		tree->dreg = state->reg1;
		tree->opcode = virtop;
		tree->inst_basereg = this_reg;
		tree->inst_offset = G_STRUCT_OFFSET (MonoDelegate, invoke_impl);
		mono_bblock_add_inst (cfg->cbb, tree);
		return;
	}
#endif
	
	if ((method->flags & METHOD_ATTRIBUTE_VIRTUAL) &&
		((method->flags &  METHOD_ATTRIBUTE_FINAL) ||
			(method->klass && method->klass->flags & TYPE_ATTRIBUTE_SEALED))) {
		/*
		 * the method is virtual, but we can statically dispatch since either
		 * it's class or the method itself are sealed.
		 * But first we need to ensure it's not a null reference.
		 */
		MONO_EMIT_NEW_UNALU (cfg, OP_CHECK_THIS, -1, this_reg);

		tree->dreg = state->reg1;
		tree->opcode = novirtop;
		mono_bblock_add_inst (cfg->cbb, tree);
		return;
	}

	if (method->klass->generic_class && !method->klass->vtable) {
	   MonoMethod *gmethod = mono_method_get_declaring_generic_method (method);
	   mono_class_setup_vtable (gmethod->klass);

	   slot = gmethod->slot;
	   g_assert (slot != -1);
	} else {
		/* Initialize method->slot */
		mono_class_setup_vtable (method->klass);
		slot = method->slot;
	}

	vtable_reg = mono_regstate_next_int (cfg->rs);
	MONO_EMIT_NEW_LOAD_MEMBASE (cfg, vtable_reg, this_reg, G_STRUCT_OFFSET (MonoObject, vtable));
	if (method->klass->flags & TYPE_ATTRIBUTE_INTERFACE) {
		slot_reg = -1;
#ifdef MONO_ARCH_HAVE_IMT
		if (mono_use_imt) {
			guint32 imt_slot = mono_method_get_imt_slot (method);
			emit_imt_argument (cfg, (MonoCallInst*)tree, have_imt_arg, imt_reg);
			slot_reg = vtable_reg;
			tree->inst_offset = ((gint32)imt_slot - MONO_IMT_SIZE) * SIZEOF_VOID_P;
		}
#endif
		if (slot_reg == -1) {
			g_assert (!have_imt_arg);
			slot_reg = mono_regstate_next_int (cfg->rs);
			mini_emit_load_intf_reg_vtable (cfg, slot_reg, vtable_reg, method->klass);
			tree->inst_offset = slot * SIZEOF_VOID_P;
		}
	} else {
		slot_reg = vtable_reg;
		tree->inst_offset = G_STRUCT_OFFSET (MonoVTable, vtable) + (slot * SIZEOF_VOID_P);
	}
	
	tree->dreg = state->reg1;
	tree->opcode = virtop;
	tree->sreg1 = slot_reg;
	((MonoCallInst*)tree)->virtual = TRUE;
	mono_bblock_add_inst (cfg->cbb, tree);
}

static void 
mini_emit_virtual_call (MonoCompile *cfg, void *st, MonoInst *tree, int novirtop, int virtop)
{
	mini_emit_virtual_call_internal (cfg, st, tree, novirtop, virtop, FALSE, -1);
}

static void 
mini_emit_virtual_imt_call (MonoCompile *cfg, void *st, MonoInst *tree, int novirtop, int virtop, int imt_reg)
{
	mini_emit_virtual_call_internal (cfg, st, tree, novirtop, virtop, TRUE, imt_reg);
}

static void
emit_rgctx_argument (MonoCompile *s, MonoInst *tree, int reg, int new_opcode)
{
#ifdef MONO_ARCH_RGCTX_REG
	MonoCallInst *call = (MonoCallInst*)tree;
	int rgctx_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_UNALU (s, OP_MOVE, rgctx_reg, reg);

	call->inst.opcode = new_opcode;

	mono_call_inst_add_outarg_reg (s, call, rgctx_reg, MONO_ARCH_RGCTX_REG, FALSE);
#else
	g_assert_not_reached ();
#endif
}

/*
 * Emit code for ISINST/CASTCLASS
 */
static void
mini_emit_isinst (MonoCompile *s, int klass_reg, MonoClass *klass, MonoInst *false_target, MonoInst *true_target)
{
	int idepth_reg = mono_regstate_next_int (s->rs);
	int stypes_reg = mono_regstate_next_int (s->rs);
	int stype = mono_regstate_next_int (s->rs);

	if (klass->idepth > MONO_DEFAULT_SUPERTABLE_SIZE) {
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU2_MEMBASE, idepth_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, idepth));
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BLT_UN, idepth_reg, klass->idepth, false_target);
	}
	MONO_EMIT_NEW_LOAD_MEMBASE (s, stypes_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, supertypes));
	MONO_EMIT_NEW_LOAD_MEMBASE (s, stype, stypes_reg, ((klass->idepth - 1) * SIZEOF_VOID_P));
	if (s->compile_aot) {
		int const_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_CLASSCONST (s, const_reg, klass);
		MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, stype, const_reg, true_target);
	} else {
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, stype, klass, true_target);
	}
}

/*
 * Emit code for ISINST/CASTCLASS for interfaces
 */
static void
mini_emit_isinst_iface (MonoCompile *s, int vtable_reg, MonoClass *klass, MonoInst *false_target, MonoInst *true_target)
{
	int intf_reg = mono_regstate_next_int (s->rs);

	mini_emit_max_iid_check_vtable (s, vtable_reg, klass, false_target);
	mini_emit_load_intf_bit_reg_vtable (s, intf_reg, vtable_reg, klass);
	if (true_target)
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, intf_reg, 0, true_target);
	else
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, EQ, intf_reg, 0, "InvalidCastException");
}

/*
 * Variant of the above that takes a register to the class, not the vtable.
 */
static void
mini_emit_isinst_iface_class (MonoCompile *s, int klass_reg, MonoClass *klass, MonoInst *false_target, MonoInst *true_target)
{
	int intf_bit_reg = mono_regstate_next_int (s->rs);

	mini_emit_max_iid_check_class (s, klass_reg, klass, false_target);
	mini_emit_load_intf_bit_reg_class (s, intf_bit_reg, klass_reg, klass);
	if (true_target)
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, intf_bit_reg, 0, true_target);
	else
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, EQ, intf_bit_reg, 0, "InvalidCastException");
}

static void
mini_emit_castclass_iface (MonoCompile *s, int vtable_reg, MonoClass *klass)
{
	mini_emit_isinst_iface (s, vtable_reg, klass, NULL, NULL);
}

/*
 * Variant of the aboce that takes a register to the class, not the vtable.
 */
static void
mini_emit_castclass_iface_class (MonoCompile *s, int klass_reg, MonoClass *klass)
{
	mini_emit_isinst_iface_class (s, klass_reg, klass, NULL, NULL);
}

static void
mini_emit_castclass (MonoCompile *s, int obj_reg, int klass_reg, MonoClass *klass, MonoInst *object_is_null)
{
	if (klass->rank) {
		int rank_reg = mono_regstate_next_int (s->rs);
		int eclass_reg = mono_regstate_next_int (s->rs);

		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, rank_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, rank));
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, rank_reg, klass->rank, "InvalidCastException");
//		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, eclass_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, cast_class));
		if (klass->cast_class == mono_defaults.object_class) {
			int parent_reg = mono_regstate_next_int (s->rs);
			int const_reg = -1;
			MONO_EMIT_NEW_LOAD_MEMBASE (s, parent_reg, eclass_reg, G_STRUCT_OFFSET (MonoClass, parent));
			if (s->compile_aot) {
				const_reg = mono_regstate_next_int (s->rs);
				MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class->parent);
				MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BNE_UN, parent_reg, const_reg, object_is_null);
			} else {
				MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, parent_reg,
					mono_defaults.enum_class->parent, object_is_null);
			}
			if (s->compile_aot) {
				MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
				MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, eclass_reg, const_reg, "InvalidCastException");
			} else {
				MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, eclass_reg, mono_defaults.enum_class, "InvalidCastException");
			}
		} else if (klass->cast_class == mono_defaults.enum_class->parent) {
			int const_reg = mono_regstate_next_int (s->rs);
			if (s->compile_aot) {
				MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class->parent);
				MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, eclass_reg, const_reg, object_is_null);
			} else {
				MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, eclass_reg,
					mono_defaults.enum_class->parent, object_is_null);
			}
			if (s->compile_aot) {
				MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
				MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, eclass_reg, const_reg, "InvalidCastException");
			} else {
				MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, eclass_reg, mono_defaults.enum_class, "InvalidCastException");
			}
		} else if (klass->cast_class == mono_defaults.enum_class) {
			if (s->compile_aot) {
				int const_reg = mono_regstate_next_int (s->rs);
				MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
				MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, eclass_reg, const_reg, "InvalidCastException");
			} else {
				MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, eclass_reg, mono_defaults.enum_class, "InvalidCastException");
			}
		} else if (klass->cast_class->flags & TYPE_ATTRIBUTE_INTERFACE) {
			mini_emit_castclass_iface_class (s, eclass_reg, klass->cast_class);
		} else {
			// Pass -1 as obj_reg to skip the check below for arrays of arrays
			mini_emit_castclass (s, -1, eclass_reg, klass->cast_class, object_is_null);
		}

		if ((klass->rank == 1) && (klass->byval_arg.type == MONO_TYPE_SZARRAY) && (obj_reg != -1)) {
			/* Check that the object is a vector too */
			int bounds_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_LOAD_MEMBASE (s, bounds_reg, obj_reg, G_STRUCT_OFFSET (MonoArray, bounds));
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, bounds_reg, 0, "InvalidCastException");
		}
	} else {
		int idepth_reg = mono_regstate_next_int (s->rs);
		int stypes_reg = mono_regstate_next_int (s->rs);
		int stype = mono_regstate_next_int (s->rs);

		if (klass->idepth > MONO_DEFAULT_SUPERTABLE_SIZE) {
			MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU2_MEMBASE, idepth_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, idepth));
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT_UN, idepth_reg, klass->idepth, "InvalidCastException");
		}
		MONO_EMIT_NEW_LOAD_MEMBASE (s, stypes_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, supertypes));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, stype, stypes_reg, ((klass->idepth - 1) * SIZEOF_VOID_P));
		if (s->compile_aot) {
			int const_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, const_reg, klass);
			MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, stype, const_reg, "InvalidCastException");
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, stype, klass, "InvalidCastException");
		}
	}
}

static void
mini_emit_memcpy (MonoCompile *s, int destreg, int doffset, int srcreg, int soffset, int size, int align)
{
	int cur_reg;

	if (align < 4) {
		/* This could be optimized further if neccesary */
		while (size >= 1) {
			cur_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI1_MEMBASE, cur_reg, srcreg, soffset);
			MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI1_MEMBASE_REG, destreg, doffset, cur_reg);
			doffset += 1;
			soffset += 1;
			size -= 1;
		}
	}

	/* FIXME: consider alignment for archs that need it. */
#if !NO_UNALIGNED_ACCESS
	if (sizeof (gpointer) == 8) {
		while (size >= 8) {
			cur_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI8_MEMBASE, cur_reg, srcreg, soffset);
			MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI8_MEMBASE_REG, destreg, doffset, cur_reg);
			doffset += 8;
			soffset += 8;
			size -= 8;
		}
	}	
#endif

	while (size >= 4) {
		cur_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, cur_reg, srcreg, soffset);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI4_MEMBASE_REG, destreg, doffset, cur_reg);
		doffset += 4;
		soffset += 4;
		size -= 4;
	}
	while (size >= 2) {
		cur_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI2_MEMBASE, cur_reg, srcreg, soffset);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI2_MEMBASE_REG, destreg, doffset, cur_reg);
		doffset += 2;
		soffset += 2;
		size -= 2;
	}
	while (size >= 1) {
		cur_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI1_MEMBASE, cur_reg, srcreg, soffset);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI1_MEMBASE_REG, destreg, doffset, cur_reg);
		doffset += 1;
		soffset += 1;
		size -= 1;
	}
}

static void
mini_emit_memset (MonoCompile *s, int destreg, int offset, int size, int val, int align)
{
	int val_reg = mono_regstate_next_int (s->rs);

	if (sizeof (gpointer) == 8)
		MONO_EMIT_NEW_I8CONST (s, val_reg, val);
	else
		MONO_EMIT_NEW_ICONST (s, val_reg, val);

	if (align < 4) {
		/* This could be optimized further if neccesary */
		while (size >= 1) {
			MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI1_MEMBASE_REG, destreg, offset, val_reg);
			offset += 1;
			size -= 1;
		}
		return;
	}	

	/* FIXME: consider alignment for archs that need it. */
#if !NO_UNALIGNED_ACCESS
	if (sizeof (gpointer) == 8) {
		if (offset % 8) {
			MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI4_MEMBASE_REG, destreg, offset, val_reg);
			offset += 4;
			size -= 4;
		}
		while (size >= 8) {
			MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI8_MEMBASE_REG, destreg, offset, val_reg);
			offset += 8;
			size -= 8;
		}
	}	
#endif

	while (size >= 4) {
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI4_MEMBASE_REG, destreg, offset, val_reg);
		offset += 4;
		size -= 4;
	}
	while (size >= 2) {
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI2_MEMBASE_REG, destreg, offset, val_reg);
		offset += 2;
		size -= 2;
	}
	while (size >= 1) {
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI1_MEMBASE_REG, destreg, offset, val_reg);
		offset += 1;
		size -= 1;
	}
}

#ifdef MONO_ARCH_NEED_GOT_VAR

/* This is similar to NEW_AOTCONST in mini.c, but generates linear IL */
static void
mini_emit_aotconst (MonoCompile *cfg, int dreg, MonoJumpInfoType patch_type, gpointer cons)
{
	MonoInst *dest, *pinfo;
	int got_var_reg;

	g_assert (cfg->got_var);

	pinfo = mono_mempool_alloc0 ((cfg)->mempool, sizeof (MonoInst));
	pinfo->opcode = OP_PATCH_INFO;
	pinfo->inst_left = cons;
	pinfo->inst_right = (gpointer)(patch_type);

	if (cfg->got_var->opcode == OP_REGOFFSET) {
		got_var_reg = mono_regstate_next_int (cfg->rs);
		MONO_EMIT_NEW_LOAD_MEMBASE (cfg, got_var_reg, cfg->got_var->inst_basereg, cfg->got_var->inst_offset);
	} else {
		g_assert (cfg->got_var->opcode == OP_REGVAR);
		got_var_reg = cfg->got_var->dreg;
	}

	MONO_INST_NEW ((cfg), dest, OP_GOT_ENTRY);
	dest->inst_left = cfg->got_var; /* not used */
	dest->inst_right = pinfo;

	dest->inst_basereg = got_var_reg;
	dest->dreg = dreg;
	mono_bblock_add_inst (cfg->cbb, dest);
}

#endif

static int
ldind_to_load_membase (int opcode)
{
	switch (opcode) {
	case CEE_LDIND_I1:
		return OP_LOADI1_MEMBASE;
	case CEE_LDIND_U1:
		return OP_LOADU1_MEMBASE;
	case CEE_LDIND_I2:
		return OP_LOADI2_MEMBASE;
	case CEE_LDIND_U2:
		return OP_LOADU2_MEMBASE;
	case CEE_LDIND_I4:
		return OP_LOADI4_MEMBASE;
	case CEE_LDIND_U4:
		return OP_LOADU4_MEMBASE;
	case CEE_LDIND_I:
		return OP_LOAD_MEMBASE;
	case CEE_LDIND_REF:
		return OP_LOAD_MEMBASE;
	case CEE_LDIND_I8:
		return OP_LOADI8_MEMBASE;
	default:
		g_assert_not_reached ();
	}

	return -1;
}

static int
bialu_to_bialu_imm (int opcode)
{
	switch (opcode) {
	case OP_LMUL:
		return OP_LMUL_IMM;
	case OP_LSHL:
		return OP_LSHL_IMM;
	case OP_LSHR:
		return OP_LSHR_IMM;
	case OP_LSHR_UN:
		return OP_LSHR_UN_IMM;
	default:
		g_assert_not_reached ();
	}

	return -1;
}

static int
cbranch_to_fcbranch (int opcode)
{
	switch (opcode) {
	case CEE_BNE_UN:
		return OP_FBNE_UN;
	case CEE_BEQ:
		return OP_FBEQ;
	case CEE_BLT:
		return OP_FBLT;
	case CEE_BLT_UN:
		return OP_FBLT_UN;
	case CEE_BGT:
		return OP_FBGT;
	case CEE_BGT_UN:
		return OP_FBGT_UN;
	case CEE_BGE:
		return OP_FBGE;
	case CEE_BGE_UN:
		return OP_FBGE_UN;
	case CEE_BLE:
		return OP_FBLE;
	case CEE_BLE_UN:
		return OP_FBLE_UN;
	default:
		g_assert_not_reached ();
	}

	return -1;
}

static int
ceq_to_fceq (int opcode)
{
	switch (opcode) {
	case OP_CEQ:
		return OP_FCEQ;
	case OP_CLT:
		return OP_FCLT;
	case OP_CGT:
		return OP_FCGT;
	case OP_CLT_UN:
		return OP_FCLT_UN;
	case OP_CGT_UN:
		return OP_FCGT_UN;
	default:
		g_assert_not_reached ();
	}

	return -1;
}

static int
call_reg_to_call_membase (int opcode)
{
	switch (opcode) {
	case OP_CALL_REG:
		return OP_CALL_MEMBASE;
	case OP_FCALL_REG:
		return OP_FCALL_MEMBASE;
	case OP_VCALL_REG:
		return OP_VCALL_MEMBASE;
	case OP_LCALL_REG:
		return OP_LCALL_MEMBASE;
	case OP_VOIDCALL_REG:
		return OP_VOIDCALL_MEMBASE;
	default:
		g_assert_not_reached ();
	}

	return -1;
}

#if HAVE_ARRAY_ELEM_INIT
const guint8 mono_burg_arity [MBMAX_OPCODES] = {
	 [OP_REGVAR] = 0, /* OP_REGVAR */
	 [CEE_LDIND_I] = 1, /* CEE_LDIND_I */
	 [OP_REGOFFSET] = 0, /* OP_REGOFFSET */
	 [OP_LDADDR] = 1, /* OP_LDADDR */
	 [CEE_LDOBJ] = 1, /* CEE_LDOBJ */
	 [OP_ICONST] = 0, /* OP_ICONST */
	 [CEE_ADD] = 2, /* CEE_ADD */
	 [CEE_LDIND_REF] = 1, /* CEE_LDIND_REF */
	 [OP_AOTCONST] = 0, /* OP_AOTCONST */
	 [OP_LOAD_GOTADDR] = 0, /* OP_LOAD_GOTADDR */
	 [CEE_STIND_I] = 2, /* CEE_STIND_I */
	 [OP_PATCH_INFO] = 0, /* OP_PATCH_INFO */
	 [OP_GOT_ENTRY] = 2, /* OP_GOT_ENTRY */
	 [OP_DUMMY_USE] = 1, /* OP_DUMMY_USE */
	 [OP_DUMMY_STORE] = 0, /* OP_DUMMY_STORE */
	 [OP_NOT_REACHED] = 0, /* OP_NOT_REACHED */
	 [OP_MEMORY_BARRIER] = 0, /* OP_MEMORY_BARRIER */
	 [CEE_LDIND_I1] = 1, /* CEE_LDIND_I1 */
	 [CEE_LDIND_U1] = 1, /* CEE_LDIND_U1 */
	 [CEE_LDIND_I2] = 1, /* CEE_LDIND_I2 */
	 [CEE_LDIND_U2] = 1, /* CEE_LDIND_U2 */
	 [CEE_LDIND_I4] = 1, /* CEE_LDIND_I4 */
	 [CEE_LDIND_U4] = 1, /* CEE_LDIND_U4 */
	 [OP_OBJADDR] = 1, /* OP_OBJADDR */
	 [OP_VTADDR] = 1, /* OP_VTADDR */
	 [CEE_STIND_REF] = 2, /* CEE_STIND_REF */
	 [CEE_STIND_I4] = 2, /* CEE_STIND_I4 */
	 [CEE_STIND_I1] = 2, /* CEE_STIND_I1 */
	 [CEE_STIND_I2] = 2, /* CEE_STIND_I2 */
	 [OP_MEMCPY] = 2, /* OP_MEMCPY */
	 [OP_MEMSET] = 1, /* OP_MEMSET */
	 [OP_GROUP] = 2, /* OP_GROUP */
	 [OP_LDELEMA2D] = 2, /* OP_LDELEMA2D */
	 [CEE_CONV_I1] = 1, /* CEE_CONV_I1 */
	 [CEE_CONV_I2] = 1, /* CEE_CONV_I2 */
	 [CEE_CONV_I4] = 1, /* CEE_CONV_I4 */
	 [CEE_CONV_U1] = 1, /* CEE_CONV_U1 */
	 [CEE_CONV_U2] = 1, /* CEE_CONV_U2 */
	 [CEE_CONV_U4] = 1, /* CEE_CONV_U4 */
	 [CEE_CONV_U] = 1, /* CEE_CONV_U */
	 [CEE_CONV_I] = 1, /* CEE_CONV_I */
	 [CEE_CONV_OVF_I4] = 1, /* CEE_CONV_OVF_I4 */
	 [CEE_CONV_OVF_U4] = 1, /* CEE_CONV_OVF_U4 */
	 [CEE_CONV_OVF_I4_UN] = 1, /* CEE_CONV_OVF_I4_UN */
	 [CEE_CONV_OVF_I1] = 1, /* CEE_CONV_OVF_I1 */
	 [CEE_CONV_OVF_I1_UN] = 1, /* CEE_CONV_OVF_I1_UN */
	 [CEE_CONV_OVF_U1] = 1, /* CEE_CONV_OVF_U1 */
	 [CEE_CONV_OVF_U1_UN] = 1, /* CEE_CONV_OVF_U1_UN */
	 [CEE_CONV_OVF_I2] = 1, /* CEE_CONV_OVF_I2 */
	 [CEE_CONV_OVF_I2_UN] = 1, /* CEE_CONV_OVF_I2_UN */
	 [CEE_CONV_OVF_U2] = 1, /* CEE_CONV_OVF_U2 */
	 [CEE_CONV_OVF_U2_UN] = 1, /* CEE_CONV_OVF_U2_UN */
	 [OP_CEQ] = 1, /* OP_CEQ */
	 [OP_CLT] = 1, /* OP_CLT */
	 [OP_CLT_UN] = 1, /* OP_CLT_UN */
	 [OP_CGT] = 1, /* OP_CGT */
	 [OP_CGT_UN] = 1, /* OP_CGT_UN */
	 [OP_LABEL] = 0, /* OP_LABEL */
	 [OP_NOP] = 0, /* OP_NOP */
	 [OP_BREAK] = 0, /* OP_BREAK */
	 [OP_TLS_GET] = 0, /* OP_TLS_GET */
	 [OP_SWITCH] = 1, /* OP_SWITCH */
	 [OP_BR] = 0, /* OP_BR */
	 [OP_ARGLIST] = 1, /* OP_ARGLIST */
	 [OP_CALL] = 0, /* OP_CALL */
	 [OP_CALLVIRT] = 1, /* OP_CALLVIRT */
	 [OP_VOIDCALLVIRT] = 1, /* OP_VOIDCALLVIRT */
	 [OP_FCALLVIRT] = 1, /* OP_FCALLVIRT */
	 [OP_VCALLVIRT] = 2, /* OP_VCALLVIRT */
	 [OP_CALLVIRT_IMT] = 2, /* OP_CALLVIRT_IMT */
	 [OP_VOIDCALLVIRT_IMT] = 2, /* OP_VOIDCALLVIRT_IMT */
	 [OP_FCALLVIRT_IMT] = 2, /* OP_FCALLVIRT_IMT */
	 [OP_VCALLVIRT_IMT] = 2, /* OP_VCALLVIRT_IMT */
	 [OP_CALL_REG] = 1, /* OP_CALL_REG */
	 [OP_VOIDCALL] = 0, /* OP_VOIDCALL */
	 [OP_VOIDCALL_REG] = 1, /* OP_VOIDCALL_REG */
	 [OP_FCALL] = 0, /* OP_FCALL */
	 [OP_FCALL_REG] = 1, /* OP_FCALL_REG */
	 [OP_VCALL] = 1, /* OP_VCALL */
	 [OP_VCALL_REG] = 2, /* OP_VCALL_REG */
	 [OP_TRAMPCALL_VTABLE] = 1, /* OP_TRAMPCALL_VTABLE */
	 [OP_CALL_RGCTX] = 1, /* OP_CALL_RGCTX */
	 [OP_FCALL_RGCTX] = 1, /* OP_FCALL_RGCTX */
	 [OP_VOIDCALL_RGCTX] = 1, /* OP_VOIDCALL_RGCTX */
	 [OP_VCALL_RGCTX] = 2, /* OP_VCALL_RGCTX */
	 [OP_CALL_REG_RGCTX] = 2, /* OP_CALL_REG_RGCTX */
	 [OP_FCALL_REG_RGCTX] = 2, /* OP_FCALL_REG_RGCTX */
	 [OP_VOIDCALL_REG_RGCTX] = 2, /* OP_VOIDCALL_REG_RGCTX */
	 [OP_VCALL_REG_RGCTX] = 2, /* OP_VCALL_REG_RGCTX */
	 [OP_SAVE_LMF] = 0, /* OP_SAVE_LMF */
	 [OP_RESTORE_LMF] = 0, /* OP_RESTORE_LMF */
	 [OP_I8CONST] = 0, /* OP_I8CONST */
	 [CEE_POP] = 1, /* CEE_POP */
	 [OP_JMP] = 0, /* OP_JMP */
	 [OP_THROW] = 1, /* OP_THROW */
	 [OP_RETHROW] = 1, /* OP_RETHROW */
	 [OP_THROW_OR_NULL] = 2, /* OP_THROW_OR_NULL */
	 [OP_CALL_HANDLER] = 0, /* OP_CALL_HANDLER */
	 [OP_START_HANDLER] = 0, /* OP_START_HANDLER */
	 [OP_ENDFINALLY] = 0, /* OP_ENDFINALLY */
	 [OP_ENDFILTER] = 1, /* OP_ENDFILTER */
	 [OP_CHECK_THIS] = 1, /* OP_CHECK_THIS */
	 [OP_CHECK_THIS_PASSTHROUGH] = 1, /* OP_CHECK_THIS_PASSTHROUGH */
	 [CEE_ISINST] = 1, /* CEE_ISINST */
	 [OP_CISINST] = 1, /* OP_CISINST */
	 [OP_UNBOXCAST] = 1, /* OP_UNBOXCAST */
	 [OP_UNBOXCAST_REG] = 2, /* OP_UNBOXCAST_REG */
	 [CEE_CASTCLASS] = 1, /* CEE_CASTCLASS */
	 [OP_CCASTCLASS] = 1, /* OP_CCASTCLASS */
	 [CEE_STELEM_REF] = 2, /* CEE_STELEM_REF */
	 [CEE_NEWARR] = 1, /* CEE_NEWARR */
	 [OP_LMUL] = 2, /* OP_LMUL */
	 [OP_LMUL_OVF] = 2, /* OP_LMUL_OVF */
	 [OP_LMUL_OVF_UN] = 2, /* OP_LMUL_OVF_UN */
	 [OP_LDIV] = 2, /* OP_LDIV */
	 [OP_LDIV_UN] = 2, /* OP_LDIV_UN */
	 [OP_LREM] = 2, /* OP_LREM */
	 [OP_LREM_UN] = 2, /* OP_LREM_UN */
	 [OP_LSHL] = 2, /* OP_LSHL */
	 [OP_LSHR] = 2, /* OP_LSHR */
	 [OP_LSHR_UN] = 2, /* OP_LSHR_UN */
	 [OP_GETCHR] = 2, /* OP_GETCHR */
	 [OP_STR_CHAR_ADDR] = 2, /* OP_STR_CHAR_ADDR */
	 [OP_STRLEN] = 1, /* OP_STRLEN */
	 [OP_GETTYPE] = 1, /* OP_GETTYPE */
	 [OP_GETHASHCODE] = 1, /* OP_GETHASHCODE */
	 [CEE_LDLEN] = 1, /* CEE_LDLEN */
	 [OP_ARRAY_RANK] = 1, /* OP_ARRAY_RANK */
	 [OP_CHECK_ARRAY_TYPE] = 1, /* OP_CHECK_ARRAY_TYPE */
	 [CEE_LDELEMA] = 2, /* CEE_LDELEMA */
	 [CEE_REFANYVAL] = 1, /* CEE_REFANYVAL */
	 [OP_REFANYVAL_REG] = 2, /* OP_REFANYVAL_REG */
	 [OP_REFANYTYPE] = 1, /* OP_REFANYTYPE */
	 [CEE_MKREFANY] = 2, /* CEE_MKREFANY */
	 [OP_MKREFANY_REGS] = 2, /* OP_MKREFANY_REGS */
	 [CEE_LDIND_R4] = 1, /* CEE_LDIND_R4 */
	 [CEE_LDIND_R8] = 1, /* CEE_LDIND_R8 */
	 [CEE_STIND_R4] = 2, /* CEE_STIND_R4 */
	 [CEE_STIND_R8] = 2, /* CEE_STIND_R8 */
	 [OP_R4CONST] = 0, /* OP_R4CONST */
	 [OP_R8CONST] = 0, /* OP_R8CONST */
	 [OP_FADD] = 2, /* OP_FADD */
	 [OP_FSUB] = 2, /* OP_FSUB */
	 [OP_FMUL] = 2, /* OP_FMUL */
	 [OP_FDIV] = 2, /* OP_FDIV */
	 [OP_FREM] = 2, /* OP_FREM */
	 [OP_FNEG] = 1, /* OP_FNEG */
	 [OP_CKFINITE] = 1, /* OP_CKFINITE */
	 [OP_SIN] = 1, /* OP_SIN */
	 [OP_COS] = 1, /* OP_COS */
	 [OP_ABS] = 1, /* OP_ABS */
	 [OP_TAN] = 1, /* OP_TAN */
	 [OP_ATAN] = 1, /* OP_ATAN */
	 [OP_SQRT] = 1, /* OP_SQRT */
	 [OP_FCONV_TO_I4] = 1, /* OP_FCONV_TO_I4 */
	 [OP_FCONV_TO_U4] = 1, /* OP_FCONV_TO_U4 */
	 [OP_FCONV_TO_OVF_I4] = 1, /* OP_FCONV_TO_OVF_I4 */
	 [OP_FCONV_TO_OVF_U4] = 1, /* OP_FCONV_TO_OVF_U4 */
	 [OP_FCONV_TO_OVF_I8] = 1, /* OP_FCONV_TO_OVF_I8 */
	 [OP_FCONV_TO_OVF_U8] = 1, /* OP_FCONV_TO_OVF_U8 */
	 [OP_FCONV_TO_OVF_I] = 1, /* OP_FCONV_TO_OVF_I */
	 [OP_FCONV_TO_OVF_U] = 1, /* OP_FCONV_TO_OVF_U */
	 [OP_FCONV_TO_I] = 1, /* OP_FCONV_TO_I */
	 [OP_FCONV_TO_U] = 1, /* OP_FCONV_TO_U */
	 [OP_FCONV_TO_I2] = 1, /* OP_FCONV_TO_I2 */
	 [OP_FCONV_TO_U2] = 1, /* OP_FCONV_TO_U2 */
	 [OP_FCONV_TO_I1] = 1, /* OP_FCONV_TO_I1 */
	 [OP_FCONV_TO_U1] = 1, /* OP_FCONV_TO_U1 */
	 [OP_FCONV_TO_R8] = 1, /* OP_FCONV_TO_R8 */
	 [OP_FCONV_TO_R4] = 1, /* OP_FCONV_TO_R4 */
	 [CEE_CONV_R4] = 1, /* CEE_CONV_R4 */
	 [CEE_CONV_R8] = 1, /* CEE_CONV_R8 */
	 [CEE_CONV_R_UN] = 1, /* CEE_CONV_R_UN */
	 [OP_COMPARE] = 2, /* OP_COMPARE */
	 [CEE_LDIND_I8] = 1, /* CEE_LDIND_I8 */
	 [CEE_STIND_I8] = 2, /* CEE_STIND_I8 */
	 [OP_LADD] = 2, /* OP_LADD */
	 [OP_LSUB] = 2, /* OP_LSUB */
	 [OP_LNEG] = 1, /* OP_LNEG */
	 [OP_LNOT] = 1, /* OP_LNOT */
	 [OP_LAND] = 2, /* OP_LAND */
	 [OP_LOR] = 2, /* OP_LOR */
	 [OP_LXOR] = 2, /* OP_LXOR */
	 [OP_LADD_OVF] = 2, /* OP_LADD_OVF */
	 [OP_LADD_OVF_UN] = 2, /* OP_LADD_OVF_UN */
	 [OP_LSUB_OVF] = 2, /* OP_LSUB_OVF */
	 [OP_LSUB_OVF_UN] = 2, /* OP_LSUB_OVF_UN */
	 [OP_LONG_SHRUN_32] = 1, /* OP_LONG_SHRUN_32 */
	 [CEE_CONV_I8] = 1, /* CEE_CONV_I8 */
	 [CEE_CONV_U8] = 1, /* CEE_CONV_U8 */
	 [OP_FCONV_TO_I8] = 1, /* OP_FCONV_TO_I8 */
	 [OP_FCONV_TO_U8] = 1, /* OP_FCONV_TO_U8 */
	 [OP_LCONV_TO_I4] = 1, /* OP_LCONV_TO_I4 */
	 [OP_LCONV_TO_U4] = 1, /* OP_LCONV_TO_U4 */
	 [OP_LCONV_TO_U8] = 1, /* OP_LCONV_TO_U8 */
	 [OP_LCONV_TO_I8] = 1, /* OP_LCONV_TO_I8 */
	 [OP_LCONV_TO_U] = 1, /* OP_LCONV_TO_U */
	 [OP_LCONV_TO_I] = 1, /* OP_LCONV_TO_I */
	 [OP_LCONV_TO_I1] = 1, /* OP_LCONV_TO_I1 */
	 [OP_LCONV_TO_U1] = 1, /* OP_LCONV_TO_U1 */
	 [OP_LCONV_TO_I2] = 1, /* OP_LCONV_TO_I2 */
	 [OP_LCONV_TO_U2] = 1, /* OP_LCONV_TO_U2 */
	 [OP_LCONV_TO_OVF_I1] = 1, /* OP_LCONV_TO_OVF_I1 */
	 [OP_LCONV_TO_OVF_I1_UN] = 1, /* OP_LCONV_TO_OVF_I1_UN */
	 [OP_LCONV_TO_OVF_U1] = 1, /* OP_LCONV_TO_OVF_U1 */
	 [OP_LCONV_TO_OVF_U1_UN] = 1, /* OP_LCONV_TO_OVF_U1_UN */
	 [OP_LCONV_TO_OVF_I2] = 1, /* OP_LCONV_TO_OVF_I2 */
	 [OP_LCONV_TO_OVF_I2_UN] = 1, /* OP_LCONV_TO_OVF_I2_UN */
	 [OP_LCONV_TO_OVF_U2] = 1, /* OP_LCONV_TO_OVF_U2 */
	 [OP_LCONV_TO_OVF_U2_UN] = 1, /* OP_LCONV_TO_OVF_U2_UN */
	 [OP_LCONV_TO_OVF_I4] = 1, /* OP_LCONV_TO_OVF_I4 */
	 [OP_LCONV_TO_OVF_I4_UN] = 1, /* OP_LCONV_TO_OVF_I4_UN */
	 [OP_LCONV_TO_OVF_U4] = 1, /* OP_LCONV_TO_OVF_U4 */
	 [OP_LCONV_TO_OVF_U4_UN] = 1, /* OP_LCONV_TO_OVF_U4_UN */
	 [OP_LCONV_TO_OVF_I_UN] = 1, /* OP_LCONV_TO_OVF_I_UN */
	 [OP_LCONV_TO_OVF_I8_UN] = 1, /* OP_LCONV_TO_OVF_I8_UN */
	 [OP_LCONV_TO_OVF_U8] = 1, /* OP_LCONV_TO_OVF_U8 */
	 [OP_LCONV_TO_OVF_U] = 1, /* OP_LCONV_TO_OVF_U */
	 [OP_LCONV_TO_OVF_I8] = 1, /* OP_LCONV_TO_OVF_I8 */
	 [OP_LCONV_TO_OVF_U_UN] = 1, /* OP_LCONV_TO_OVF_U_UN */
	 [OP_LCONV_TO_OVF_U8_UN] = 1, /* OP_LCONV_TO_OVF_U8_UN */
	 [OP_LCONV_TO_OVF_I] = 1, /* OP_LCONV_TO_OVF_I */
	 [OP_LCONV_TO_R_UN] = 1, /* OP_LCONV_TO_R_UN */
	 [CEE_CONV_OVF_I8] = 1, /* CEE_CONV_OVF_I8 */
	 [CEE_CONV_OVF_I] = 1, /* CEE_CONV_OVF_I */
	 [CEE_CONV_OVF_I8_UN] = 1, /* CEE_CONV_OVF_I8_UN */
	 [CEE_CONV_OVF_U8_UN] = 1, /* CEE_CONV_OVF_U8_UN */
	 [CEE_CONV_OVF_I_UN] = 1, /* CEE_CONV_OVF_I_UN */
	 [CEE_CONV_OVF_U_UN] = 1, /* CEE_CONV_OVF_U_UN */
	 [CEE_CONV_OVF_U8] = 1, /* CEE_CONV_OVF_U8 */
	 [CEE_CONV_OVF_U] = 1, /* CEE_CONV_OVF_U */
	 [OP_LCALLVIRT] = 1, /* OP_LCALLVIRT */
	 [OP_LCALLVIRT_IMT] = 2, /* OP_LCALLVIRT_IMT */
	 [OP_LCALL] = 0, /* OP_LCALL */
	 [OP_LCALL_RGCTX] = 1, /* OP_LCALL_RGCTX */
	 [OP_LCALL_REG] = 1, /* OP_LCALL_REG */
	 [OP_LCALL_REG_RGCTX] = 2, /* OP_LCALL_REG_RGCTX */
	 [OP_LCOMPARE] = 2, /* OP_LCOMPARE */
	 [CEE_BNE_UN] = 1, /* CEE_BNE_UN */
	 [CEE_BEQ] = 1, /* CEE_BEQ */
	 [CEE_BLT] = 1, /* CEE_BLT */
	 [CEE_BLT_UN] = 1, /* CEE_BLT_UN */
	 [CEE_BGT] = 1, /* CEE_BGT */
	 [CEE_BGT_UN] = 1, /* CEE_BGT_UN */
	 [CEE_BGE] = 1, /* CEE_BGE */
	 [CEE_BGE_UN] = 1, /* CEE_BGE_UN */
	 [CEE_BLE] = 1, /* CEE_BLE */
	 [CEE_BLE_UN] = 1, /* CEE_BLE_UN */
	 [CEE_AND] = 2, /* CEE_AND */
	 [CEE_OR] = 2, /* CEE_OR */
	 [CEE_XOR] = 2, /* CEE_XOR */
	 [CEE_NEG] = 1, /* CEE_NEG */
	 [CEE_NOT] = 1, /* CEE_NOT */
	 [CEE_ADD_OVF] = 2, /* CEE_ADD_OVF */
	 [CEE_ADD_OVF_UN] = 2, /* CEE_ADD_OVF_UN */
	 [CEE_SUB] = 2, /* CEE_SUB */
	 [CEE_SUB_OVF] = 2, /* CEE_SUB_OVF */
	 [CEE_SUB_OVF_UN] = 2, /* CEE_SUB_OVF_UN */
	 [CEE_SHL] = 2, /* CEE_SHL */
	 [CEE_SHR] = 2, /* CEE_SHR */
	 [CEE_SHR_UN] = 2, /* CEE_SHR_UN */
	 [CEE_MUL] = 2, /* CEE_MUL */
	 [CEE_MUL_OVF] = 2, /* CEE_MUL_OVF */
	 [CEE_MUL_OVF_UN] = 2, /* CEE_MUL_OVF_UN */
	 [CEE_DIV] = 2, /* CEE_DIV */
	 [CEE_DIV_UN] = 2, /* CEE_DIV_UN */
	 [CEE_REM] = 2, /* CEE_REM */
	 [CEE_REM_UN] = 2, /* CEE_REM_UN */
	 [OP_IMUL_IMM] = 1, /* OP_IMUL_IMM */
	 [OP_LMUL_IMM] = 1, /* OP_LMUL_IMM */
	 [OP_LCONV_TO_R8] = 1, /* OP_LCONV_TO_R8 */
	 [OP_LCONV_TO_R4] = 1, /* OP_LCONV_TO_R4 */
	 [OP_LOCALLOC] = 1, /* OP_LOCALLOC */
	 [OP_SETRET] = 1, /* OP_SETRET */
	 [OP_OUTARG] = 1, /* OP_OUTARG */
	 [OP_OUTARG_REG] = 1, /* OP_OUTARG_REG */
	 [OP_OUTARG_R4] = 1, /* OP_OUTARG_R4 */
	 [OP_OUTARG_R8] = 1, /* OP_OUTARG_R8 */
	 [OP_AMD64_OUTARG_XMMREG_R4] = 1, /* OP_AMD64_OUTARG_XMMREG_R4 */
	 [OP_AMD64_OUTARG_XMMREG_R8] = 1, /* OP_AMD64_OUTARG_XMMREG_R8 */
	 [OP_OUTARG_VT] = 1, /* OP_OUTARG_VT */
	 [OP_AMD64_OUTARG_ALIGN_STACK] = 0, /* OP_AMD64_OUTARG_ALIGN_STACK */
	 [OP_AMD64_SAVE_SP_TO_LMF] = 0, /* OP_AMD64_SAVE_SP_TO_LMF */
	 [OP_INARG_VT] = 1, /* OP_INARG_VT */
	 [OP_VTARG_ADDR] = 1, /* OP_VTARG_ADDR */
	 [OP_IMIN] = 2, /* OP_IMIN */
	 [OP_IMIN_UN] = 2, /* OP_IMIN_UN */
	 [OP_IMAX] = 2, /* OP_IMAX */
	 [OP_IMAX_UN] = 2, /* OP_IMAX_UN */
	 [OP_LMIN] = 2, /* OP_LMIN */
	 [OP_LMIN_UN] = 2, /* OP_LMIN_UN */
	 [OP_LMAX] = 2, /* OP_LMAX */
	 [OP_LMAX_UN] = 2, /* OP_LMAX_UN */
	 [OP_ATOMIC_ADD_NEW_I4] = 2, /* OP_ATOMIC_ADD_NEW_I4 */
	 [OP_ATOMIC_ADD_NEW_I8] = 2, /* OP_ATOMIC_ADD_NEW_I8 */
	 [OP_ATOMIC_ADD_I4] = 2, /* OP_ATOMIC_ADD_I4 */
	 [OP_ATOMIC_ADD_I8] = 2, /* OP_ATOMIC_ADD_I8 */
	 [OP_ATOMIC_EXCHANGE_I4] = 2, /* OP_ATOMIC_EXCHANGE_I4 */
	 [OP_ATOMIC_EXCHANGE_I8] = 2, /* OP_ATOMIC_EXCHANGE_I8 */
	 [OP_ATOMIC_CAS_IMM_I4] = 2, /* OP_ATOMIC_CAS_IMM_I4 */
};

void
mono_burg_init (void) {
}

#else
guint8 mono_burg_arity [MBMAX_OPCODES];
void
mono_burg_init (void)
{
	mono_burg_arity [OP_REGVAR] = 0; /* OP_REGVAR */
	mono_burg_arity [CEE_LDIND_I] = 1; /* CEE_LDIND_I */
	mono_burg_arity [OP_REGOFFSET] = 0; /* OP_REGOFFSET */
	mono_burg_arity [OP_LDADDR] = 1; /* OP_LDADDR */
	mono_burg_arity [CEE_LDOBJ] = 1; /* CEE_LDOBJ */
	mono_burg_arity [OP_ICONST] = 0; /* OP_ICONST */
	mono_burg_arity [CEE_ADD] = 2; /* CEE_ADD */
	mono_burg_arity [CEE_LDIND_REF] = 1; /* CEE_LDIND_REF */
	mono_burg_arity [OP_AOTCONST] = 0; /* OP_AOTCONST */
	mono_burg_arity [OP_LOAD_GOTADDR] = 0; /* OP_LOAD_GOTADDR */
	mono_burg_arity [CEE_STIND_I] = 2; /* CEE_STIND_I */
	mono_burg_arity [OP_PATCH_INFO] = 0; /* OP_PATCH_INFO */
	mono_burg_arity [OP_GOT_ENTRY] = 2; /* OP_GOT_ENTRY */
	mono_burg_arity [OP_DUMMY_USE] = 1; /* OP_DUMMY_USE */
	mono_burg_arity [OP_DUMMY_STORE] = 0; /* OP_DUMMY_STORE */
	mono_burg_arity [OP_NOT_REACHED] = 0; /* OP_NOT_REACHED */
	mono_burg_arity [OP_MEMORY_BARRIER] = 0; /* OP_MEMORY_BARRIER */
	mono_burg_arity [CEE_LDIND_I1] = 1; /* CEE_LDIND_I1 */
	mono_burg_arity [CEE_LDIND_U1] = 1; /* CEE_LDIND_U1 */
	mono_burg_arity [CEE_LDIND_I2] = 1; /* CEE_LDIND_I2 */
	mono_burg_arity [CEE_LDIND_U2] = 1; /* CEE_LDIND_U2 */
	mono_burg_arity [CEE_LDIND_I4] = 1; /* CEE_LDIND_I4 */
	mono_burg_arity [CEE_LDIND_U4] = 1; /* CEE_LDIND_U4 */
	mono_burg_arity [OP_OBJADDR] = 1; /* OP_OBJADDR */
	mono_burg_arity [OP_VTADDR] = 1; /* OP_VTADDR */
	mono_burg_arity [CEE_STIND_REF] = 2; /* CEE_STIND_REF */
	mono_burg_arity [CEE_STIND_I4] = 2; /* CEE_STIND_I4 */
	mono_burg_arity [CEE_STIND_I1] = 2; /* CEE_STIND_I1 */
	mono_burg_arity [CEE_STIND_I2] = 2; /* CEE_STIND_I2 */
	mono_burg_arity [OP_MEMCPY] = 2; /* OP_MEMCPY */
	mono_burg_arity [OP_MEMSET] = 1; /* OP_MEMSET */
	mono_burg_arity [OP_GROUP] = 2; /* OP_GROUP */
	mono_burg_arity [OP_LDELEMA2D] = 2; /* OP_LDELEMA2D */
	mono_burg_arity [CEE_CONV_I1] = 1; /* CEE_CONV_I1 */
	mono_burg_arity [CEE_CONV_I2] = 1; /* CEE_CONV_I2 */
	mono_burg_arity [CEE_CONV_I4] = 1; /* CEE_CONV_I4 */
	mono_burg_arity [CEE_CONV_U1] = 1; /* CEE_CONV_U1 */
	mono_burg_arity [CEE_CONV_U2] = 1; /* CEE_CONV_U2 */
	mono_burg_arity [CEE_CONV_U4] = 1; /* CEE_CONV_U4 */
	mono_burg_arity [CEE_CONV_U] = 1; /* CEE_CONV_U */
	mono_burg_arity [CEE_CONV_I] = 1; /* CEE_CONV_I */
	mono_burg_arity [CEE_CONV_OVF_I4] = 1; /* CEE_CONV_OVF_I4 */
	mono_burg_arity [CEE_CONV_OVF_U4] = 1; /* CEE_CONV_OVF_U4 */
	mono_burg_arity [CEE_CONV_OVF_I4_UN] = 1; /* CEE_CONV_OVF_I4_UN */
	mono_burg_arity [CEE_CONV_OVF_I1] = 1; /* CEE_CONV_OVF_I1 */
	mono_burg_arity [CEE_CONV_OVF_I1_UN] = 1; /* CEE_CONV_OVF_I1_UN */
	mono_burg_arity [CEE_CONV_OVF_U1] = 1; /* CEE_CONV_OVF_U1 */
	mono_burg_arity [CEE_CONV_OVF_U1_UN] = 1; /* CEE_CONV_OVF_U1_UN */
	mono_burg_arity [CEE_CONV_OVF_I2] = 1; /* CEE_CONV_OVF_I2 */
	mono_burg_arity [CEE_CONV_OVF_I2_UN] = 1; /* CEE_CONV_OVF_I2_UN */
	mono_burg_arity [CEE_CONV_OVF_U2] = 1; /* CEE_CONV_OVF_U2 */
	mono_burg_arity [CEE_CONV_OVF_U2_UN] = 1; /* CEE_CONV_OVF_U2_UN */
	mono_burg_arity [OP_CEQ] = 1; /* OP_CEQ */
	mono_burg_arity [OP_CLT] = 1; /* OP_CLT */
	mono_burg_arity [OP_CLT_UN] = 1; /* OP_CLT_UN */
	mono_burg_arity [OP_CGT] = 1; /* OP_CGT */
	mono_burg_arity [OP_CGT_UN] = 1; /* OP_CGT_UN */
	mono_burg_arity [OP_LABEL] = 0; /* OP_LABEL */
	mono_burg_arity [OP_NOP] = 0; /* OP_NOP */
	mono_burg_arity [OP_BREAK] = 0; /* OP_BREAK */
	mono_burg_arity [OP_TLS_GET] = 0; /* OP_TLS_GET */
	mono_burg_arity [OP_SWITCH] = 1; /* OP_SWITCH */
	mono_burg_arity [OP_BR] = 0; /* OP_BR */
	mono_burg_arity [OP_ARGLIST] = 1; /* OP_ARGLIST */
	mono_burg_arity [OP_CALL] = 0; /* OP_CALL */
	mono_burg_arity [OP_CALLVIRT] = 1; /* OP_CALLVIRT */
	mono_burg_arity [OP_VOIDCALLVIRT] = 1; /* OP_VOIDCALLVIRT */
	mono_burg_arity [OP_FCALLVIRT] = 1; /* OP_FCALLVIRT */
	mono_burg_arity [OP_VCALLVIRT] = 2; /* OP_VCALLVIRT */
	mono_burg_arity [OP_CALLVIRT_IMT] = 2; /* OP_CALLVIRT_IMT */
	mono_burg_arity [OP_VOIDCALLVIRT_IMT] = 2; /* OP_VOIDCALLVIRT_IMT */
	mono_burg_arity [OP_FCALLVIRT_IMT] = 2; /* OP_FCALLVIRT_IMT */
	mono_burg_arity [OP_VCALLVIRT_IMT] = 2; /* OP_VCALLVIRT_IMT */
	mono_burg_arity [OP_CALL_REG] = 1; /* OP_CALL_REG */
	mono_burg_arity [OP_VOIDCALL] = 0; /* OP_VOIDCALL */
	mono_burg_arity [OP_VOIDCALL_REG] = 1; /* OP_VOIDCALL_REG */
	mono_burg_arity [OP_FCALL] = 0; /* OP_FCALL */
	mono_burg_arity [OP_FCALL_REG] = 1; /* OP_FCALL_REG */
	mono_burg_arity [OP_VCALL] = 1; /* OP_VCALL */
	mono_burg_arity [OP_VCALL_REG] = 2; /* OP_VCALL_REG */
	mono_burg_arity [OP_TRAMPCALL_VTABLE] = 1; /* OP_TRAMPCALL_VTABLE */
	mono_burg_arity [OP_CALL_RGCTX] = 1; /* OP_CALL_RGCTX */
	mono_burg_arity [OP_FCALL_RGCTX] = 1; /* OP_FCALL_RGCTX */
	mono_burg_arity [OP_VOIDCALL_RGCTX] = 1; /* OP_VOIDCALL_RGCTX */
	mono_burg_arity [OP_VCALL_RGCTX] = 2; /* OP_VCALL_RGCTX */
	mono_burg_arity [OP_CALL_REG_RGCTX] = 2; /* OP_CALL_REG_RGCTX */
	mono_burg_arity [OP_FCALL_REG_RGCTX] = 2; /* OP_FCALL_REG_RGCTX */
	mono_burg_arity [OP_VOIDCALL_REG_RGCTX] = 2; /* OP_VOIDCALL_REG_RGCTX */
	mono_burg_arity [OP_VCALL_REG_RGCTX] = 2; /* OP_VCALL_REG_RGCTX */
	mono_burg_arity [OP_SAVE_LMF] = 0; /* OP_SAVE_LMF */
	mono_burg_arity [OP_RESTORE_LMF] = 0; /* OP_RESTORE_LMF */
	mono_burg_arity [OP_I8CONST] = 0; /* OP_I8CONST */
	mono_burg_arity [CEE_POP] = 1; /* CEE_POP */
	mono_burg_arity [OP_JMP] = 0; /* OP_JMP */
	mono_burg_arity [OP_THROW] = 1; /* OP_THROW */
	mono_burg_arity [OP_RETHROW] = 1; /* OP_RETHROW */
	mono_burg_arity [OP_THROW_OR_NULL] = 2; /* OP_THROW_OR_NULL */
	mono_burg_arity [OP_CALL_HANDLER] = 0; /* OP_CALL_HANDLER */
	mono_burg_arity [OP_START_HANDLER] = 0; /* OP_START_HANDLER */
	mono_burg_arity [OP_ENDFINALLY] = 0; /* OP_ENDFINALLY */
	mono_burg_arity [OP_ENDFILTER] = 1; /* OP_ENDFILTER */
	mono_burg_arity [OP_CHECK_THIS] = 1; /* OP_CHECK_THIS */
	mono_burg_arity [OP_CHECK_THIS_PASSTHROUGH] = 1; /* OP_CHECK_THIS_PASSTHROUGH */
	mono_burg_arity [CEE_ISINST] = 1; /* CEE_ISINST */
	mono_burg_arity [OP_CISINST] = 1; /* OP_CISINST */
	mono_burg_arity [OP_UNBOXCAST] = 1; /* OP_UNBOXCAST */
	mono_burg_arity [OP_UNBOXCAST_REG] = 2; /* OP_UNBOXCAST_REG */
	mono_burg_arity [CEE_CASTCLASS] = 1; /* CEE_CASTCLASS */
	mono_burg_arity [OP_CCASTCLASS] = 1; /* OP_CCASTCLASS */
	mono_burg_arity [CEE_STELEM_REF] = 2; /* CEE_STELEM_REF */
	mono_burg_arity [CEE_NEWARR] = 1; /* CEE_NEWARR */
	mono_burg_arity [OP_LMUL] = 2; /* OP_LMUL */
	mono_burg_arity [OP_LMUL_OVF] = 2; /* OP_LMUL_OVF */
	mono_burg_arity [OP_LMUL_OVF_UN] = 2; /* OP_LMUL_OVF_UN */
	mono_burg_arity [OP_LDIV] = 2; /* OP_LDIV */
	mono_burg_arity [OP_LDIV_UN] = 2; /* OP_LDIV_UN */
	mono_burg_arity [OP_LREM] = 2; /* OP_LREM */
	mono_burg_arity [OP_LREM_UN] = 2; /* OP_LREM_UN */
	mono_burg_arity [OP_LSHL] = 2; /* OP_LSHL */
	mono_burg_arity [OP_LSHR] = 2; /* OP_LSHR */
	mono_burg_arity [OP_LSHR_UN] = 2; /* OP_LSHR_UN */
	mono_burg_arity [OP_GETCHR] = 2; /* OP_GETCHR */
	mono_burg_arity [OP_STR_CHAR_ADDR] = 2; /* OP_STR_CHAR_ADDR */
	mono_burg_arity [OP_STRLEN] = 1; /* OP_STRLEN */
	mono_burg_arity [OP_GETTYPE] = 1; /* OP_GETTYPE */
	mono_burg_arity [OP_GETHASHCODE] = 1; /* OP_GETHASHCODE */
	mono_burg_arity [CEE_LDLEN] = 1; /* CEE_LDLEN */
	mono_burg_arity [OP_ARRAY_RANK] = 1; /* OP_ARRAY_RANK */
	mono_burg_arity [OP_CHECK_ARRAY_TYPE] = 1; /* OP_CHECK_ARRAY_TYPE */
	mono_burg_arity [CEE_LDELEMA] = 2; /* CEE_LDELEMA */
	mono_burg_arity [CEE_REFANYVAL] = 1; /* CEE_REFANYVAL */
	mono_burg_arity [OP_REFANYVAL_REG] = 2; /* OP_REFANYVAL_REG */
	mono_burg_arity [OP_REFANYTYPE] = 1; /* OP_REFANYTYPE */
	mono_burg_arity [CEE_MKREFANY] = 2; /* CEE_MKREFANY */
	mono_burg_arity [OP_MKREFANY_REGS] = 2; /* OP_MKREFANY_REGS */
	mono_burg_arity [CEE_LDIND_R4] = 1; /* CEE_LDIND_R4 */
	mono_burg_arity [CEE_LDIND_R8] = 1; /* CEE_LDIND_R8 */
	mono_burg_arity [CEE_STIND_R4] = 2; /* CEE_STIND_R4 */
	mono_burg_arity [CEE_STIND_R8] = 2; /* CEE_STIND_R8 */
	mono_burg_arity [OP_R4CONST] = 0; /* OP_R4CONST */
	mono_burg_arity [OP_R8CONST] = 0; /* OP_R8CONST */
	mono_burg_arity [OP_FADD] = 2; /* OP_FADD */
	mono_burg_arity [OP_FSUB] = 2; /* OP_FSUB */
	mono_burg_arity [OP_FMUL] = 2; /* OP_FMUL */
	mono_burg_arity [OP_FDIV] = 2; /* OP_FDIV */
	mono_burg_arity [OP_FREM] = 2; /* OP_FREM */
	mono_burg_arity [OP_FNEG] = 1; /* OP_FNEG */
	mono_burg_arity [OP_CKFINITE] = 1; /* OP_CKFINITE */
	mono_burg_arity [OP_SIN] = 1; /* OP_SIN */
	mono_burg_arity [OP_COS] = 1; /* OP_COS */
	mono_burg_arity [OP_ABS] = 1; /* OP_ABS */
	mono_burg_arity [OP_TAN] = 1; /* OP_TAN */
	mono_burg_arity [OP_ATAN] = 1; /* OP_ATAN */
	mono_burg_arity [OP_SQRT] = 1; /* OP_SQRT */
	mono_burg_arity [OP_FCONV_TO_I4] = 1; /* OP_FCONV_TO_I4 */
	mono_burg_arity [OP_FCONV_TO_U4] = 1; /* OP_FCONV_TO_U4 */
	mono_burg_arity [OP_FCONV_TO_OVF_I4] = 1; /* OP_FCONV_TO_OVF_I4 */
	mono_burg_arity [OP_FCONV_TO_OVF_U4] = 1; /* OP_FCONV_TO_OVF_U4 */
	mono_burg_arity [OP_FCONV_TO_OVF_I8] = 1; /* OP_FCONV_TO_OVF_I8 */
	mono_burg_arity [OP_FCONV_TO_OVF_U8] = 1; /* OP_FCONV_TO_OVF_U8 */
	mono_burg_arity [OP_FCONV_TO_OVF_I] = 1; /* OP_FCONV_TO_OVF_I */
	mono_burg_arity [OP_FCONV_TO_OVF_U] = 1; /* OP_FCONV_TO_OVF_U */
	mono_burg_arity [OP_FCONV_TO_I] = 1; /* OP_FCONV_TO_I */
	mono_burg_arity [OP_FCONV_TO_U] = 1; /* OP_FCONV_TO_U */
	mono_burg_arity [OP_FCONV_TO_I2] = 1; /* OP_FCONV_TO_I2 */
	mono_burg_arity [OP_FCONV_TO_U2] = 1; /* OP_FCONV_TO_U2 */
	mono_burg_arity [OP_FCONV_TO_I1] = 1; /* OP_FCONV_TO_I1 */
	mono_burg_arity [OP_FCONV_TO_U1] = 1; /* OP_FCONV_TO_U1 */
	mono_burg_arity [OP_FCONV_TO_R8] = 1; /* OP_FCONV_TO_R8 */
	mono_burg_arity [OP_FCONV_TO_R4] = 1; /* OP_FCONV_TO_R4 */
	mono_burg_arity [CEE_CONV_R4] = 1; /* CEE_CONV_R4 */
	mono_burg_arity [CEE_CONV_R8] = 1; /* CEE_CONV_R8 */
	mono_burg_arity [CEE_CONV_R_UN] = 1; /* CEE_CONV_R_UN */
	mono_burg_arity [OP_COMPARE] = 2; /* OP_COMPARE */
	mono_burg_arity [CEE_LDIND_I8] = 1; /* CEE_LDIND_I8 */
	mono_burg_arity [CEE_STIND_I8] = 2; /* CEE_STIND_I8 */
	mono_burg_arity [OP_LADD] = 2; /* OP_LADD */
	mono_burg_arity [OP_LSUB] = 2; /* OP_LSUB */
	mono_burg_arity [OP_LNEG] = 1; /* OP_LNEG */
	mono_burg_arity [OP_LNOT] = 1; /* OP_LNOT */
	mono_burg_arity [OP_LAND] = 2; /* OP_LAND */
	mono_burg_arity [OP_LOR] = 2; /* OP_LOR */
	mono_burg_arity [OP_LXOR] = 2; /* OP_LXOR */
	mono_burg_arity [OP_LADD_OVF] = 2; /* OP_LADD_OVF */
	mono_burg_arity [OP_LADD_OVF_UN] = 2; /* OP_LADD_OVF_UN */
	mono_burg_arity [OP_LSUB_OVF] = 2; /* OP_LSUB_OVF */
	mono_burg_arity [OP_LSUB_OVF_UN] = 2; /* OP_LSUB_OVF_UN */
	mono_burg_arity [OP_LONG_SHRUN_32] = 1; /* OP_LONG_SHRUN_32 */
	mono_burg_arity [CEE_CONV_I8] = 1; /* CEE_CONV_I8 */
	mono_burg_arity [CEE_CONV_U8] = 1; /* CEE_CONV_U8 */
	mono_burg_arity [OP_FCONV_TO_I8] = 1; /* OP_FCONV_TO_I8 */
	mono_burg_arity [OP_FCONV_TO_U8] = 1; /* OP_FCONV_TO_U8 */
	mono_burg_arity [OP_LCONV_TO_I4] = 1; /* OP_LCONV_TO_I4 */
	mono_burg_arity [OP_LCONV_TO_U4] = 1; /* OP_LCONV_TO_U4 */
	mono_burg_arity [OP_LCONV_TO_U8] = 1; /* OP_LCONV_TO_U8 */
	mono_burg_arity [OP_LCONV_TO_I8] = 1; /* OP_LCONV_TO_I8 */
	mono_burg_arity [OP_LCONV_TO_U] = 1; /* OP_LCONV_TO_U */
	mono_burg_arity [OP_LCONV_TO_I] = 1; /* OP_LCONV_TO_I */
	mono_burg_arity [OP_LCONV_TO_I1] = 1; /* OP_LCONV_TO_I1 */
	mono_burg_arity [OP_LCONV_TO_U1] = 1; /* OP_LCONV_TO_U1 */
	mono_burg_arity [OP_LCONV_TO_I2] = 1; /* OP_LCONV_TO_I2 */
	mono_burg_arity [OP_LCONV_TO_U2] = 1; /* OP_LCONV_TO_U2 */
	mono_burg_arity [OP_LCONV_TO_OVF_I1] = 1; /* OP_LCONV_TO_OVF_I1 */
	mono_burg_arity [OP_LCONV_TO_OVF_I1_UN] = 1; /* OP_LCONV_TO_OVF_I1_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_U1] = 1; /* OP_LCONV_TO_OVF_U1 */
	mono_burg_arity [OP_LCONV_TO_OVF_U1_UN] = 1; /* OP_LCONV_TO_OVF_U1_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_I2] = 1; /* OP_LCONV_TO_OVF_I2 */
	mono_burg_arity [OP_LCONV_TO_OVF_I2_UN] = 1; /* OP_LCONV_TO_OVF_I2_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_U2] = 1; /* OP_LCONV_TO_OVF_U2 */
	mono_burg_arity [OP_LCONV_TO_OVF_U2_UN] = 1; /* OP_LCONV_TO_OVF_U2_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_I4] = 1; /* OP_LCONV_TO_OVF_I4 */
	mono_burg_arity [OP_LCONV_TO_OVF_I4_UN] = 1; /* OP_LCONV_TO_OVF_I4_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_U4] = 1; /* OP_LCONV_TO_OVF_U4 */
	mono_burg_arity [OP_LCONV_TO_OVF_U4_UN] = 1; /* OP_LCONV_TO_OVF_U4_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_I_UN] = 1; /* OP_LCONV_TO_OVF_I_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_I8_UN] = 1; /* OP_LCONV_TO_OVF_I8_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_U8] = 1; /* OP_LCONV_TO_OVF_U8 */
	mono_burg_arity [OP_LCONV_TO_OVF_U] = 1; /* OP_LCONV_TO_OVF_U */
	mono_burg_arity [OP_LCONV_TO_OVF_I8] = 1; /* OP_LCONV_TO_OVF_I8 */
	mono_burg_arity [OP_LCONV_TO_OVF_U_UN] = 1; /* OP_LCONV_TO_OVF_U_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_U8_UN] = 1; /* OP_LCONV_TO_OVF_U8_UN */
	mono_burg_arity [OP_LCONV_TO_OVF_I] = 1; /* OP_LCONV_TO_OVF_I */
	mono_burg_arity [OP_LCONV_TO_R_UN] = 1; /* OP_LCONV_TO_R_UN */
	mono_burg_arity [CEE_CONV_OVF_I8] = 1; /* CEE_CONV_OVF_I8 */
	mono_burg_arity [CEE_CONV_OVF_I] = 1; /* CEE_CONV_OVF_I */
	mono_burg_arity [CEE_CONV_OVF_I8_UN] = 1; /* CEE_CONV_OVF_I8_UN */
	mono_burg_arity [CEE_CONV_OVF_U8_UN] = 1; /* CEE_CONV_OVF_U8_UN */
	mono_burg_arity [CEE_CONV_OVF_I_UN] = 1; /* CEE_CONV_OVF_I_UN */
	mono_burg_arity [CEE_CONV_OVF_U_UN] = 1; /* CEE_CONV_OVF_U_UN */
	mono_burg_arity [CEE_CONV_OVF_U8] = 1; /* CEE_CONV_OVF_U8 */
	mono_burg_arity [CEE_CONV_OVF_U] = 1; /* CEE_CONV_OVF_U */
	mono_burg_arity [OP_LCALLVIRT] = 1; /* OP_LCALLVIRT */
	mono_burg_arity [OP_LCALLVIRT_IMT] = 2; /* OP_LCALLVIRT_IMT */
	mono_burg_arity [OP_LCALL] = 0; /* OP_LCALL */
	mono_burg_arity [OP_LCALL_RGCTX] = 1; /* OP_LCALL_RGCTX */
	mono_burg_arity [OP_LCALL_REG] = 1; /* OP_LCALL_REG */
	mono_burg_arity [OP_LCALL_REG_RGCTX] = 2; /* OP_LCALL_REG_RGCTX */
	mono_burg_arity [OP_LCOMPARE] = 2; /* OP_LCOMPARE */
	mono_burg_arity [CEE_BNE_UN] = 1; /* CEE_BNE_UN */
	mono_burg_arity [CEE_BEQ] = 1; /* CEE_BEQ */
	mono_burg_arity [CEE_BLT] = 1; /* CEE_BLT */
	mono_burg_arity [CEE_BLT_UN] = 1; /* CEE_BLT_UN */
	mono_burg_arity [CEE_BGT] = 1; /* CEE_BGT */
	mono_burg_arity [CEE_BGT_UN] = 1; /* CEE_BGT_UN */
	mono_burg_arity [CEE_BGE] = 1; /* CEE_BGE */
	mono_burg_arity [CEE_BGE_UN] = 1; /* CEE_BGE_UN */
	mono_burg_arity [CEE_BLE] = 1; /* CEE_BLE */
	mono_burg_arity [CEE_BLE_UN] = 1; /* CEE_BLE_UN */
	mono_burg_arity [CEE_AND] = 2; /* CEE_AND */
	mono_burg_arity [CEE_OR] = 2; /* CEE_OR */
	mono_burg_arity [CEE_XOR] = 2; /* CEE_XOR */
	mono_burg_arity [CEE_NEG] = 1; /* CEE_NEG */
	mono_burg_arity [CEE_NOT] = 1; /* CEE_NOT */
	mono_burg_arity [CEE_ADD_OVF] = 2; /* CEE_ADD_OVF */
	mono_burg_arity [CEE_ADD_OVF_UN] = 2; /* CEE_ADD_OVF_UN */
	mono_burg_arity [CEE_SUB] = 2; /* CEE_SUB */
	mono_burg_arity [CEE_SUB_OVF] = 2; /* CEE_SUB_OVF */
	mono_burg_arity [CEE_SUB_OVF_UN] = 2; /* CEE_SUB_OVF_UN */
	mono_burg_arity [CEE_SHL] = 2; /* CEE_SHL */
	mono_burg_arity [CEE_SHR] = 2; /* CEE_SHR */
	mono_burg_arity [CEE_SHR_UN] = 2; /* CEE_SHR_UN */
	mono_burg_arity [CEE_MUL] = 2; /* CEE_MUL */
	mono_burg_arity [CEE_MUL_OVF] = 2; /* CEE_MUL_OVF */
	mono_burg_arity [CEE_MUL_OVF_UN] = 2; /* CEE_MUL_OVF_UN */
	mono_burg_arity [CEE_DIV] = 2; /* CEE_DIV */
	mono_burg_arity [CEE_DIV_UN] = 2; /* CEE_DIV_UN */
	mono_burg_arity [CEE_REM] = 2; /* CEE_REM */
	mono_burg_arity [CEE_REM_UN] = 2; /* CEE_REM_UN */
	mono_burg_arity [OP_IMUL_IMM] = 1; /* OP_IMUL_IMM */
	mono_burg_arity [OP_LMUL_IMM] = 1; /* OP_LMUL_IMM */
	mono_burg_arity [OP_LCONV_TO_R8] = 1; /* OP_LCONV_TO_R8 */
	mono_burg_arity [OP_LCONV_TO_R4] = 1; /* OP_LCONV_TO_R4 */
	mono_burg_arity [OP_LOCALLOC] = 1; /* OP_LOCALLOC */
	mono_burg_arity [OP_SETRET] = 1; /* OP_SETRET */
	mono_burg_arity [OP_OUTARG] = 1; /* OP_OUTARG */
	mono_burg_arity [OP_OUTARG_REG] = 1; /* OP_OUTARG_REG */
	mono_burg_arity [OP_OUTARG_R4] = 1; /* OP_OUTARG_R4 */
	mono_burg_arity [OP_OUTARG_R8] = 1; /* OP_OUTARG_R8 */
	mono_burg_arity [OP_AMD64_OUTARG_XMMREG_R4] = 1; /* OP_AMD64_OUTARG_XMMREG_R4 */
	mono_burg_arity [OP_AMD64_OUTARG_XMMREG_R8] = 1; /* OP_AMD64_OUTARG_XMMREG_R8 */
	mono_burg_arity [OP_OUTARG_VT] = 1; /* OP_OUTARG_VT */
	mono_burg_arity [OP_AMD64_OUTARG_ALIGN_STACK] = 0; /* OP_AMD64_OUTARG_ALIGN_STACK */
	mono_burg_arity [OP_AMD64_SAVE_SP_TO_LMF] = 0; /* OP_AMD64_SAVE_SP_TO_LMF */
	mono_burg_arity [OP_INARG_VT] = 1; /* OP_INARG_VT */
	mono_burg_arity [OP_VTARG_ADDR] = 1; /* OP_VTARG_ADDR */
	mono_burg_arity [OP_IMIN] = 2; /* OP_IMIN */
	mono_burg_arity [OP_IMIN_UN] = 2; /* OP_IMIN_UN */
	mono_burg_arity [OP_IMAX] = 2; /* OP_IMAX */
	mono_burg_arity [OP_IMAX_UN] = 2; /* OP_IMAX_UN */
	mono_burg_arity [OP_LMIN] = 2; /* OP_LMIN */
	mono_burg_arity [OP_LMIN_UN] = 2; /* OP_LMIN_UN */
	mono_burg_arity [OP_LMAX] = 2; /* OP_LMAX */
	mono_burg_arity [OP_LMAX_UN] = 2; /* OP_LMAX_UN */
	mono_burg_arity [OP_ATOMIC_ADD_NEW_I4] = 2; /* OP_ATOMIC_ADD_NEW_I4 */
	mono_burg_arity [OP_ATOMIC_ADD_NEW_I8] = 2; /* OP_ATOMIC_ADD_NEW_I8 */
	mono_burg_arity [OP_ATOMIC_ADD_I4] = 2; /* OP_ATOMIC_ADD_I4 */
	mono_burg_arity [OP_ATOMIC_ADD_I8] = 2; /* OP_ATOMIC_ADD_I8 */
	mono_burg_arity [OP_ATOMIC_EXCHANGE_I4] = 2; /* OP_ATOMIC_EXCHANGE_I4 */
	mono_burg_arity [OP_ATOMIC_EXCHANGE_I8] = 2; /* OP_ATOMIC_EXCHANGE_I8 */
	mono_burg_arity [OP_ATOMIC_CAS_IMM_I4] = 2; /* OP_ATOMIC_CAS_IMM_I4 */
}

#endif /* HAVE_ARRAY_ELEM_INIT */
#if MONOBURG_LOG
const char * const mono_burg_rule_string [] = {
	NULL,
	"base: reg",
	"base: CEE_LDIND_I(OP_REGVAR)",
	"base: OP_REGOFFSET",
	"base: OP_LDADDR(OP_REGOFFSET)",
	"base: CEE_LDOBJ(OP_REGOFFSET)",
	"base: CEE_ADD(base, OP_ICONST)",
	"base: CEE_ADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST)",
	"base: CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)",
	"reg: OP_ICONST",
	"reg: OP_AOTCONST",
	"reg: OP_LOAD_GOTADDR",
	"stmt: CEE_STIND_I(OP_REGVAR, OP_LOAD_GOTADDR)",
	"reg: OP_GOT_ENTRY(reg, OP_PATCH_INFO)",
	"reg: OP_GOT_ENTRY(CEE_LDIND_I(OP_REGVAR), OP_PATCH_INFO)",
	"stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGVAR))",
	"stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGOFFSET))",
	"stmt: OP_DUMMY_USE(CEE_LDIND_REF(OP_REGOFFSET))",
	"stmt: OP_DUMMY_STORE",
	"stmt: OP_NOT_REACHED",
	"stmt: OP_MEMORY_BARRIER",
	"reg: CEE_LDIND_I(base)",
	"reg: CEE_LDIND_REF(base)",
	"reg: CEE_LDIND_I1(base)",
	"reg: CEE_LDIND_U1(base)",
	"reg: CEE_LDIND_I2(base)",
	"reg: CEE_LDIND_U2(base)",
	"reg: CEE_LDIND_I4(base)",
	"reg: CEE_LDIND_U4(base)",
	"reg: OP_LDADDR(OP_REGOFFSET)",
	"reg: CEE_LDOBJ(OP_REGOFFSET)",
	"reg: CEE_LDOBJ(reg)",
	"reg: OP_OBJADDR(reg)",
	"reg: OP_VTADDR(reg)",
	"reg: CEE_LDIND_U1(OP_REGVAR)",
	"reg: CEE_LDIND_U2(OP_REGVAR)",
	"stmt: CEE_STIND_REF(base, CEE_LDIND_REF(OP_REGVAR))",
	"stmt: CEE_STIND_REF(base, CEE_LDIND_I(OP_REGVAR))",
	"stmt: CEE_STIND_I(base, CEE_LDIND_REF(OP_REGVAR))",
	"stmt: CEE_STIND_I(base, CEE_LDIND_I(OP_REGVAR))",
	"stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR))",
	"stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_I(OP_REGVAR))",
	"stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR))",
	"stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_I(OP_REGVAR))",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR))",
	"stmt: CEE_STIND_I(base, reg)",
	"stmt: CEE_STIND_REF(base, reg)",
	"stmt: CEE_STIND_I(base, OP_ICONST)",
	"stmt: CEE_STIND_REF(base, OP_ICONST)",
	"reg: CEE_LDIND_REF(OP_REGVAR)",
	"reg: CEE_LDIND_I(OP_REGVAR)",
	"reg: CEE_LDIND_I4(OP_REGVAR)",
	"reg: CEE_LDIND_U4(OP_REGVAR)",
	"stmt: CEE_STIND_I4(base, reg)",
	"stmt: CEE_STIND_I4(base, CEE_LDIND_I4(OP_REGVAR))",
	"stmt: CEE_STIND_I4(base, OP_ICONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base))",
	"stmt: CEE_STIND_I1(base, reg)",
	"stmt: CEE_STIND_I1(base, OP_ICONST)",
	"stmt: CEE_STIND_I2(base, reg)",
	"stmt: CEE_STIND_I2(base, OP_ICONST)",
	"stmt: CEE_STIND_I2(OP_REGVAR, reg)",
	"stmt: CEE_STIND_I1(OP_REGVAR, reg)",
	"stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_U1(OP_REGVAR))",
	"stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_I1(OP_REGVAR))",
	"stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_U2(OP_REGVAR))",
	"stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_I2(OP_REGVAR))",
	"stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, reg)",
	"stmt: CEE_STIND_REF(OP_REGVAR, reg)",
	"stmt: CEE_STIND_I(OP_REGVAR, reg)",
	"stmt: OP_MEMCPY(base, base)",
	"stmt: OP_MEMSET(base)",
	"reg: OP_LDELEMA2D(reg, OP_GROUP(reg, reg))",
	"reg: CEE_CONV_I1(reg)",
	"reg: CEE_CONV_I2(reg)",
	"reg: CEE_CONV_I4(reg)",
	"reg: CEE_CONV_U1(reg)",
	"reg: CEE_CONV_U2(reg)",
	"reg: CEE_CONV_U4(reg)",
	"reg: CEE_CONV_U(reg)",
	"reg: CEE_CONV_I(reg)",
	"reg: CEE_CONV_OVF_I4(reg)",
	"reg: CEE_CONV_OVF_U4(reg)",
	"reg: CEE_CONV_OVF_I4_UN(reg)",
	"reg: CEE_CONV_OVF_I1(reg)",
	"reg: CEE_CONV_OVF_I1_UN(reg)",
	"reg: CEE_CONV_OVF_U1(reg)",
	"reg: CEE_CONV_OVF_U1_UN(reg)",
	"reg: CEE_CONV_OVF_I2(reg)",
	"reg: CEE_CONV_OVF_I2_UN(reg)",
	"reg: CEE_CONV_OVF_U2(reg)",
	"reg: CEE_CONV_OVF_U2_UN(reg)",
	"reg: OP_CEQ(cflags)",
	"reg: OP_CLT(cflags)",
	"reg: OP_CLT_UN(cflags)",
	"reg: OP_CGT(cflags)",
	"reg: OP_CGT_UN(cflags)",
	"stmt: OP_LABEL",
	"stmt: OP_NOP",
	"stmt: OP_BREAK",
	"reg: OP_TLS_GET",
	"stmt: OP_SWITCH(reg)",
	"stmt: OP_BR",
	"stmt: OP_ARGLIST(reg)",
	"reg: OP_CALL",
	"reg: OP_CALLVIRT(reg)",
	"stmt: OP_VOIDCALLVIRT(reg)",
	"freg: OP_FCALLVIRT(reg)",
	"stmt: OP_VCALLVIRT(reg, reg)",
	"reg: OP_CALLVIRT_IMT(reg, reg)",
	"stmt: OP_VOIDCALLVIRT_IMT(reg, reg)",
	"freg: OP_FCALLVIRT_IMT(reg, reg)",
	"stmt: OP_VCALLVIRT_IMT(OP_GROUP(reg, reg), reg)",
	"reg: OP_CALL_REG(reg)",
	"stmt: OP_VOIDCALL",
	"stmt: OP_VOIDCALL_REG(reg)",
	"freg: OP_FCALL",
	"freg: OP_FCALL_REG(reg)",
	"stmt: OP_VCALL(reg)",
	"stmt: OP_VCALL_REG(reg, reg)",
	"stmt: OP_TRAMPCALL_VTABLE(reg)",
	"reg: OP_CALL_RGCTX(reg)",
	"freg: OP_FCALL_RGCTX(reg)",
	"stmt: OP_VOIDCALL_RGCTX(reg)",
	"stmt: OP_VCALL_RGCTX(reg, reg)",
	"reg: OP_CALL_REG_RGCTX(reg, reg)",
	"freg: OP_FCALL_REG_RGCTX(reg, reg)",
	"stmt: OP_VOIDCALL_REG_RGCTX(reg, reg)",
	"stmt: OP_VCALL_REG_RGCTX(OP_GROUP(reg, reg), reg)",
	"stmt: OP_SAVE_LMF",
	"stmt: OP_RESTORE_LMF",
	"reg: OP_CALL_REG(OP_ICONST)",
	"reg: OP_CALL_REG(OP_I8CONST)",
	"stmt: OP_VOIDCALL_REG(OP_ICONST)",
	"stmt: OP_VOIDCALL_REG(OP_I8CONST)",
	"freg: OP_FCALL_REG(OP_ICONST)",
	"freg: OP_FCALL_REG(OP_I8CONST)",
	"stmt: CEE_POP(reg)",
	"stmt: CEE_POP(OP_ICONST)",
	"stmt: CEE_POP(CEE_LDIND_I1(base))",
	"stmt: CEE_POP(CEE_LDIND_U1(base))",
	"stmt: CEE_POP(CEE_LDIND_I2(base))",
	"stmt: CEE_POP(CEE_LDIND_U2(base))",
	"stmt: CEE_POP(CEE_LDIND_I4(base))",
	"stmt: CEE_POP(CEE_LDIND_U4(base))",
	"stmt: CEE_POP(CEE_LDIND_I(base))",
	"stmt: CEE_POP(CEE_LDIND_REF(base))",
	"stmt: CEE_POP(CEE_LDIND_I1(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_U1(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_I2(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_U2(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_I4(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_U4(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_I(OP_REGVAR))",
	"stmt: CEE_POP(CEE_LDIND_REF(OP_REGVAR))",
	"stmt: OP_JMP",
	"stmt: OP_THROW(reg)",
	"stmt: OP_RETHROW(reg)",
	"stmt: OP_THROW_OR_NULL(reg, reg)",
	"stmt: OP_THROW(CEE_LDIND_REF(OP_REGVAR))",
	"stmt: OP_CALL_HANDLER",
	"stmt: OP_START_HANDLER",
	"stmt: OP_ENDFINALLY",
	"stmt: OP_ENDFILTER(reg)",
	"stmt: OP_CHECK_THIS(reg)",
	"reg: OP_CHECK_THIS_PASSTHROUGH(reg)",
	"reg: CEE_ISINST(reg)",
	"reg: OP_CISINST(reg)",
	"reg: OP_UNBOXCAST(reg)",
	"reg: OP_UNBOXCAST_REG(reg, reg)",
	"reg: CEE_CASTCLASS(reg)",
	"reg: OP_CCASTCLASS(reg)",
	"stmt: CEE_STELEM_REF(OP_GROUP(reg, reg), reg)",
	"reg: OP_GROUP(reg, reg)",
	"reg: CEE_NEWARR(reg)",
	"lreg: OP_LMUL(lreg, lreg)",
	"lreg: OP_LMUL_OVF(lreg, lreg)",
	"lreg: OP_LMUL_OVF_UN(lreg, lreg)",
	"lreg: OP_LDIV(lreg, lreg)",
	"lreg: OP_LDIV_UN(lreg, lreg)",
	"lreg: OP_LREM(lreg, lreg)",
	"lreg: OP_LREM_UN(lreg, lreg)",
	"lreg: OP_LSHL(lreg, reg)",
	"lreg: OP_LSHR(lreg, reg)",
	"lreg: OP_LSHR_UN(lreg, reg)",
	"reg: OP_GETCHR(reg, reg)",
	"reg: OP_GETCHR(reg, OP_ICONST)",
	"reg: OP_STR_CHAR_ADDR(reg, reg)",
	"reg: OP_STRLEN(reg)",
	"reg: OP_GETTYPE(reg)",
	"reg: OP_GETHASHCODE(reg)",
	"reg: CEE_LDLEN(reg)",
	"reg: OP_ARRAY_RANK(reg)",
	"reg: OP_CHECK_ARRAY_TYPE(reg)",
	"reg: CEE_LDELEMA(reg, OP_ICONST)",
	"reg: CEE_LDELEMA(reg, reg)",
	"reg: CEE_REFANYVAL(reg)",
	"reg: OP_REFANYVAL_REG(reg, reg)",
	"reg: OP_REFANYTYPE(reg)",
	"stmt: CEE_MKREFANY(OP_GROUP(reg, OP_ICONST), reg)",
	"stmt: CEE_MKREFANY(OP_GROUP(reg, OP_I8CONST), reg)",
	"stmt: OP_MKREFANY_REGS(OP_GROUP(reg, reg), OP_GROUP(reg, reg))",
	"freg: CEE_LDIND_R4(base)",
	"freg: CEE_LDIND_R8(base)",
	"stmt: CEE_STIND_R4(base, freg)",
	"stmt: CEE_STIND_R8(base, freg)",
	"freg: OP_R4CONST",
	"freg: OP_R8CONST",
	"freg: OP_FADD(freg, freg)",
	"freg: OP_FSUB(freg, freg)",
	"freg: OP_FMUL(freg, freg)",
	"freg: OP_FDIV(freg, freg)",
	"freg: OP_FREM(freg, freg)",
	"freg: OP_FNEG(freg)",
	"freg: OP_CKFINITE(freg)",
	"freg: OP_SIN(freg)",
	"freg: OP_COS(freg)",
	"freg: OP_ABS(freg)",
	"freg: OP_TAN(freg)",
	"freg: OP_ATAN(freg)",
	"freg: OP_SQRT(freg)",
	"reg: OP_FCONV_TO_I4(freg)",
	"reg: OP_FCONV_TO_U4(freg)",
	"reg: OP_FCONV_TO_OVF_I4(freg)",
	"reg: OP_FCONV_TO_OVF_U4(freg)",
	"reg: OP_FCONV_TO_OVF_I8(freg)",
	"reg: OP_FCONV_TO_OVF_U8(freg)",
	"reg: OP_FCONV_TO_OVF_I(freg)",
	"reg: OP_FCONV_TO_OVF_U(freg)",
	"reg: OP_FCONV_TO_I(freg)",
	"reg: OP_FCONV_TO_U(freg)",
	"reg: OP_FCONV_TO_I2(freg)",
	"reg: OP_FCONV_TO_U2(freg)",
	"reg: OP_FCONV_TO_I1(freg)",
	"reg: OP_FCONV_TO_U1(freg)",
	"freg: OP_FCONV_TO_R8(freg)",
	"freg: OP_FCONV_TO_R4(freg)",
	"freg: CEE_CONV_R4(reg)",
	"freg: CEE_CONV_R8(reg)",
	"freg: CEE_CONV_R_UN(reg)",
	"fpcflags: OP_COMPARE(freg, freg)",
	"stmt: CEE_POP(freg)",
	"reg: OP_I8CONST",
	"reg: CEE_LDIND_I8(base)",
	"stmt: CEE_STIND_I8(base, reg)",
	"stmt: CEE_STIND_REF(base, OP_I8CONST)",
	"stmt: CEE_STIND_I(base, OP_I8CONST)",
	"stmt: CEE_STIND_I8(base, OP_I8CONST)",
	"stmt: CEE_STIND_REF(base, OP_ICONST)",
	"stmt: CEE_STIND_I(base, OP_ICONST)",
	"stmt: CEE_STIND_I8(base, OP_ICONST)",
	"stmt: CEE_STIND_I8(reg, reg)",
	"base: OP_LADD(base, OP_ICONST)",
	"base: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST)",
	"base: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)",
	"reg: OP_LADD(reg, reg)",
	"reg: OP_LADD(reg, OP_ICONST)",
	"reg: OP_LADD(reg, OP_ICONST)",
	"reg: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)",
	"reg: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST)",
	"reg: OP_LSUB(reg, reg)",
	"reg: OP_LNEG(reg)",
	"reg: OP_LNOT(reg)",
	"reg: OP_LAND(reg, reg)",
	"reg: OP_LOR(reg, reg)",
	"reg: OP_LXOR(reg, reg)",
	"reg: OP_LADD_OVF(reg, reg)",
	"reg: OP_LADD_OVF_UN(reg, reg)",
	"reg: OP_LSUB_OVF(reg, reg)",
	"reg: OP_LSUB_OVF_UN(reg, reg)",
	"reg: OP_LONG_SHRUN_32(reg)",
	"reg: OP_LSHL(reg, reg)",
	"reg: OP_LSHL(reg, OP_ICONST)",
	"reg: OP_LSHR(reg, reg)",
	"reg: OP_LSHR(reg, OP_ICONST)",
	"reg: OP_LSHR_UN(reg, reg)",
	"reg: OP_LSHR_UN(reg, OP_ICONST)",
	"reg: CEE_CONV_I8(reg)",
	"reg: CEE_CONV_U8(reg)",
	"i8con: CEE_CONV_U8(OP_ICONST)",
	"reg: OP_FCONV_TO_I8(freg)",
	"reg: OP_FCONV_TO_U8(freg)",
	"reg: OP_LCONV_TO_I4(i8con)",
	"reg: OP_LCONV_TO_I4(reg)",
	"reg: OP_LCONV_TO_U4(reg)",
	"reg: OP_LCONV_TO_U8(reg)",
	"reg: OP_LCONV_TO_I8(reg)",
	"reg: OP_LCONV_TO_U(reg)",
	"reg: OP_LCONV_TO_I(reg)",
	"reg: OP_LCONV_TO_I1(reg)",
	"reg: OP_LCONV_TO_U1(reg)",
	"reg: OP_LCONV_TO_I2(reg)",
	"reg: OP_LCONV_TO_U2(reg)",
	"reg: OP_LCONV_TO_OVF_I1(reg)",
	"reg: OP_LCONV_TO_OVF_I1_UN(reg)",
	"reg: OP_LCONV_TO_OVF_U1(reg)",
	"reg: OP_LCONV_TO_OVF_U1_UN(reg)",
	"reg: OP_LCONV_TO_OVF_I2(reg)",
	"reg: OP_LCONV_TO_OVF_I2_UN(reg)",
	"reg: OP_LCONV_TO_OVF_U2(reg)",
	"reg: OP_LCONV_TO_OVF_U2_UN(reg)",
	"reg: OP_LCONV_TO_OVF_I4(reg)",
	"reg: OP_LCONV_TO_OVF_I4_UN(reg)",
	"reg: OP_LCONV_TO_OVF_U4(reg)",
	"reg: OP_LCONV_TO_OVF_U4_UN(reg)",
	"reg: OP_LCONV_TO_OVF_I_UN(reg)",
	"reg: OP_LCONV_TO_OVF_I8_UN(reg)",
	"reg: OP_LCONV_TO_OVF_U8(reg)",
	"reg: OP_LCONV_TO_OVF_U(reg)",
	"reg: OP_LCONV_TO_OVF_I8(reg)",
	"reg: OP_LCONV_TO_OVF_U_UN(reg)",
	"reg: OP_LCONV_TO_OVF_U8_UN(reg)",
	"reg: OP_LCONV_TO_OVF_I(reg)",
	"freg: OP_LCONV_TO_R_UN(lreg)",
	"reg: CEE_CONV_OVF_I8(reg)",
	"reg: CEE_CONV_OVF_I(reg)",
	"reg: CEE_CONV_OVF_I8_UN(reg)",
	"reg: CEE_CONV_OVF_U8_UN(reg)",
	"reg: CEE_CONV_OVF_I_UN(reg)",
	"reg: CEE_CONV_OVF_U_UN(reg)",
	"reg: CEE_CONV_OVF_U8(reg)",
	"reg: CEE_CONV_OVF_U(reg)",
	"reg: OP_LCALLVIRT(reg)",
	"reg: OP_LCALLVIRT_IMT(reg, reg)",
	"reg: OP_LCALL",
	"reg: OP_LCALL_RGCTX(reg)",
	"reg: OP_LCALL_REG(reg)",
	"reg: OP_LCALL_REG_RGCTX(reg, reg)",
	"reg: OP_LCALL_REG(OP_ICONST)",
	"reg: OP_LCALL_REG(OP_I8CONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, reg)",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR))",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base))",
	"cflags: OP_LCOMPARE(reg, reg)",
	"cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), reg)",
	"cflags: OP_LCOMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR))",
	"cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), OP_ICONST)",
	"cflags: OP_LCOMPARE(reg, OP_ICONST)",
	"cflags: OP_LCOMPARE(reg, OP_I8CONST)",
	"stmt: CEE_BNE_UN(cflags)",
	"stmt: CEE_BEQ(cflags)",
	"stmt: CEE_BLT(cflags)",
	"stmt: CEE_BLT_UN(cflags)",
	"stmt: CEE_BGT(cflags)",
	"stmt: CEE_BGT_UN(cflags)",
	"stmt: CEE_BGE(cflags)",
	"stmt: CEE_BGE_UN(cflags)",
	"stmt: CEE_BLE(cflags)",
	"stmt: CEE_BLE_UN(cflags)",
	"reg: CEE_AND(reg, reg)",
	"reg: CEE_AND(reg, OP_ICONST)",
	"reg: CEE_OR(reg, reg)",
	"reg: CEE_OR(reg, OP_ICONST)",
	"reg: CEE_XOR(reg, reg)",
	"reg: CEE_XOR(reg, OP_ICONST)",
	"reg: CEE_NEG(reg)",
	"reg: CEE_NOT(reg)",
	"reg: CEE_ADD(reg, reg)",
	"reg: CEE_ADD(reg, OP_ICONST)",
	"reg: CEE_ADD_OVF(reg, reg)",
	"reg: CEE_ADD_OVF_UN(reg, reg)",
	"reg: CEE_SUB(reg, reg)",
	"reg: CEE_SUB(reg, CEE_LDIND_I4(OP_REGVAR))",
	"reg: CEE_SUB(reg, OP_ICONST)",
	"reg: CEE_SUB_OVF(reg, reg)",
	"reg: CEE_SUB_OVF_UN(reg, reg)",
	"reg: CEE_SHL(reg, reg)",
	"reg: CEE_SHL(reg, OP_ICONST)",
	"reg: CEE_SHR(reg, reg)",
	"reg: CEE_SHR(reg, OP_ICONST)",
	"reg: CEE_SHR_UN(reg, reg)",
	"reg: CEE_SHR_UN(reg, OP_ICONST)",
	"reg: CEE_MUL(reg, reg)",
	"reg: CEE_MUL(reg, OP_ICONST)",
	"reg: CEE_MUL_OVF(reg, reg)",
	"reg: CEE_MUL_OVF_UN(reg, reg)",
	"reg: CEE_DIV(reg, reg)",
	"reg: CEE_DIV_UN(reg, reg)",
	"reg: CEE_REM(reg, reg)",
	"reg: CEE_REM_UN(reg, reg)",
	"reg: OP_IMUL_IMM(reg)",
	"reg: OP_LMUL_IMM(reg)",
	"c32flags: OP_COMPARE(reg, reg)",
	"c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR))",
	"c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), OP_ICONST)",
	"c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), reg)",
	"c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)",
	"c32flags: OP_COMPARE(reg, OP_ICONST)",
	"stmt: CEE_BNE_UN(c32flags)",
	"stmt: CEE_BEQ(c32flags)",
	"stmt: CEE_BLT(c32flags)",
	"stmt: CEE_BLT_UN(c32flags)",
	"stmt: CEE_BGT(c32flags)",
	"stmt: CEE_BGT_UN(c32flags)",
	"stmt: CEE_BGE(c32flags)",
	"stmt: CEE_BGE_UN(c32flags)",
	"stmt: CEE_BLE(c32flags)",
	"stmt: CEE_BLE_UN(c32flags)",
	"reg: OP_CEQ(c32flags)",
	"reg: OP_CLT(c32flags)",
	"reg: OP_CLT_UN(c32flags)",
	"reg: OP_CGT(c32flags)",
	"reg: OP_CGT_UN(c32flags)",
	"reg: CEE_LDIND_I8(OP_REGVAR)",
	"stmt: CEE_STIND_I8(OP_REGVAR, reg)",
	"reg: CEE_LDIND_I1(OP_REGVAR)",
	"reg: CEE_LDIND_I2(OP_REGVAR)",
	"stmt: OP_START_HANDLER",
	"stmt: OP_ENDFINALLY",
	"stmt: OP_ENDFILTER(reg)",
	"freg: OP_LCONV_TO_R_UN(reg)",
	"freg: OP_LCONV_TO_R8(reg)",
	"freg: OP_LCONV_TO_R4(reg)",
	"cflags: OP_COMPARE(CEE_LDIND_I4(base), reg)",
	"cflags: OP_COMPARE(CEE_LDIND_I4(base), OP_ICONST)",
	"cflags: OP_COMPARE(reg, CEE_LDIND_I4(base))",
	"stmt: CEE_STIND_I1(base, OP_CEQ(cflags))",
	"reg: OP_LOCALLOC(OP_ICONST)",
	"reg: OP_LOCALLOC(reg)",
	"stmt: OP_SETRET(reg)",
	"stmt: OP_SETRET(reg)",
	"reg: CEE_LDIND_REF(OP_REGVAR)",
	"reg: CEE_LDIND_I(OP_REGVAR)",
	"reg: CEE_LDIND_I4(OP_REGVAR)",
	"reg: CEE_LDIND_U4(OP_REGVAR)",
	"stmt: OP_SETRET(CEE_LDIND_REF(OP_REGVAR))",
	"stmt: OP_SETRET(CEE_LDIND_I4(OP_REGVAR))",
	"stmt: OP_SETRET(CEE_LDIND_U4(OP_REGVAR))",
	"stmt: OP_SETRET(CEE_LDIND_I(OP_REGVAR))",
	"stmt: OP_SETRET(freg)",
	"stmt: OP_SETRET(OP_ICONST)",
	"stmt: OP_OUTARG(reg)",
	"stmt: OP_OUTARG_REG(reg)",
	"stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR))",
	"stmt: OP_OUTARG(CEE_LDIND_I4(OP_REGVAR))",
	"stmt: OP_OUTARG(CEE_LDIND_U4(OP_REGVAR))",
	"stmt: OP_OUTARG(CEE_LDIND_I(OP_REGVAR))",
	"stmt: OP_OUTARG(CEE_LDIND_I(base))",
	"stmt: OP_OUTARG(CEE_LDIND_REF(base))",
	"stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR))",
	"stmt: OP_OUTARG(CEE_LDOBJ(reg))",
	"stmt: OP_OUTARG(freg)",
	"stmt: OP_OUTARG_R4(freg)",
	"stmt: OP_OUTARG_R8(freg)",
	"stmt: OP_AMD64_OUTARG_XMMREG_R4(freg)",
	"stmt: OP_AMD64_OUTARG_XMMREG_R8(freg)",
	"stmt: OP_OUTARG_VT(CEE_LDOBJ(base))",
	"stmt: OP_OUTARG_VT(OP_ICONST)",
	"stmt: OP_OUTARG_VT(reg)",
	"stmt: OP_AMD64_OUTARG_ALIGN_STACK",
	"stmt: OP_AMD64_SAVE_SP_TO_LMF",
	"base: OP_INARG_VT(base)",
	"reg: OP_LDADDR(OP_INARG_VT(base))",
	"reg: CEE_LDOBJ(OP_INARG_VT(base))",
	"reg: OP_LDADDR(OP_REGOFFSET)",
	"reg: CEE_LDOBJ(OP_REGOFFSET)",
	"reg: CEE_LDELEMA(reg, reg)",
	"stmt: CEE_STIND_R8(OP_REGVAR, freg)",
	"stmt: CEE_STIND_I4(base, CEE_ADD(CEE_LDIND_I4(base), OP_ICONST))",
	"stmt: CEE_STIND_I4(base, CEE_SUB(CEE_LDIND_I4(base), OP_ICONST))",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_SUB(CEE_LDIND_I4(OP_REGVAR), OP_ICONST))",
	"stmt: CEE_STIND_I(OP_REGVAR, CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST))",
	"stmt: CEE_STIND_I4(OP_REGVAR, CEE_ADD(CEE_LDIND_I4(OP_REGVAR), OP_ICONST))",
	"reg: CEE_LDIND_I2(OP_REGVAR)",
	"stmt: CEE_STIND_I8(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST)",
	"stmt: CEE_STIND_I8(OP_REGVAR, OP_I8CONST)",
	"stmt: CEE_STIND_I4(OP_REGVAR, OP_I8CONST)",
	"stmt: CEE_STIND_I2(OP_REGVAR, OP_I8CONST)",
	"stmt: CEE_STIND_I1(OP_REGVAR, OP_I8CONST)",
	"stmt: CEE_STIND_REF(OP_REGVAR, OP_I8CONST)",
	"stmt: CEE_STIND_I(OP_REGVAR, OP_I8CONST)",
	"reg: OP_CEQ(OP_COMPARE(freg, freg))",
	"reg: OP_CLT(OP_COMPARE(freg, freg))",
	"reg: OP_CGT(OP_COMPARE(freg, freg))",
	"reg: OP_CLT_UN(OP_COMPARE(freg, freg))",
	"reg: OP_CGT_UN(OP_COMPARE(freg, freg))",
	"stmt: CEE_BNE_UN(fpcflags)",
	"stmt: CEE_BEQ(fpcflags)",
	"stmt: CEE_BLT(fpcflags)",
	"stmt: CEE_BLT_UN(fpcflags)",
	"stmt: CEE_BGT(fpcflags)",
	"stmt: CEE_BGT_UN(fpcflags)",
	"stmt: CEE_BGE(fpcflags)",
	"stmt: CEE_BGE_UN(fpcflags)",
	"stmt: CEE_BLE(fpcflags)",
	"stmt: CEE_BLE_UN(fpcflags)",
	"stmt: CEE_POP(freg)",
	"freg: OP_FCONV_TO_R8(freg)",
	"freg: OP_FCONV_TO_R4(freg)",
	"reg: CEE_ADD(reg, CEE_LDIND_I4(base))",
	"reg: CEE_SUB(reg, CEE_LDIND_I4(base))",
	"reg: CEE_MUL(reg, CEE_LDIND_I4(base))",
	"reg: OP_VTARG_ADDR(base)",
	"reg: OP_LDADDR(OP_VTARG_ADDR(base))",
	"reg: CEE_LDOBJ(OP_VTARG_ADDR(base))",
	"reg: OP_LSHL(reg, reg)",
	"reg: OP_LSHR(reg, reg)",
	"reg: OP_LSHR_UN(reg, reg)",
	"reg: OP_LMUL(reg, reg)",
	"reg: OP_LDIV(reg, reg)",
	"reg: OP_LDIV_UN(reg, reg)",
	"reg: OP_LREM(reg, reg)",
	"reg: OP_LREM_UN(reg, reg)",
	"reg: OP_LMUL_OVF(reg, reg)",
	"reg: OP_LMUL_OVF_UN(reg, reg)",
	"reg: OP_IMIN(reg, reg)",
	"reg: OP_IMIN_UN(reg, reg)",
	"reg: OP_IMAX(reg, reg)",
	"reg: OP_IMAX_UN(reg, reg)",
	"reg: OP_LMIN(reg, reg)",
	"reg: OP_LMIN_UN(reg, reg)",
	"reg: OP_LMAX(reg, reg)",
	"reg: OP_LMAX_UN(reg, reg)",
	"reg: OP_LMUL(reg, OP_I8CONST)",
	"reg: OP_LSHL(reg, OP_ICONST)",
	"reg: OP_LSHR(reg, OP_ICONST)",
	"reg: OP_LSHR_UN(reg, OP_ICONST)",
	"reg: OP_ATOMIC_ADD_NEW_I4(base, reg)",
	"reg: OP_ATOMIC_ADD_NEW_I8(base, reg)",
	"reg: OP_ATOMIC_ADD_I4(base, reg)",
	"reg: OP_ATOMIC_ADD_I8(base, reg)",
	"reg: OP_ATOMIC_EXCHANGE_I4(base, reg)",
	"reg: OP_ATOMIC_EXCHANGE_I8(base, reg)",
	"reg: OP_ATOMIC_CAS_IMM_I4(base, reg)",
	"reg: OP_CALL_REG(CEE_LDIND_I(base))",
	"freg: OP_FCALL_REG(CEE_LDIND_I(base))",
	"reg: OP_LCALL_REG(CEE_LDIND_I(base))",
	"stmt: OP_VOIDCALL_REG(CEE_LDIND_I(base))",
	"stmt: OP_VCALL_REG(CEE_LDIND_I(base), reg)",
};
#endif /* MONOBURG_LOG */

const guint16 mono_burg_nts_data [] = {
	0,
	MB_NTERM_reg, 0,
	0,
	MB_NTERM_base, 0,
	MB_NTERM_base, MB_NTERM_reg, 0,
	MB_NTERM_base, MB_NTERM_base, 0,
	MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg, 0,
	MB_NTERM_cflags, 0,
	MB_NTERM_reg, MB_NTERM_reg, 0,
	MB_NTERM_lreg, MB_NTERM_lreg, 0,
	MB_NTERM_lreg, MB_NTERM_reg, 0,
	MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg, 0,
	MB_NTERM_base, MB_NTERM_freg, 0,
	MB_NTERM_freg, MB_NTERM_freg, 0,
	MB_NTERM_freg, 0,
	MB_NTERM_i8con, 0,
	MB_NTERM_lreg, 0,
	MB_NTERM_c32flags, 0,
	MB_NTERM_reg, MB_NTERM_base, 0,
	MB_NTERM_base, MB_NTERM_cflags, 0,
	MB_NTERM_fpcflags, 0,
	0
};

const guint8 mono_burg_nts [] = {
	0,
	1, /* MB_NTERM_reg,  */ /* base: reg */
	3, /*  */ /* base: CEE_LDIND_I(OP_REGVAR) */
	3, /*  */ /* base: OP_REGOFFSET */
	3, /*  */ /* base: OP_LDADDR(OP_REGOFFSET) */
	3, /*  */ /* base: CEE_LDOBJ(OP_REGOFFSET) */
	4, /* MB_NTERM_base,  */ /* base: CEE_ADD(base, OP_ICONST) */
	3, /*  */ /* base: CEE_ADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
	3, /*  */ /* base: CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	3, /*  */ /* reg: OP_ICONST */
	3, /*  */ /* reg: OP_AOTCONST */
	3, /*  */ /* reg: OP_LOAD_GOTADDR */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, OP_LOAD_GOTADDR) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_GOT_ENTRY(reg, OP_PATCH_INFO) */
	3, /*  */ /* reg: OP_GOT_ENTRY(CEE_LDIND_I(OP_REGVAR), OP_PATCH_INFO) */
	3, /*  */ /* stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGOFFSET)) */
	3, /*  */ /* stmt: OP_DUMMY_USE(CEE_LDIND_REF(OP_REGOFFSET)) */
	3, /*  */ /* stmt: OP_DUMMY_STORE */
	3, /*  */ /* stmt: OP_NOT_REACHED */
	3, /*  */ /* stmt: OP_MEMORY_BARRIER */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_I(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_REF(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_I1(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_U1(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_I2(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_U2(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_I4(base) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_U4(base) */
	3, /*  */ /* reg: OP_LDADDR(OP_REGOFFSET) */
	3, /*  */ /* reg: CEE_LDOBJ(OP_REGOFFSET) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_LDOBJ(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_OBJADDR(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_VTADDR(reg) */
	3, /*  */ /* reg: CEE_LDIND_U1(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_U2(OP_REGVAR) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_REF(base, CEE_LDIND_REF(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_REF(base, CEE_LDIND_I(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I(base, CEE_LDIND_REF(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I(base, CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR)) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_REF(base, reg) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I(base, OP_ICONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_REF(base, OP_ICONST) */
	3, /*  */ /* reg: CEE_LDIND_REF(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_I(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_I4(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_U4(OP_REGVAR) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I4(base, reg) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(base, CEE_LDIND_I4(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(base, OP_ICONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base)) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I1(base, reg) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I1(base, OP_ICONST) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I2(base, reg) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I2(base, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I2(OP_REGVAR, reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I1(OP_REGVAR, reg) */
	3, /*  */ /* stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_U1(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_I1(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_U2(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_I2(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I4(OP_REGVAR, reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_REF(OP_REGVAR, reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I(OP_REGVAR, reg) */
	9, /* MB_NTERM_base, MB_NTERM_base,  */ /* stmt: OP_MEMCPY(base, base) */
	4, /* MB_NTERM_base,  */ /* stmt: OP_MEMSET(base) */
	12, /* MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LDELEMA2D(reg, OP_GROUP(reg, reg)) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_I1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_I2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_I4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_U1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_U2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_U4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_U(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_I(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I4_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I1_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U1_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I2_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U2_UN(reg) */
	16, /* MB_NTERM_cflags,  */ /* reg: OP_CEQ(cflags) */
	16, /* MB_NTERM_cflags,  */ /* reg: OP_CLT(cflags) */
	16, /* MB_NTERM_cflags,  */ /* reg: OP_CLT_UN(cflags) */
	16, /* MB_NTERM_cflags,  */ /* reg: OP_CGT(cflags) */
	16, /* MB_NTERM_cflags,  */ /* reg: OP_CGT_UN(cflags) */
	3, /*  */ /* stmt: OP_LABEL */
	3, /*  */ /* stmt: OP_NOP */
	3, /*  */ /* stmt: OP_BREAK */
	3, /*  */ /* reg: OP_TLS_GET */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_SWITCH(reg) */
	3, /*  */ /* stmt: OP_BR */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_ARGLIST(reg) */
	3, /*  */ /* reg: OP_CALL */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CALLVIRT(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_VOIDCALLVIRT(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: OP_FCALLVIRT(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VCALLVIRT(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_CALLVIRT_IMT(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VOIDCALLVIRT_IMT(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* freg: OP_FCALLVIRT_IMT(reg, reg) */
	12, /* MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VCALLVIRT_IMT(OP_GROUP(reg, reg), reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CALL_REG(reg) */
	3, /*  */ /* stmt: OP_VOIDCALL */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_VOIDCALL_REG(reg) */
	3, /*  */ /* freg: OP_FCALL */
	1, /* MB_NTERM_reg,  */ /* freg: OP_FCALL_REG(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_VCALL(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VCALL_REG(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_TRAMPCALL_VTABLE(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CALL_RGCTX(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: OP_FCALL_RGCTX(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_VOIDCALL_RGCTX(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VCALL_RGCTX(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_CALL_REG_RGCTX(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* freg: OP_FCALL_REG_RGCTX(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VOIDCALL_REG_RGCTX(reg, reg) */
	12, /* MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_VCALL_REG_RGCTX(OP_GROUP(reg, reg), reg) */
	3, /*  */ /* stmt: OP_SAVE_LMF */
	3, /*  */ /* stmt: OP_RESTORE_LMF */
	3, /*  */ /* reg: OP_CALL_REG(OP_ICONST) */
	3, /*  */ /* reg: OP_CALL_REG(OP_I8CONST) */
	3, /*  */ /* stmt: OP_VOIDCALL_REG(OP_ICONST) */
	3, /*  */ /* stmt: OP_VOIDCALL_REG(OP_I8CONST) */
	3, /*  */ /* freg: OP_FCALL_REG(OP_ICONST) */
	3, /*  */ /* freg: OP_FCALL_REG(OP_I8CONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_POP(reg) */
	3, /*  */ /* stmt: CEE_POP(OP_ICONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_I1(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_U1(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_I2(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_U2(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_I4(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_U4(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_I(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_POP(CEE_LDIND_REF(base)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_I1(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_U1(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_I2(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_U2(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_I4(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_U4(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* stmt: CEE_POP(CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_JMP */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_THROW(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_RETHROW(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_THROW_OR_NULL(reg, reg) */
	3, /*  */ /* stmt: OP_THROW(CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_CALL_HANDLER */
	3, /*  */ /* stmt: OP_START_HANDLER */
	3, /*  */ /* stmt: OP_ENDFINALLY */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_ENDFILTER(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_CHECK_THIS(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CHECK_THIS_PASSTHROUGH(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_ISINST(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CISINST(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_UNBOXCAST(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_UNBOXCAST_REG(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CASTCLASS(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CCASTCLASS(reg) */
	12, /* MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: CEE_STELEM_REF(OP_GROUP(reg, reg), reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_GROUP(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_NEWARR(reg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LMUL(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LMUL_OVF(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LMUL_OVF_UN(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LDIV(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LDIV_UN(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LREM(lreg, lreg) */
	21, /* MB_NTERM_lreg, MB_NTERM_lreg,  */ /* lreg: OP_LREM_UN(lreg, lreg) */
	24, /* MB_NTERM_lreg, MB_NTERM_reg,  */ /* lreg: OP_LSHL(lreg, reg) */
	24, /* MB_NTERM_lreg, MB_NTERM_reg,  */ /* lreg: OP_LSHR(lreg, reg) */
	24, /* MB_NTERM_lreg, MB_NTERM_reg,  */ /* lreg: OP_LSHR_UN(lreg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_GETCHR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_GETCHR(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_STR_CHAR_ADDR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_STRLEN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_GETTYPE(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_GETHASHCODE(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_LDLEN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_ARRAY_RANK(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_CHECK_ARRAY_TYPE(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_LDELEMA(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_LDELEMA(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_REFANYVAL(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_REFANYVAL_REG(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_REFANYTYPE(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: CEE_MKREFANY(OP_GROUP(reg, OP_ICONST), reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: CEE_MKREFANY(OP_GROUP(reg, OP_I8CONST), reg) */
	27, /* MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: OP_MKREFANY_REGS(OP_GROUP(reg, reg), OP_GROUP(reg, reg)) */
	4, /* MB_NTERM_base,  */ /* freg: CEE_LDIND_R4(base) */
	4, /* MB_NTERM_base,  */ /* freg: CEE_LDIND_R8(base) */
	32, /* MB_NTERM_base, MB_NTERM_freg,  */ /* stmt: CEE_STIND_R4(base, freg) */
	32, /* MB_NTERM_base, MB_NTERM_freg,  */ /* stmt: CEE_STIND_R8(base, freg) */
	3, /*  */ /* freg: OP_R4CONST */
	3, /*  */ /* freg: OP_R8CONST */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* freg: OP_FADD(freg, freg) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* freg: OP_FSUB(freg, freg) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* freg: OP_FMUL(freg, freg) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* freg: OP_FDIV(freg, freg) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* freg: OP_FREM(freg, freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_FNEG(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_CKFINITE(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_SIN(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_COS(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_ABS(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_TAN(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_ATAN(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_SQRT(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_I4(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_U4(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_I4(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_U4(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_I8(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_U8(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_I(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_OVF_U(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_I(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_U(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_I2(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_U2(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_I1(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_U1(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_FCONV_TO_R8(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_FCONV_TO_R4(freg) */
	1, /* MB_NTERM_reg,  */ /* freg: CEE_CONV_R4(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: CEE_CONV_R8(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: CEE_CONV_R_UN(reg) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* fpcflags: OP_COMPARE(freg, freg) */
	38, /* MB_NTERM_freg,  */ /* stmt: CEE_POP(freg) */
	3, /*  */ /* reg: OP_I8CONST */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDIND_I8(base) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I8(base, reg) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_REF(base, OP_I8CONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I(base, OP_I8CONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I8(base, OP_I8CONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_REF(base, OP_ICONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I(base, OP_ICONST) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I8(base, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* stmt: CEE_STIND_I8(reg, reg) */
	4, /* MB_NTERM_base,  */ /* base: OP_LADD(base, OP_ICONST) */
	3, /*  */ /* base: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
	3, /*  */ /* base: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LADD(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LADD(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LADD(reg, OP_ICONST) */
	3, /*  */ /* reg: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	3, /*  */ /* reg: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSUB(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LNEG(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LNOT(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LAND(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LOR(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LXOR(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LADD_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LADD_OVF_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSUB_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSUB_OVF_UN(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LONG_SHRUN_32(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHL(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHL(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHR(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHR_UN(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHR_UN(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_I8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_U8(reg) */
	3, /*  */ /* i8con: CEE_CONV_U8(OP_ICONST) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_I8(freg) */
	38, /* MB_NTERM_freg,  */ /* reg: OP_FCONV_TO_U8(freg) */
	40, /* MB_NTERM_i8con,  */ /* reg: OP_LCONV_TO_I4(i8con) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_I4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_U4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_U8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_I8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_U(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_I(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_I1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_U1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_I2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_U2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I1_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U1(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U1_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I2_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U2(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U2_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I4_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U4(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U4_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I8_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_U8_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCONV_TO_OVF_I(reg) */
	42, /* MB_NTERM_lreg,  */ /* freg: OP_LCONV_TO_R_UN(lreg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I8_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U8_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_I_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U8(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_CONV_OVF_U(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCALLVIRT(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LCALLVIRT_IMT(reg, reg) */
	3, /*  */ /* reg: OP_LCALL */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCALL_RGCTX(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LCALL_REG(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LCALL_REG_RGCTX(reg, reg) */
	3, /*  */ /* reg: OP_LCALL_REG(OP_ICONST) */
	3, /*  */ /* reg: OP_LCALL_REG(OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I4(OP_REGVAR, reg) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base)) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* cflags: OP_LCOMPARE(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), reg) */
	3, /*  */ /* cflags: OP_LCOMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* cflags: OP_LCOMPARE(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* cflags: OP_LCOMPARE(reg, OP_I8CONST) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BNE_UN(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BEQ(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BLT(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BLT_UN(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BGT(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BGT_UN(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BGE(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BGE_UN(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BLE(cflags) */
	16, /* MB_NTERM_cflags,  */ /* stmt: CEE_BLE_UN(cflags) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_AND(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_AND(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_OR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_OR(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_XOR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_XOR(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_NEG(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_NOT(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_ADD(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_ADD(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_ADD_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_ADD_OVF_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SUB(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_SUB(reg, CEE_LDIND_I4(OP_REGVAR)) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_SUB(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SUB_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SUB_OVF_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SHL(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_SHL(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SHR(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_SHR(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_SHR_UN(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_SHR_UN(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_MUL(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: CEE_MUL(reg, OP_ICONST) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_MUL_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_MUL_OVF_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_DIV(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_DIV_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_REM(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_REM_UN(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_IMUL_IMM(reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LMUL_IMM(reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* c32flags: OP_COMPARE(reg, reg) */
	3, /*  */ /* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
	3, /*  */ /* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), reg) */
	3, /*  */ /* c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* c32flags: OP_COMPARE(reg, OP_ICONST) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BNE_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BEQ(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BLT(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BLT_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BGT(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BGT_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BGE(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BGE_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BLE(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* stmt: CEE_BLE_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* reg: OP_CEQ(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* reg: OP_CLT(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* reg: OP_CLT_UN(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* reg: OP_CGT(c32flags) */
	44, /* MB_NTERM_c32flags,  */ /* reg: OP_CGT_UN(c32flags) */
	3, /*  */ /* reg: CEE_LDIND_I8(OP_REGVAR) */
	1, /* MB_NTERM_reg,  */ /* stmt: CEE_STIND_I8(OP_REGVAR, reg) */
	3, /*  */ /* reg: CEE_LDIND_I1(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_I2(OP_REGVAR) */
	3, /*  */ /* stmt: OP_START_HANDLER */
	3, /*  */ /* stmt: OP_ENDFINALLY */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_ENDFILTER(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: OP_LCONV_TO_R_UN(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: OP_LCONV_TO_R8(reg) */
	1, /* MB_NTERM_reg,  */ /* freg: OP_LCONV_TO_R4(reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* cflags: OP_COMPARE(CEE_LDIND_I4(base), reg) */
	4, /* MB_NTERM_base,  */ /* cflags: OP_COMPARE(CEE_LDIND_I4(base), OP_ICONST) */
	46, /* MB_NTERM_reg, MB_NTERM_base,  */ /* cflags: OP_COMPARE(reg, CEE_LDIND_I4(base)) */
	49, /* MB_NTERM_base, MB_NTERM_cflags,  */ /* stmt: CEE_STIND_I1(base, OP_CEQ(cflags)) */
	3, /*  */ /* reg: OP_LOCALLOC(OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LOCALLOC(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_SETRET(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_SETRET(reg) */
	3, /*  */ /* reg: CEE_LDIND_REF(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_I(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_I4(OP_REGVAR) */
	3, /*  */ /* reg: CEE_LDIND_U4(OP_REGVAR) */
	3, /*  */ /* stmt: OP_SETRET(CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_SETRET(CEE_LDIND_I4(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_SETRET(CEE_LDIND_U4(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_SETRET(CEE_LDIND_I(OP_REGVAR)) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_SETRET(freg) */
	3, /*  */ /* stmt: OP_SETRET(OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_OUTARG(reg) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_OUTARG_REG(reg) */
	3, /*  */ /* stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_OUTARG(CEE_LDIND_I4(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_OUTARG(CEE_LDIND_U4(OP_REGVAR)) */
	3, /*  */ /* stmt: OP_OUTARG(CEE_LDIND_I(OP_REGVAR)) */
	4, /* MB_NTERM_base,  */ /* stmt: OP_OUTARG(CEE_LDIND_I(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: OP_OUTARG(CEE_LDIND_REF(base)) */
	3, /*  */ /* stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR)) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_OUTARG(CEE_LDOBJ(reg)) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_OUTARG(freg) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_OUTARG_R4(freg) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_OUTARG_R8(freg) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_AMD64_OUTARG_XMMREG_R4(freg) */
	38, /* MB_NTERM_freg,  */ /* stmt: OP_AMD64_OUTARG_XMMREG_R8(freg) */
	4, /* MB_NTERM_base,  */ /* stmt: OP_OUTARG_VT(CEE_LDOBJ(base)) */
	3, /*  */ /* stmt: OP_OUTARG_VT(OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* stmt: OP_OUTARG_VT(reg) */
	3, /*  */ /* stmt: OP_AMD64_OUTARG_ALIGN_STACK */
	3, /*  */ /* stmt: OP_AMD64_SAVE_SP_TO_LMF */
	4, /* MB_NTERM_base,  */ /* base: OP_INARG_VT(base) */
	4, /* MB_NTERM_base,  */ /* reg: OP_LDADDR(OP_INARG_VT(base)) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDOBJ(OP_INARG_VT(base)) */
	3, /*  */ /* reg: OP_LDADDR(OP_REGOFFSET) */
	3, /*  */ /* reg: CEE_LDOBJ(OP_REGOFFSET) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: CEE_LDELEMA(reg, reg) */
	38, /* MB_NTERM_freg,  */ /* stmt: CEE_STIND_R8(OP_REGVAR, freg) */
	9, /* MB_NTERM_base, MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(base, CEE_ADD(CEE_LDIND_I4(base), OP_ICONST)) */
	9, /* MB_NTERM_base, MB_NTERM_base,  */ /* stmt: CEE_STIND_I4(base, CEE_SUB(CEE_LDIND_I4(base), OP_ICONST)) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_SUB(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, CEE_ADD(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)) */
	3, /*  */ /* reg: CEE_LDIND_I2(OP_REGVAR) */
	3, /*  */ /* stmt: CEE_STIND_I8(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST) */
	3, /*  */ /* stmt: CEE_STIND_I8(OP_REGVAR, OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_I4(OP_REGVAR, OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_I2(OP_REGVAR, OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_I1(OP_REGVAR, OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_REF(OP_REGVAR, OP_I8CONST) */
	3, /*  */ /* stmt: CEE_STIND_I(OP_REGVAR, OP_I8CONST) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* reg: OP_CEQ(OP_COMPARE(freg, freg)) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* reg: OP_CLT(OP_COMPARE(freg, freg)) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* reg: OP_CGT(OP_COMPARE(freg, freg)) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* reg: OP_CLT_UN(OP_COMPARE(freg, freg)) */
	35, /* MB_NTERM_freg, MB_NTERM_freg,  */ /* reg: OP_CGT_UN(OP_COMPARE(freg, freg)) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BNE_UN(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BEQ(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BLT(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BLT_UN(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BGT(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BGT_UN(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BGE(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BGE_UN(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BLE(fpcflags) */
	52, /* MB_NTERM_fpcflags,  */ /* stmt: CEE_BLE_UN(fpcflags) */
	38, /* MB_NTERM_freg,  */ /* stmt: CEE_POP(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_FCONV_TO_R8(freg) */
	38, /* MB_NTERM_freg,  */ /* freg: OP_FCONV_TO_R4(freg) */
	46, /* MB_NTERM_reg, MB_NTERM_base,  */ /* reg: CEE_ADD(reg, CEE_LDIND_I4(base)) */
	46, /* MB_NTERM_reg, MB_NTERM_base,  */ /* reg: CEE_SUB(reg, CEE_LDIND_I4(base)) */
	46, /* MB_NTERM_reg, MB_NTERM_base,  */ /* reg: CEE_MUL(reg, CEE_LDIND_I4(base)) */
	4, /* MB_NTERM_base,  */ /* reg: OP_VTARG_ADDR(base) */
	4, /* MB_NTERM_base,  */ /* reg: OP_LDADDR(OP_VTARG_ADDR(base)) */
	4, /* MB_NTERM_base,  */ /* reg: CEE_LDOBJ(OP_VTARG_ADDR(base)) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHL(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHR(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LSHR_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMUL(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LDIV(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LDIV_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LREM(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LREM_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMUL_OVF(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMUL_OVF_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_IMIN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_IMIN_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_IMAX(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_IMAX_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMIN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMIN_UN(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMAX(reg, reg) */
	18, /* MB_NTERM_reg, MB_NTERM_reg,  */ /* reg: OP_LMAX_UN(reg, reg) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LMUL(reg, OP_I8CONST) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHL(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHR(reg, OP_ICONST) */
	1, /* MB_NTERM_reg,  */ /* reg: OP_LSHR_UN(reg, OP_ICONST) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_ADD_NEW_I4(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_ADD_NEW_I8(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_ADD_I4(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_ADD_I8(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_EXCHANGE_I4(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_EXCHANGE_I8(base, reg) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* reg: OP_ATOMIC_CAS_IMM_I4(base, reg) */
	4, /* MB_NTERM_base,  */ /* reg: OP_CALL_REG(CEE_LDIND_I(base)) */
	4, /* MB_NTERM_base,  */ /* freg: OP_FCALL_REG(CEE_LDIND_I(base)) */
	4, /* MB_NTERM_base,  */ /* reg: OP_LCALL_REG(CEE_LDIND_I(base)) */
	4, /* MB_NTERM_base,  */ /* stmt: OP_VOIDCALL_REG(CEE_LDIND_I(base)) */
	6, /* MB_NTERM_base, MB_NTERM_reg,  */ /* stmt: OP_VCALL_REG(CEE_LDIND_I(base), reg) */
};

inline static guint16
/* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), OP_ICONST) */
mono_burg_cost_342 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));

	return 0;

}

inline static guint16
/* cflags: OP_LCOMPARE(reg, OP_ICONST) */
mono_burg_cost_343 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));

	return 0;

}

inline static guint16
/* cflags: OP_LCOMPARE(reg, OP_I8CONST) */
mono_burg_cost_344 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_l));

	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I4(base, CEE_ADD(CEE_LDIND_I4(base), OP_ICONST)) */
mono_burg_cost_464 (MBState *state, MBCOST_DATA *data)
{

	MBTREE_TYPE *t1 = state->right->left->left->tree;
	MBTREE_TYPE *t2 = state->left->tree;
	MBCOND (t1->inst_basereg == t2->inst_basereg &&
		t1->inst_offset == t2->inst_offset);
	return 2;

}

inline static guint16
/* stmt: CEE_STIND_I4(base, CEE_SUB(CEE_LDIND_I4(base), OP_ICONST)) */
mono_burg_cost_465 (MBState *state, MBCOST_DATA *data)
{

	MBTREE_TYPE *t1 = state->right->left->left->tree;
	MBTREE_TYPE *t2 = state->left->tree;
	MBCOND (t1->inst_basereg == t2->inst_basereg &&
		t1->inst_offset == t2->inst_offset);
	return 2;

}

inline static guint16
/* stmt: CEE_STIND_I8(OP_REGVAR, OP_ICONST) */
mono_burg_cost_470 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
mono_burg_cost_471 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST) */
mono_burg_cost_472 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST) */
mono_burg_cost_473 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST) */
mono_burg_cost_474 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST) */
mono_burg_cost_475 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I8(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_476 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I4(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_477 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I2(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_478 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I1(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_479 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_REF(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_480 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* stmt: CEE_STIND_I(OP_REGVAR, OP_I8CONST) */
mono_burg_cost_481 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (!state->right->tree->inst_c0);
	
	return 0;

}

inline static guint16
/* reg: OP_LMUL(reg, OP_I8CONST) */
mono_burg_cost_524 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));
	return 0;

}

inline static guint16
/* reg: OP_LSHL(reg, OP_ICONST) */
mono_burg_cost_525 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));
	return 0;

}

inline static guint16
/* reg: OP_LSHR(reg, OP_ICONST) */
mono_burg_cost_526 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));
	return 0;

}

inline static guint16
/* reg: OP_LSHR_UN(reg, OP_ICONST) */
mono_burg_cost_527 (MBState *state, MBCOST_DATA *data)
{

	MBCOND (mono_arch_is_inst_imm (state->right->tree->inst_c0));
	return 0;

}

void mono_burg_emit (int ern, MBState *state, MBTREE_TYPE *tree, MBCGEN_TYPE *s)
 {
	switch (ern) {/* base: reg */
	case 1:
	{

	/* we create a new MonoInst in chain rules and set state->tree to it
	 * since a MBState (and hence state->tree) is reused in chain rules and
	 * normal rules.
	 */
	MonoInst *inst;
    MONO_INST_NEW (s, inst, OP_NOP); /* The opcode doesn't matter */
	inst->inst_basereg = state->reg1;
	inst->inst_offset = 0;
	state->tree = inst;
	//g_print ("mybase: (assigned R%d from R%d - R%d)\n", state->reg1, tree->dreg, state->reg2);
	//tree->inst_offset = 0;
	//tree->inst_basereg = state->reg1;

	}
	return;
/* base: CEE_LDIND_I(OP_REGVAR) */
	case 2:
	{

	tree->inst_offset = 0;
	tree->inst_basereg = state->left->tree->dreg;

	}
	return;
/* base: OP_REGOFFSET */
	case 3:
	{

	/* nothing to do */

	}
	return;
/* base: OP_LDADDR(OP_REGOFFSET) */
	case 4:
	case 5:
	{

	tree->inst_offset = state->left->tree->inst_offset;
	tree->inst_basereg = state->left->tree->inst_basereg;

	}
	return;
/* base: CEE_ADD(base, OP_ICONST) */
	case 6:
	case 257:
	{

	tree->inst_offset = state->left->tree->inst_offset + state->right->tree->inst_c0;
	tree->inst_basereg = state->left->tree->inst_basereg;

	}
	return;
/* base: CEE_ADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
	case 7:
	case 8:
	case 258:
	case 259:
	{

	tree->inst_offset = state->right->tree->inst_c0;
	tree->inst_basereg = state->left->left->tree->dreg;

	}
	return;
/* reg: OP_ICONST */
	case 9:
	{

	tree->dreg = state->reg1;
     	MONO_EMIT_NEW_ICONST (s, state->reg1, tree->inst_c0);

	}
	return;
/* reg: OP_AOTCONST */
	case 10:
	{

     	MONO_EMIT_NEW_AOTCONST (s, state->reg1, tree->inst_p0, tree->inst_c1);

	}
	return;
/* reg: OP_LOAD_GOTADDR */
	case 11:
	case 105:
	case 109:
	case 121:
	case 329:
	{

	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I(OP_REGVAR, OP_LOAD_GOTADDR) */
	case 12:
	{

	tree->opcode = OP_LOAD_GOTADDR;
	tree->dreg = state->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_GOT_ENTRY(reg, OP_PATCH_INFO) */
	case 13:
	{

	tree->inst_basereg = state->left->tree->dreg;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_GOT_ENTRY(CEE_LDIND_I(OP_REGVAR), OP_PATCH_INFO) */
	case 14:
	{

	tree->inst_basereg = state->left->left->tree->dreg;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGVAR)) */
	case 15:
	case 16:
	case 17:
	case 18:
	case 19:
	case 103:
	{


	}
	return;
/* stmt: OP_MEMORY_BARRIER */
	case 20:
	case 102:
	case 104:
	case 107:
	case 119:
	case 134:
	case 135:
	case 160:
	case 165:
	case 166:
	case 167:
	case 345:
	case 346:
	case 347:
	case 348:
	case 349:
	case 350:
	case 351:
	case 352:
	case 353:
	case 354:
	case 413:
	case 414:
	case 456:
	{

	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_LDIND_I(base) */
	case 21:
	case 22:
	case 23:
	case 24:
	case 25:
	case 26:
	case 27:
	case 28:
	{

	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, ldind_to_load_membase (tree->opcode),
				   state->reg1, state->left->tree->inst_basereg, state->left->tree->inst_offset);

	}
	return;
/* reg: OP_LDADDR(OP_REGOFFSET) */
	case 29:
	case 30:
	{

	/* use LEA */
	tree->opcode = OP_MOVE;
	tree->sreg1 = state->left->tree->inst_basereg;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);
	if (state->left->tree->inst_offset)
		MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, tree->dreg, tree->dreg, state->left->tree->inst_offset);

	}
	return;
/* reg: CEE_LDOBJ(reg) */
	case 31:
	case 32:
	case 33:
	case 86:
	case 290:
	case 291:
	case 292:
	case 293:
	case 314:
	case 315:
	case 316:
	case 317:
	{

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_LDIND_U1(OP_REGVAR) */
	case 34:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->tree->dreg, 0xff);

	}
	return;
/* reg: CEE_LDIND_U2(OP_REGVAR) */
	case 35:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->tree->dreg, 0xffff);

	}
	return;
/* stmt: CEE_STIND_REF(base, CEE_LDIND_REF(OP_REGVAR)) */
	case 36:
	case 37:
	case 38:
	case 39:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STORE_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->left->tree->dreg);

	}
	return;
/* stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR)) */
	case 40:
	case 41:
	case 42:
	case 43:
	case 44:
	case 337:
	{

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->left->tree->dreg, state->right->left->tree->dreg);

	}
	return;
/* stmt: CEE_STIND_I(base, reg) */
	case 45:
	case 46:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STORE_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);

	}
	return;
/* stmt: CEE_STIND_I(base, OP_ICONST) */
	case 47:
	case 48:
	{

	MONO_EMIT_STORE_MEMBASE_IMM (s, tree, OP_STORE_MEMBASE_IMM, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_LDIND_REF(OP_REGVAR) */
	case 49:
	case 50:
	case 51:
	case 52:
	{

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->tree->dreg);

	}
	return;
/* stmt: CEE_STIND_I4(base, reg) */
	case 53:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STOREI4_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);

	}
	return;
/* stmt: CEE_STIND_I4(base, CEE_LDIND_I4(OP_REGVAR)) */
	case 54:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STOREI4_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->left->tree->dreg);

	}
	return;
/* stmt: CEE_STIND_I4(base, OP_ICONST) */
	case 55:
	{

	MONO_EMIT_STORE_MEMBASE_IMM (s, tree, OP_STOREI4_MEMBASE_IMM, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset, state->right->tree->inst_c0);

	}
	return;
/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base)) */
	case 56:
	case 338:
	{

	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADI4_MEMBASE, state->left->tree->dreg, 
				   state->right->left->tree->inst_basereg, 
				   state->right->left->tree->inst_offset);

	}
	return;
/* stmt: CEE_STIND_I1(base, reg) */
	case 57:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STOREI1_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);

	}
	return;
/* stmt: CEE_STIND_I1(base, OP_ICONST) */
	case 58:
	{

	MONO_EMIT_STORE_MEMBASE_IMM (s, tree, OP_STOREI1_MEMBASE_IMM, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset, state->right->tree->inst_c0);

	}
	return;
/* stmt: CEE_STIND_I2(base, reg) */
	case 59:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STOREI2_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);

	}
	return;
/* stmt: CEE_STIND_I2(base, OP_ICONST) */
	case 60:
	{

	MONO_EMIT_STORE_MEMBASE_IMM (s, tree, OP_STOREI2_MEMBASE_IMM, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset, state->right->tree->inst_c0);

	}
	return;
/* stmt: CEE_STIND_I2(OP_REGVAR, reg) */
	case 61:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->left->tree->dreg, state->right->reg1, 0xffff);

	}
	return;
/* stmt: CEE_STIND_I1(OP_REGVAR, reg) */
	case 62:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->left->tree->dreg, state->right->reg1, 0xff);

	}
	return;
/* stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_U1(OP_REGVAR)) */
	case 63:
	case 64:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->left->tree->dreg, state->right->left->tree->dreg, 0xff);

	}
	return;
/* stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_U2(OP_REGVAR)) */
	case 65:
	case 66:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->left->tree->dreg, state->right->left->tree->dreg, 0xffff);

	}
	return;
/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
	case 67:
	case 68:
	case 69:
	case 70:
	case 71:
	{

	tree->inst_c0 = state->right->tree->inst_c0;
	if (tree->opcode == CEE_STIND_I2)
		tree->inst_c0 &= 0xffff;
	if (tree->opcode == CEE_STIND_I1)
		tree->inst_c0 &= 0xff;
	
	tree->opcode = OP_ICONST;
	tree->dreg = state->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I4(OP_REGVAR, reg) */
	case 72:
	case 73:
	case 74:
	case 336:
	{

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->left->tree->dreg, state->right->reg1);

	}
	return;
/* stmt: OP_MEMCPY(base, base) */
	case 75:
	{

	int size = tree->backend.memcpy_args->size;
	int align = tree->backend.memcpy_args->align;
	if (size > 0)
		mini_emit_memcpy (s, state->left->tree->inst_basereg, state->left->tree->inst_offset,
				  state->right->tree->inst_basereg, state->right->tree->inst_offset, size, align);

	}
	return;
/* stmt: OP_MEMSET(base) */
	case 76:
	{

	int size = tree->backend.memcpy_args->size;
	int align = tree->backend.memcpy_args->align;
	if (size > 0)
		mini_emit_memset (s, state->left->tree->inst_basereg, state->left->tree->inst_offset, size, tree->inst_imm, align);

	}
	return;
/* reg: OP_LDELEMA2D(reg, OP_GROUP(reg, reg)) */
	case 77:
	{

	int bounds_reg = mono_regstate_next_int (s->rs);
	int add_reg = mono_regstate_next_int (s->rs);
	int mult_reg = mono_regstate_next_int (s->rs);
	int mult2_reg = mono_regstate_next_int (s->rs);
	int low1_reg = mono_regstate_next_int (s->rs);
	int low2_reg = mono_regstate_next_int (s->rs);
	int high1_reg = mono_regstate_next_int (s->rs);
	int high2_reg = mono_regstate_next_int (s->rs);
	int realidx1_reg = mono_regstate_next_int (s->rs);
	int realidx2_reg = mono_regstate_next_int (s->rs);
	int sum_reg = mono_regstate_next_int (s->rs);
	int index1, index2;
	guint32 size = mono_class_array_element_size (tree->klass);
	
	index1 = state->right->left->reg1;
	index2 = state->right->right->reg1;

	/* range checking */
	MONO_EMIT_NEW_LOAD_MEMBASE (s, bounds_reg, 
				       state->left->reg1, G_STRUCT_OFFSET (MonoArray, bounds));

	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, low1_reg, 
				       bounds_reg, G_STRUCT_OFFSET (MonoArrayBounds, lower_bound));
	MONO_EMIT_NEW_BIALU (s, CEE_SUB, realidx1_reg, index1, low1_reg);
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, high1_reg, 
				       bounds_reg, G_STRUCT_OFFSET (MonoArrayBounds, length));
	MONO_EMIT_NEW_COMPARE_EXC (s, LE_UN, high1_reg, realidx1_reg, "IndexOutOfRangeException");

	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, low2_reg, 
				       bounds_reg, sizeof (MonoArrayBounds) + G_STRUCT_OFFSET (MonoArrayBounds, lower_bound));
	MONO_EMIT_NEW_BIALU (s, CEE_SUB, realidx2_reg, index2, low2_reg);
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, high2_reg, 
				       bounds_reg, sizeof (MonoArrayBounds) + G_STRUCT_OFFSET (MonoArrayBounds, length));
	MONO_EMIT_NEW_COMPARE_EXC (s, LE_UN, high2_reg, realidx2_reg, "IndexOutOfRangeException");

	MONO_EMIT_NEW_BIALU (s, CEE_MUL, mult_reg, high2_reg, realidx1_reg);
	MONO_EMIT_NEW_BIALU (s, CEE_ADD, sum_reg, mult_reg, realidx2_reg);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_MUL_IMM, mult2_reg, sum_reg, size);
	MONO_EMIT_NEW_BIALU (s, CEE_ADD, add_reg, mult2_reg, state->left->reg1);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, state->reg1, add_reg, G_STRUCT_OFFSET (MonoArray, vector));

	}
	return;
/* reg: CEE_CONV_I1(reg) */
	case 78:
	case 79:
	case 80:
	case 218:
	case 219:
	case 220:
	case 221:
	case 222:
	case 223:
	case 224:
	case 225:
	case 226:
	case 227:
	case 228:
	case 229:
	case 230:
	case 231:
	case 232:
	case 233:
	case 234:
	case 235:
	case 236:
	case 237:
	case 238:
	case 239:
	case 241:
	case 242:
	case 243:
	case 244:
	case 285:
	case 286:
	case 318:
	{

	MONO_EMIT_UNALU (s, tree, tree->opcode, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_U1(reg) */
	case 81:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->reg1, 0xff);

	}
	return;
/* reg: CEE_CONV_U2(reg) */
	case 82:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->reg1, 0xffff);

	}
	return;
/* reg: CEE_CONV_U4(reg) */
	case 83:
	case 84:
	case 85:
	{

	if (sizeof (void *) == 8)
		MONO_EMIT_UNALU (s, tree, tree->opcode, state->reg1, state->left->reg1);
	else
		MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_U4(reg) */
	case 87:
	case 88:
	{

	if (sizeof (void *) == 8) {
		MONO_EMIT_UNALU (s, tree, tree->opcode, state->reg1, state->left->reg1);
	} else {	
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, 0, "OverflowException");
		MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);
	}

	}
	return;
/* reg: CEE_CONV_OVF_I1(reg) */
	case 89:
	{
     
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, GT, state->left->reg1, 127, "OverflowException");
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, LT, state->left->reg1, -128, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I1, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_I1_UN(reg) */
	case 90:
	{

	/* probe values between 0 to 127 */
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 127, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I1, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_U1(reg) */
	case 91:
	case 92:
	case 300:
	case 301:
	{

	/* probe value to be within 0 to 255 */
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 255, "OverflowException");
	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->reg1, 0xff);

	}
	return;
/* reg: CEE_CONV_OVF_I2(reg) */
	case 93:
	{
	
	/* Probe value to be within -32768 and 32767 */
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, GT, state->left->reg1, 32767, "OverflowException");
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, LT, state->left->reg1, -32768, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I2, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_I2_UN(reg) */
	case 94:
	{

	/* Convert uint value into short, value within 0 and 32767 */
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 32767, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I2, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_U2(reg) */
	case 95:
	case 96:
	{

	/* Probe value to be within 0 and 65535 */
	MONO_EMIT_NEW_ICOMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 0xffff, "OverflowException");
	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->reg1, 0xffff);

	}
	return;
/* reg: OP_CEQ(cflags) */
	case 97:
	case 98:
	case 99:
	case 100:
	case 101:
	{
	
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_SWITCH(reg) */
	case 106:
	{

	MonoInst *label;
	int offset_reg = mono_regstate_next_int (s->rs);
	int target_reg = mono_regstate_next_int (s->rs);
	int n = GPOINTER_TO_INT (tree->klass);
	
	MONO_NEW_LABEL (s, label);

	if (sizeof (gpointer) == 8) {
		MONO_EMIT_NEW_BIALU_IMM (s, OP_ICOMPARE_IMM, -1, state->left->reg1, n);
		MONO_EMIT_NEW_BRANCH_BLOCK (s, OP_IBGE_UN, tree->inst_many_bb [n]);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SHL_IMM, offset_reg, state->left->reg1, 3);
		/* The upper word might not be zero, and we add it to a 64 bit address later */
		MONO_EMIT_NEW_UNALU (s, CEE_CONV_U8, offset_reg, offset_reg);
	} else {
		MONO_EMIT_NEW_BIALU_IMM (s, OP_COMPARE_IMM, -1, state->left->reg1, n);
		MONO_EMIT_NEW_BRANCH_BLOCK (s, CEE_BGE_UN, tree->inst_many_bb [n]);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SHL_IMM, offset_reg, state->left->reg1, 2);
	}
	mono_bblock_add_inst (s->cbb, label);
	if (s->compile_aot) {
		int table_reg = mono_regstate_next_int (s->rs);
		int sum_reg = mono_regstate_next_int (s->rs);
		MonoJumpInfoBBTable *table;

		table = mono_mempool_alloc (s->mempool, sizeof (MonoJumpInfoBBTable));
		table->table = tree->inst_many_bb;
		table->table_size = n;

		MONO_EMIT_NEW_AOTCONST (s, table_reg, table, MONO_PATCH_INFO_SWITCH);

		MONO_EMIT_NEW_BIALU (s, CEE_ADD, sum_reg, table_reg, offset_reg);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, target_reg, sum_reg, 0);
	} else {
		mono_create_jump_table (s, label, tree->inst_many_bb, n);

		 /* the backend must patch the address. we use 0xf0f0f0f0 to avoid the usage 
		  * of special (short) opcodes on x86 */
		if (sizeof (gpointer) == 8)
			MONO_EMIT_NEW_LOAD_MEMBASE (s, target_reg, offset_reg, (gint64)0xf0f0f0f0f0f0f0f1LL);
		else
			MONO_EMIT_NEW_LOAD_MEMBASE (s, target_reg, offset_reg, 0xf0f0f0f0);
	}
	MONO_EMIT_UNALU (s, tree, OP_BR_REG, -1, target_reg);

	}
	return;
/* stmt: OP_ARGLIST(reg) */
	case 108:
	case 120:
	case 415:
	{

	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CALLVIRT(reg) */
	case 110:
	{

	mini_emit_virtual_call (s, state, tree, OP_CALL, OP_CALL_MEMBASE);

	}
	return;
/* stmt: OP_VOIDCALLVIRT(reg) */
	case 111:
	{

	mini_emit_virtual_call (s, state, tree, OP_VOIDCALL, OP_VOIDCALL_MEMBASE);

	}
	return;
/* freg: OP_FCALLVIRT(reg) */
	case 112:
	{

	mini_emit_virtual_call (s, state, tree, OP_FCALL, OP_FCALL_MEMBASE);

	}
	return;
/* stmt: OP_VCALLVIRT(reg, reg) */
	case 113:
	{

	mini_emit_virtual_call (s, state, tree, OP_VCALL, OP_VCALL_MEMBASE);

	}
	return;
/* reg: OP_CALLVIRT_IMT(reg, reg) */
	case 114:
	{

	tree->opcode = OP_CALLVIRT;
	mini_emit_virtual_imt_call (s, state, tree, OP_CALL, OP_CALL_MEMBASE, state->right->reg1);

	}
	return;
/* stmt: OP_VOIDCALLVIRT_IMT(reg, reg) */
	case 115:
	{

	tree->opcode = OP_VOIDCALLVIRT;
	mini_emit_virtual_imt_call (s, state, tree, OP_VOIDCALL, OP_VOIDCALL_MEMBASE, state->right->reg1);

	}
	return;
/* freg: OP_FCALLVIRT_IMT(reg, reg) */
	case 116:
	{

	tree->opcode = OP_FCALLVIRT;
	mini_emit_virtual_imt_call (s, state, tree, OP_FCALL, OP_FCALL_MEMBASE, state->right->reg1);

	}
	return;
/* stmt: OP_VCALLVIRT_IMT(OP_GROUP(reg, reg), reg) */
	case 117:
	{

	int imt_reg = state->left->right->reg1;
	tree->opcode = OP_VCALLVIRT;
	tree->inst_left = tree->inst_left->inst_left;
	state->left = state->left->left;
	mini_emit_virtual_imt_call (s, state, tree, OP_VCALL, OP_VCALL_MEMBASE, imt_reg);

	}
	return;
/* reg: OP_CALL_REG(reg) */
	case 118:
	case 122:
	case 169:
	case 331:
	case 386:
	case 387:
	case 416:
	case 417:
	case 418:
	{

	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VCALL(reg) */
	case 123:
	{

	mono_arch_emit_this_vret_args (s, (MonoCallInst*)tree, -1, -1, state->left->reg1);
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VCALL_REG(reg, reg) */
	case 124:
	{

	mono_arch_emit_this_vret_args (s, (MonoCallInst*)tree, -1, -1, state->right->reg1);
	
	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_TRAMPCALL_VTABLE(reg) */
	case 125:
	{

	#ifdef MONO_ARCH_VTABLE_REG
		MonoCallInst *call = (MonoCallInst*)tree;
		int vtable_reg = mono_regstate_next_int (s->rs);

		MONO_EMIT_NEW_UNALU (s, OP_MOVE, vtable_reg, state->left->reg1);

		call->inst.opcode = OP_VOIDCALL;

		mono_call_inst_add_outarg_reg (s, call, vtable_reg, MONO_ARCH_VTABLE_REG, FALSE);

		mono_bblock_add_inst (s->cbb, &call->inst);
	#else
		g_assert_not_reached ();
	#endif

	}
	return;
/* reg: OP_CALL_RGCTX(reg) */
	case 126:
	{

	emit_rgctx_argument (s, tree, state->left->reg1, OP_CALL);

	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* freg: OP_FCALL_RGCTX(reg) */
	case 127:
	{

	emit_rgctx_argument (s, tree, state->left->reg1, OP_FCALL);

	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VOIDCALL_RGCTX(reg) */
	case 128:
	{

	emit_rgctx_argument (s, tree, state->left->reg1, OP_VOIDCALL);

	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VCALL_RGCTX(reg, reg) */
	case 129:
	{

	emit_rgctx_argument (s, tree, state->right->reg1, OP_VCALL);

	mono_arch_emit_this_vret_args (s, (MonoCallInst*)tree, -1, -1, state->left->reg1);
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CALL_REG_RGCTX(reg, reg) */
	case 130:
	{

	emit_rgctx_argument (s, tree, state->right->reg1, OP_CALL_REG);

	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* freg: OP_FCALL_REG_RGCTX(reg, reg) */
	case 131:
	{

	emit_rgctx_argument (s, tree, state->right->reg1, OP_FCALL_REG);

	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VOIDCALL_REG_RGCTX(reg, reg) */
	case 132:
	{

	emit_rgctx_argument (s, tree, state->right->reg1, OP_VOIDCALL_REG);

	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VCALL_REG_RGCTX(OP_GROUP(reg, reg), reg) */
	case 133:
	{

	emit_rgctx_argument (s, tree, state->left->right->reg1, OP_VCALL_REG);

	tree->inst_left = tree->inst_left->inst_left;
	state->left = state->left->left;

	mono_arch_emit_this_vret_args (s, (MonoCallInst*)tree, -1, -1, state->right->reg1);

	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CALL_REG(OP_ICONST) */
	case 136:
	case 137:
	{

	tree->opcode = OP_CALL;
	((MonoCallInst*)tree)->fptr = state->left->tree->inst_p0;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VOIDCALL_REG(OP_ICONST) */
	case 138:
	case 139:
	{

	tree->opcode = OP_VOIDCALL;
	((MonoCallInst*)tree)->fptr = state->left->tree->inst_p0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* freg: OP_FCALL_REG(OP_ICONST) */
	case 140:
	case 141:
	{

	tree->opcode = OP_FCALL;
	((MonoCallInst*)tree)->fptr = state->left->tree->inst_p0;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_THROW(reg) */
	case 161:
	{

	MONO_EMIT_UNALU (s, tree, OP_THROW, -1, state->left->reg1);

	}
	return;
/* stmt: OP_RETHROW(reg) */
	case 162:
	{

	MONO_EMIT_UNALU (s, tree, OP_RETHROW, -1, state->left->reg1);

	}
	return;
/* stmt: OP_THROW_OR_NULL(reg, reg) */
	case 163:
	{

	MonoInst *dont_throw;

	MONO_NEW_LABEL (s, dont_throw);

	/*
	 * Currently, we allways rethrow the abort exception, despite the fact
	 * that this is not correct. See thread6.cs for an example. But propagating
	 * the abort exception is more important than getting the sematics right.
	 */
	MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, state->left->reg1, 0, dont_throw);
	MONO_EMIT_UNALU (s, tree, OP_THROW, -1, state->left->reg1);
	mono_bblock_add_inst (s->cbb, dont_throw);

	}
	return;
/* stmt: OP_THROW(CEE_LDIND_REF(OP_REGVAR)) */
	case 164:
	{

	MONO_EMIT_UNALU (s, tree, OP_THROW, -1, state->left->left->tree->dreg);

	}
	return;
/* stmt: OP_ENDFILTER(reg) */
	case 168:
	{

	MONO_EMIT_UNALU (s, tree, OP_ENDFILTER, -1, state->left->reg1);

	}
	return;
/* reg: OP_CHECK_THIS_PASSTHROUGH(reg) */
	case 170:
	{

	MonoInst *check;

	MONO_INST_NEW (s, check, OP_CHECK_THIS);
	check->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, check);

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_ISINST(reg) */
	case 171:
	{

     	MonoClass *klass = tree->inst_newa_class;
	MonoInst *object_is_null, *end_label, *false_label;
	int obj_reg = state->left->reg1;
	int vtable_reg = mono_regstate_next_int (s->rs);

	MONO_NEW_LABEL (s, object_is_null);
	MONO_NEW_LABEL (s, end_label);
	MONO_NEW_LABEL (s, false_label);

	MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, obj_reg, 0, object_is_null);

	if (klass->flags & TYPE_ATTRIBUTE_INTERFACE) {
		MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		/* the object_is_null target simply copies the input register to the output */
		mini_emit_isinst_iface (s, vtable_reg, klass, false_label, object_is_null);
	} else {
		int klass_reg = mono_regstate_next_int (s->rs);

		MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));

		if (klass->rank) {
			int rank_reg = mono_regstate_next_int (s->rs);
			int eclass_reg = mono_regstate_next_int (s->rs);

			MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, rank_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, rank));
			MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, rank_reg, klass->rank, false_label);
			MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
			MONO_EMIT_NEW_LOAD_MEMBASE (s, eclass_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, cast_class));
			if (klass->cast_class == mono_defaults.object_class) {
				int parent_reg = mono_regstate_next_int (s->rs);
				int const_reg = -1;
				MONO_EMIT_NEW_LOAD_MEMBASE (s, parent_reg, eclass_reg, G_STRUCT_OFFSET (MonoClass, parent));
				if (s->compile_aot) {
					const_reg = mono_regstate_next_int (s->rs);
					MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class->parent);
					MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BNE_UN, parent_reg,
							const_reg, object_is_null);
				} else {
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, parent_reg,
							mono_defaults.enum_class->parent, object_is_null);
				}
				if (s->compile_aot) {
					MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
					MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, eclass_reg, const_reg, object_is_null);
				} else {
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, eclass_reg,
						mono_defaults.enum_class, object_is_null);
				}
				MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, false_label);
			} else if (klass->cast_class == mono_defaults.enum_class->parent) {
				int const_reg = -1;
				
				if (s->compile_aot) {
					const_reg = mono_regstate_next_int (s->rs);
					MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class->parent);
					MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, eclass_reg, const_reg, object_is_null);
				} else {
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, eclass_reg,
						mono_defaults.enum_class->parent, object_is_null);
				}
				if (s->compile_aot) {
					MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
					MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, eclass_reg, const_reg, object_is_null);
				} else {
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, eclass_reg,
						mono_defaults.enum_class, object_is_null);
				}
				MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, false_label);
			} else if (klass->cast_class == mono_defaults.enum_class) {
				if (s->compile_aot) {
					int const_reg = mono_regstate_next_int (s->rs);
					MONO_EMIT_NEW_CLASSCONST (s, const_reg, mono_defaults.enum_class);
					MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BEQ, eclass_reg, const_reg, object_is_null);
				} else {
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, eclass_reg,
						mono_defaults.enum_class, object_is_null);
				}
				MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, false_label);
			} else if (klass->cast_class->flags & TYPE_ATTRIBUTE_INTERFACE) {
				mini_emit_isinst_iface_class (s, eclass_reg, klass->cast_class, false_label, object_is_null);
			} else {
				if ((klass->rank == 1) && (klass->byval_arg.type == MONO_TYPE_SZARRAY)) {
					/* Check that the object is a vector too */
					int bounds_reg = mono_regstate_next_int (s->rs);
					MONO_EMIT_NEW_LOAD_MEMBASE (s, bounds_reg, obj_reg, G_STRUCT_OFFSET (MonoArray, bounds));
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, bounds_reg, 0, false_label);
				}

				/* the object_is_null target simply copies the input register to the output */
				mini_emit_isinst (s, eclass_reg, klass->cast_class, false_label, object_is_null);
			}
		} else if (mono_class_is_nullable (klass)) {
			MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
			/* the object_is_null target simply copies the input register to the output */
			mini_emit_isinst (s, klass_reg, klass->cast_class, false_label, object_is_null);
		} else {
			if (!s->compile_aot && !(s->opt & MONO_OPT_SHARED) && (klass->flags & TYPE_ATTRIBUTE_SEALED)) {
				/* the remoting code is broken, access the class for now */
				if (0) {
					MonoVTable *vt = mono_class_vtable (s->domain, klass);
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, vtable_reg, vt, false_label);
				} else {
					MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
					MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg, klass, false_label);
				}
				MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, object_is_null);
			} else {
				MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
				/* the object_is_null target simply copies the input register to the output */
				mini_emit_isinst (s, klass_reg, klass, false_label, object_is_null);
			}
		}
	}

	mono_bblock_add_inst (s->cbb, false_label);
	MONO_EMIT_NEW_ICONST (s, state->reg1, 0);
	MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, end_label);
	mono_bblock_add_inst (s->cbb, object_is_null);
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, obj_reg);
	mono_bblock_add_inst (s->cbb, end_label);

	}
	return;
/* reg: OP_CISINST(reg) */
	case 172:
	{

	/* This opcode takes as input an object reference and a class, and returns:
	0) if the object is an instance of the class,
	1) if the object is not instance of the class,
	2) if the object is a proxy whose type cannot be determined */
	
	MonoClass *klass = tree->inst_newa_class;
	MonoInst *end_label, *true_label, *false_label, *false2_label;
	MonoInst *no_proxy_label, *interface_fail_label;
	int obj_reg = state->left->reg1;
	int tmp_reg = mono_regstate_next_int (s->rs);
	int klass_reg = mono_regstate_next_int (s->rs);

	MONO_NEW_LABEL (s, end_label);
	MONO_NEW_LABEL (s, true_label);
	MONO_NEW_LABEL (s, false_label);
	MONO_NEW_LABEL (s, false2_label);

	MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, obj_reg, 0, false_label);

	if (klass->flags & TYPE_ATTRIBUTE_INTERFACE) {
		MONO_NEW_LABEL (s, interface_fail_label);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		mini_emit_isinst_iface (s, tmp_reg, klass, interface_fail_label, true_label);
		
		mono_bblock_add_inst (s->cbb, interface_fail_label);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoVTable, klass));
		
		if (s->compile_aot) {
			int tproxy_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, tproxy_reg, mono_defaults.transparent_proxy_class);
			MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg, tproxy_reg, false_label);
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg,
				mono_defaults.transparent_proxy_class, false_label);
		}
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, custom_type_info));
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, tmp_reg, 0, false2_label);
		
	} else {
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoVTable, klass));
		
		MONO_NEW_LABEL (s, no_proxy_label);
		
		if (s->compile_aot) {
			int tproxy_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, tproxy_reg, mono_defaults.transparent_proxy_class);
			MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg, tproxy_reg, no_proxy_label);
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg,
				mono_defaults.transparent_proxy_class, no_proxy_label);
		}
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, remote_class));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoRemoteClass, proxy_class));
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, custom_type_info));
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, tmp_reg, 0, no_proxy_label);
		
		mini_emit_isinst (s, klass_reg, klass, false2_label, true_label);
		MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, false2_label);
		
		mono_bblock_add_inst (s->cbb, no_proxy_label);
		mini_emit_isinst (s, klass_reg, klass, false_label, true_label);
	}

	mono_bblock_add_inst (s->cbb, false_label);
	MONO_EMIT_NEW_ICONST (s, state->reg1, 1);
	MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, end_label);
	mono_bblock_add_inst (s->cbb, false2_label);
	MONO_EMIT_NEW_ICONST (s, state->reg1, 2);
	MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, end_label);
	mono_bblock_add_inst (s->cbb, true_label);
	MONO_EMIT_NEW_ICONST (s, state->reg1, 0);
	mono_bblock_add_inst (s->cbb, end_label);

	}
	return;
/* reg: OP_UNBOXCAST(reg) */
	case 173:
	{

     	MonoClass *klass = tree->inst_newa_class;
	int obj_reg = state->left->reg1;
	int vtable_reg = mono_regstate_next_int (s->rs);
	int klass_reg = mono_regstate_next_int (s->rs);
	int eclass_reg = mono_regstate_next_int (s->rs);
	int rank_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, rank_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, rank));

	/* FIXME: generics */
	g_assert (klass->rank == 0);

	// Check rank == 0
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, rank_reg, 0, "InvalidCastException");

	MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
	MONO_EMIT_NEW_LOAD_MEMBASE (s, eclass_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, element_class));

	if (s->compile_aot) {
		int const_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_CLASSCONST (s, const_reg, klass->element_class);
		MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, eclass_reg, const_reg, "InvalidCastException");
	} else {
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, eclass_reg, klass->element_class, "InvalidCastException");
	}
	
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, obj_reg);

	}
	return;
/* reg: OP_UNBOXCAST_REG(reg, reg) */
	case 174:
	{

	int obj_reg = state->left->reg1;
     	int element_klass_reg = state->right->reg1;
	int vtable_reg = mono_regstate_next_int (s->rs);
	int klass_reg = mono_regstate_next_int (s->rs);
	int eclass_reg = mono_regstate_next_int (s->rs);
	int rank_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, rank_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, rank));

	// Check rank == 0
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, rank_reg, 0, "InvalidCastException");

	MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
	MONO_EMIT_NEW_LOAD_MEMBASE (s, eclass_reg, klass_reg, G_STRUCT_OFFSET (MonoClass, element_class));

	MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, eclass_reg, element_klass_reg, "InvalidCastException");

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, obj_reg);

	}
	return;
/* reg: CEE_CASTCLASS(reg) */
	case 175:
	{

     	MonoClass *klass = tree->inst_newa_class;
	MonoInst *object_is_null;
	int obj_reg = state->left->reg1;
	int vtable_reg = mono_regstate_next_int (s->rs);

	MONO_NEW_LABEL (s, object_is_null);

	MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, obj_reg, 0, object_is_null);

	if (tree->backend.record_cast_details) {
		int to_klass_reg = mono_regstate_next_int (s->rs);
		int klass_reg = mono_regstate_next_int (s->rs);
		MonoInst *tls_get = mono_get_jit_tls_intrinsic (s);

		if (!tls_get) {
			fprintf (stderr, "error: --debug=casts not supported on this platform.\n.");
			exit (1);
		}

		MONO_ADD_INS (s->cbb, tls_get);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));

		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STORE_MEMBASE_REG, tls_get->dreg, G_STRUCT_OFFSET (MonoJitTlsData, class_cast_from), klass_reg);
		MONO_EMIT_NEW_PCONST (s, to_klass_reg, klass);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STORE_MEMBASE_REG, tls_get->dreg, G_STRUCT_OFFSET (MonoJitTlsData, class_cast_to), to_klass_reg);
	}

	if (klass->flags & TYPE_ATTRIBUTE_INTERFACE) {
		MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		mini_emit_castclass_iface (s, vtable_reg, klass);
	} else {
		int klass_reg = mono_regstate_next_int (s->rs);

		MONO_EMIT_NEW_LOAD_MEMBASE (s, vtable_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));

		if (!klass->rank && !s->compile_aot && !(s->opt & MONO_OPT_SHARED) && (klass->flags & TYPE_ATTRIBUTE_SEALED)) {
			/* the remoting code is broken, access the class for now */
			if (0) {
				MonoVTable *vt = mono_class_vtable (s->domain, klass);
				MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, vtable_reg, vt, "InvalidCastException");
			} else {
				MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
				MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, klass_reg, klass, "InvalidCastException");
			}
		} else {
			MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
			mini_emit_castclass (s, obj_reg, klass_reg, klass, object_is_null);
		}
	}

	mono_bblock_add_inst (s->cbb, object_is_null);

	/* Reset the variables holding the cast details */
	if (tree->backend.record_cast_details) {
		MonoInst *tls_get = mono_get_jit_tls_intrinsic (s);

		MONO_ADD_INS (s->cbb, tls_get);
		/* It is enough to reset the from field */
		MONO_EMIT_NEW_STORE_MEMBASE_IMM (s, OP_STORE_MEMBASE_IMM, tls_get->dreg, G_STRUCT_OFFSET (MonoJitTlsData, class_cast_from), 0);
	}

	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, obj_reg);

	}
	return;
/* reg: OP_CCASTCLASS(reg) */
	case 176:
	{

	/* This opcode takes as input an object reference and a class, and returns:
	0) if the object is an instance of the class,
	1) if the object is a proxy whose type cannot be determined
	an InvalidCastException exception is thrown otherwhise*/
	
	MonoClass *klass = tree->inst_newa_class;
	MonoInst *end_label, *fail_label, *no_proxy_label, *ok_result_label;
	int obj_reg = state->left->reg1;
	int tmp_reg = mono_regstate_next_int (s->rs);
	int tmp2_reg = mono_regstate_next_int (s->rs);
	int klass_reg = mono_regstate_next_int (s->rs);

	MONO_NEW_LABEL (s, end_label);
	MONO_NEW_LABEL (s, ok_result_label);

	/* Return 0 if the object is null */
	MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, obj_reg, 0, ok_result_label);

	if (klass->flags & TYPE_ATTRIBUTE_INTERFACE) {
	
		MONO_NEW_LABEL (s, fail_label);
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		mini_emit_isinst_iface (s, tmp_reg, klass, fail_label, ok_result_label);
		
		mono_bblock_add_inst (s->cbb, fail_label);
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoVTable, klass));

		if (s->compile_aot) {
			int tproxy_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, tproxy_reg, mono_defaults.transparent_proxy_class);
			MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, klass_reg, tproxy_reg, "InvalidCastException");
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, klass_reg, mono_defaults.transparent_proxy_class, "InvalidCastException");
		}
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp2_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, custom_type_info));
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, EQ, tmp2_reg, 0, "InvalidCastException");
		
		MONO_EMIT_NEW_ICONST (s, state->reg1, 1);
		MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, end_label);
		
	} else {

		MONO_NEW_LABEL (s, no_proxy_label);
		MONO_NEW_LABEL (s, fail_label);
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoObject, vtable));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoVTable, klass));

		if (s->compile_aot) {
			int tproxy_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, tproxy_reg, mono_defaults.transparent_proxy_class);
			MONO_EMIT_NEW_COMPARE_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg, tproxy_reg, no_proxy_label);
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BNE_UN, klass_reg,
				mono_defaults.transparent_proxy_class, no_proxy_label);
		}
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, remote_class));
		MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, tmp_reg, G_STRUCT_OFFSET (MonoRemoteClass, proxy_class));
		
		MONO_EMIT_NEW_LOAD_MEMBASE (s, tmp2_reg, obj_reg, G_STRUCT_OFFSET (MonoTransparentProxy, custom_type_info));
		MONO_EMIT_NEW_COMPARE_IMM_BRANCH_LABEL (s, CEE_BEQ, tmp2_reg, 0, no_proxy_label);
		
		mini_emit_isinst (s, klass_reg, klass, fail_label, ok_result_label);
		mono_bblock_add_inst (s->cbb, fail_label);
		MONO_EMIT_NEW_ICONST (s, state->reg1, 1);
		MONO_EMIT_NEW_BRANCH_LABEL (s, OP_BR, end_label);
		
		mono_bblock_add_inst (s->cbb, no_proxy_label);
		/* Again, use ok_result_label as "object_is_null" */
		mini_emit_castclass (s, obj_reg, klass_reg, klass, ok_result_label);
	}

	mono_bblock_add_inst (s->cbb, ok_result_label);
	MONO_EMIT_NEW_ICONST (s, state->reg1, 0);
	mono_bblock_add_inst (s->cbb, end_label);

	}
	return;
/* stmt: CEE_STELEM_REF(OP_GROUP(reg, reg), reg) */
	case 177:
	case 178:
	case 179:
	case 180:
	case 181:
	case 182:
	case 183:
	case 184:
	case 185:
	case 186:
	case 187:
	case 188:
	case 189:
	{

	g_assert_not_reached ();

	}
	return;
/* reg: OP_GETCHR(reg, reg) */
	case 190:
	{

	int index_reg;
	int mult_reg = mono_regstate_next_int (s->rs);
	int add_reg = mono_regstate_next_int (s->rs);
	
	if (sizeof (gpointer) == 8) {
		/* The array reg is 64 bits but the index reg is only 32 */
		index_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_UNALU (s, OP_SEXT_I4, index_reg, state->right->reg1);
	} else {
		index_reg = state->right->reg1;
	}

	MONO_EMIT_BOUNDS_CHECK (s, state->left->reg1, MonoString, length, index_reg);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_SHL_IMM, mult_reg, index_reg, 1);
	MONO_EMIT_NEW_BIALU (s, CEE_ADD, add_reg, mult_reg, state->left->reg1);
	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADU2_MEMBASE, state->reg1, 
				   add_reg, G_STRUCT_OFFSET (MonoString, chars));

	}
	return;
/* reg: OP_GETCHR(reg, OP_ICONST) */
	case 191:
	{

	int ind = 2 * state->right->tree->inst_c0 + G_STRUCT_OFFSET (MonoString, chars);

	MONO_EMIT_BOUNDS_CHECK_IMM (s, state->left->reg1, MonoString, length, state->right->tree->inst_c0);
	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADU2_MEMBASE, state->reg1, state->left->reg1, ind);

	}
	return;
/* reg: OP_STR_CHAR_ADDR(reg, reg) */
	case 192:
	{

	int mult_reg = mono_regstate_next_int (s->rs);
	int add_reg = mono_regstate_next_int (s->rs);

	/*
	 * The corlib functions check for oob already.
	 * MONO_EMIT_BOUNDS_CHECK (s, state->left->reg1, MonoString, length, state->right->reg1);
	 */
	MONO_EMIT_NEW_BIALU_IMM (s, OP_SHL_IMM, mult_reg, state->right->reg1, 1);
	MONO_EMIT_NEW_BIALU (s, CEE_ADD, add_reg, mult_reg, state->left->reg1);
	MONO_EMIT_BIALU_IMM (s, tree, OP_ADD_IMM, state->reg1, add_reg, G_STRUCT_OFFSET (MonoString, chars));

	}
	return;
/* reg: OP_STRLEN(reg) */
	case 193:
	{
	
	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADI4_MEMBASE, state->reg1, 
				   state->left->reg1, G_STRUCT_OFFSET (MonoString, length));

	}
	return;
/* reg: OP_GETTYPE(reg) */
	case 194:
	{
	
	int vt_reg = mono_regstate_next_int (s->rs);
	MONO_EMIT_NEW_LOAD_MEMBASE (s, vt_reg, state->left->reg1, G_STRUCT_OFFSET (MonoObject, vtable));
	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOAD_MEMBASE, state->reg1, vt_reg, G_STRUCT_OFFSET (MonoVTable, type));

	}
	return;
/* reg: OP_GETHASHCODE(reg) */
	case 195:
	{

	int t1 = mono_regstate_next_int (s->rs);
	
	#define MONO_OBJECT_ALIGNMENT_SHIFT	3

	/* return (GPOINTER_TO_UINT (this) >> MONO_OBJECT_ALIGNMENT_SHIFT) * 2654435761u; */
	
	MONO_EMIT_NEW_BIALU_IMM (s, OP_SHL_IMM, t1, state->left->reg1, MONO_OBJECT_ALIGNMENT_SHIFT);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_MUL_IMM, state->reg1, t1, 2654435761u);

	}
	return;
/* reg: CEE_LDLEN(reg) */
	case 196:
	{
	
	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADI4_MEMBASE, state->reg1, 
				   state->left->reg1, G_STRUCT_OFFSET (MonoArray, max_length));

	}
	return;
/* reg: OP_ARRAY_RANK(reg) */
	case 197:
	{
	
	int vtable_reg = mono_regstate_next_int (s->rs);
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOAD_MEMBASE, vtable_reg, 
				       state->left->reg1, G_STRUCT_OFFSET (MonoObject, vtable));
	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADU1_MEMBASE, state->reg1, 
				       vtable_reg, G_STRUCT_OFFSET (MonoVTable, rank));

	}
	return;
/* reg: OP_CHECK_ARRAY_TYPE(reg) */
	case 198:
	{


	MonoClass* array_class = tree->klass;
	
	int vtable_reg = mono_regstate_next_int (s->rs);

	MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOAD_MEMBASE, vtable_reg, 
				       state->left->reg1, G_STRUCT_OFFSET (MonoObject, vtable));
				       
	if (s->opt & MONO_OPT_SHARED) {
		int class_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOAD_MEMBASE, class_reg, 
					       vtable_reg, G_STRUCT_OFFSET (MonoVTable, klass));
		if (s->compile_aot) {
			int klass_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_CLASSCONST (s, klass_reg, array_class);
			MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, class_reg, klass_reg, "ArrayTypeMismatchException");
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, class_reg, array_class, "ArrayTypeMismatchException");
		}
	} else {
		if (s->compile_aot) {
			int vt_reg = mono_regstate_next_int (s->rs);
			MONO_EMIT_NEW_VTABLECONST (s, vt_reg, mono_class_vtable (s->domain, array_class));
			MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, vtable_reg, vt_reg, "ArrayTypeMismatchException");
		} else {
			MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, vtable_reg, mono_class_vtable (s->domain, array_class), "ArrayTypeMismatchException");
		}
	}
	
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_LDELEMA(reg, OP_ICONST) */
	case 199:
	{

	guint32 size = mono_class_array_element_size (tree->klass);

	int ind = size * state->right->tree->inst_c0 + G_STRUCT_OFFSET (MonoArray, vector);

	MONO_EMIT_BOUNDS_CHECK_IMM (s, state->left->reg1, MonoArray, max_length, state->right->tree->inst_c0);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, state->reg1, state->left->reg1, ind);

	}
	return;
/* reg: CEE_LDELEMA(reg, reg) */
	case 200:
	{

	int index_reg;
	int mult_reg = mono_regstate_next_int (s->rs);
	int add_reg = mono_regstate_next_int (s->rs);
	guint32 size = mono_class_array_element_size (tree->klass);

	if (sizeof (gpointer) == 8) {
		/* The array reg is 64 bits but the index reg is only 32 */
		index_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_UNALU (s, OP_SEXT_I4, index_reg, state->right->reg1);
	} else {
		index_reg = state->right->reg1;
	}

	MONO_EMIT_BOUNDS_CHECK (s, state->left->reg1, MonoArray, max_length, index_reg);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_MUL_IMM, mult_reg, index_reg, size);
	MONO_EMIT_NEW_BIALU (s, CEE_ADD, add_reg, mult_reg, state->left->reg1);
	MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, state->reg1, add_reg, G_STRUCT_OFFSET (MonoArray, vector));

	}
	return;
/* reg: CEE_REFANYVAL(reg) */
	case 201:
	{

	int klass_reg = mono_regstate_next_int (s->rs);
	MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, state->left->reg1, G_STRUCT_OFFSET (MonoTypedRef, klass));
	if (s->compile_aot) {
		int const_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_CLASSCONST (s, const_reg, tree->inst_newa_class);
		MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, klass_reg, const_reg, "InvalidCastException");
	} else {
		MONO_EMIT_NEW_COMPARE_IMM_EXC (s, NE_UN, klass_reg, tree->inst_newa_class, "InvalidCastException");
	}
	MONO_EMIT_NEW_LOAD_MEMBASE (s, state->reg1, state->left->reg1, G_STRUCT_OFFSET (MonoTypedRef, value));

	}
	return;
/* reg: OP_REFANYVAL_REG(reg, reg) */
	case 202:
	{

	int klass_reg = mono_regstate_next_int (s->rs);
	g_assert (!s->compile_aot);
	MONO_EMIT_NEW_LOAD_MEMBASE (s, klass_reg, state->left->reg1, G_STRUCT_OFFSET (MonoTypedRef, klass));
	MONO_EMIT_NEW_COMPARE_EXC (s, NE_UN, klass_reg, state->right->reg1, "InvalidCastException");
	MONO_EMIT_NEW_LOAD_MEMBASE (s, state->reg1, state->left->reg1, G_STRUCT_OFFSET (MonoTypedRef, value));

	}
	return;
/* reg: OP_REFANYTYPE(reg) */
	case 203:
	{

	MONO_EMIT_NEW_LOAD_MEMBASE (s, state->reg1, state->left->reg1, G_STRUCT_OFFSET (MonoTypedRef, type));

	}
	return;
/* stmt: CEE_MKREFANY(OP_GROUP(reg, OP_ICONST), reg) */
	case 204:
	case 205:
	{

	MonoClass *klass = (MonoClass*)(tree->inst_left->inst_right->inst_p0);
	if (s->compile_aot) {
		int const_reg = mono_regstate_next_int (s->rs);
		int type_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_CLASSCONST (s, const_reg, klass);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG, state->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, klass), const_reg);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, type_reg, const_reg, G_STRUCT_OFFSET (MonoClass, byval_arg));
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG, state->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, type), type_reg);
	}
	else {
		MONO_EMIT_NEW_STORE_MEMBASE_IMM (s, OP_STOREP_MEMBASE_IMM, state->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, type), &klass->byval_arg);
		MONO_EMIT_NEW_STORE_MEMBASE_IMM (s, OP_STOREP_MEMBASE_IMM, state->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, klass), klass);
	}
	MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG, state->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, value), state->left->left->reg1);

	}
	return;
/* stmt: OP_MKREFANY_REGS(OP_GROUP(reg, reg), OP_GROUP(reg, reg)) */
	case 206:
	{

	g_assert (!s->compile_aot);
	MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG,
			state->right->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, type), state->left->left->reg1);
	MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG,
			state->right->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, klass), state->left->right->reg1);
	MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREP_MEMBASE_REG,
			state->right->right->reg1, G_STRUCT_OFFSET (MonoTypedRef, value), state->right->left->reg1);

	}
	return;
/* freg: CEE_LDIND_R4(base) */
	case 207:
	{

	if (SOFT_FLOAT_IMPL) {
		g_assert_not_reached ();
	} else {
		MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADR4_MEMBASE, state->reg1, 
				   state->left->tree->inst_basereg, state->left->tree->inst_offset);
	}

	}
	return;
/* freg: CEE_LDIND_R8(base) */
	case 208:
	{

	if (SOFT_FLOAT_IMPL) {
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, state->reg1, 
				       state->left->tree->inst_basereg, state->left->tree->inst_offset + MINI_LS_WORD_OFFSET);
		MONO_EMIT_NEW_LOAD_MEMBASE_OP (s, OP_LOADI4_MEMBASE, state->reg2, 
				       state->left->tree->inst_basereg, state->left->tree->inst_offset + MINI_MS_WORD_OFFSET);
	} else {
		MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADR8_MEMBASE, state->reg1, 
				   state->left->tree->inst_basereg, state->left->tree->inst_offset);
	}

	}
	return;
/* stmt: CEE_STIND_R4(base, freg) */
	case 209:
	{

	if (SOFT_FLOAT_IMPL) {
		g_assert_not_reached ();
	} else {
		MONO_EMIT_STORE_MEMBASE (s, tree, OP_STORER4_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);
	}

	}
	return;
/* stmt: CEE_STIND_R8(base, freg) */
	case 210:
	{

	if (SOFT_FLOAT_IMPL) {
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI4_MEMBASE_REG, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset + MINI_MS_WORD_OFFSET, state->right->reg2);
		MONO_EMIT_NEW_STORE_MEMBASE (s, OP_STOREI4_MEMBASE_REG, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset + MINI_LS_WORD_OFFSET, state->right->reg1);
	} else {
		MONO_EMIT_STORE_MEMBASE (s, tree, OP_STORER8_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);
	}

	}
	return;
/* freg: OP_R4CONST */
	case 211:
	{

	if (SOFT_FLOAT_IMPL) {
		DVal d;
		/* we load the r8 value */
		d.vald = *(float*)tree->inst_p0;
		MONO_EMIT_NEW_ICONST (s, state->reg1, d.vali [MINI_LS_WORD_IDX]);
		MONO_EMIT_NEW_ICONST (s, state->reg2, d.vali [MINI_MS_WORD_IDX]);
	} else {
		tree->dreg = state->reg1;
		mono_bblock_add_inst (s->cbb, tree);
	}

	}
	return;
/* freg: OP_R8CONST */
	case 212:
	{

	if (SOFT_FLOAT_IMPL) {
		DVal d;
		d.vald = *(double*)tree->inst_p0;
		MONO_EMIT_NEW_ICONST (s, state->reg1, d.vali [MINI_LS_WORD_IDX]);
		MONO_EMIT_NEW_ICONST (s, state->reg2, d.vali [MINI_MS_WORD_IDX]);
	} else {
		tree->dreg = state->reg1;
		mono_bblock_add_inst (s->cbb, tree);
	}

	}
	return;
/* freg: OP_FADD(freg, freg) */
	case 213:
	case 214:
	case 215:
	case 216:
	case 217:
	case 276:
	case 278:
	case 280:
	case 506:
	case 507:
	case 508:
	case 509:
	case 510:
	case 511:
	case 512:
	case 513:
	case 514:
	case 515:
	case 516:
	case 517:
	case 518:
	case 519:
	case 520:
	case 521:
	case 522:
	case 523:
	{

	MONO_EMIT_BIALU (s, tree, tree->opcode, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* freg: OP_FCONV_TO_R8(freg) */
	case 240:
	{

	MONO_EMIT_UNALU (s, tree, OP_FMOVE, state->reg1, state->left->reg1);

	}
	return;
/* fpcflags: OP_COMPARE(freg, freg) */
	case 245:
	{

	tree->opcode = OP_FCOMPARE;
	tree->sreg1 = state->left->reg1;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_POP(freg) */
	case 246:
	{

	/* do nothing */

	}
	return;
/* reg: OP_I8CONST */
	case 247:
	{

	MONO_EMIT_NEW_I8CONST (s, state->reg1, tree->inst_l);

	}
	return;
/* reg: CEE_LDIND_I8(base) */
	case 248:
	{

	MONO_EMIT_LOAD_MEMBASE_OP (s, tree, OP_LOADI8_MEMBASE, state->reg1, 
				   state->left->tree->inst_basereg, state->left->tree->inst_offset);

	}
	return;
/* stmt: CEE_STIND_I8(base, reg) */
	case 249:
	case 256:
	{

	MONO_EMIT_STORE_MEMBASE (s, tree, OP_STOREI8_MEMBASE_REG, state->left->tree->inst_basereg,
				 state->left->tree->inst_offset, state->right->reg1);

	}
	return;
/* stmt: CEE_STIND_REF(base, OP_I8CONST) */
	case 250:
	case 251:
	case 252:
	case 253:
	case 254:
	case 255:
	{

	MONO_EMIT_STORE_MEMBASE_IMM (s, tree, OP_STOREI8_MEMBASE_IMM, state->left->tree->inst_basereg,
				     state->left->tree->inst_offset, state->right->tree->inst_l);

	}
	return;
/* reg: OP_LADD(reg, reg) */
	case 260:
	{

	MONO_EMIT_NEW_BIALU (s, CEE_ADD, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: OP_LADD(reg, OP_ICONST) */
	case 261:
	case 262:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ADD_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	case 263:
	case 264:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ADD_IMM, state->reg1, state->left->left->tree->dreg, state->right->tree->inst_c0);

	}
	return;
/* reg: OP_LSUB(reg, reg) */
	case 265:
	{

	MONO_EMIT_NEW_BIALU (s, CEE_SUB, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: OP_LNEG(reg) */
	case 266:
	{

	MONO_EMIT_NEW_UNALU (s, CEE_NEG, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LNOT(reg) */
	case 267:
	{

	MONO_EMIT_NEW_UNALU (s, CEE_NOT, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LAND(reg, reg) */
	case 268:
	{

	MONO_EMIT_NEW_BIALU (s, CEE_AND, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: OP_LOR(reg, reg) */
	case 269:
	{

	MONO_EMIT_NEW_BIALU (s, CEE_OR, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: OP_LXOR(reg, reg) */
	case 270:
	{

	MONO_EMIT_NEW_BIALU (s, CEE_XOR, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: OP_LADD_OVF(reg, reg) */
	case 271:
	{

	MONO_EMIT_NEW_BIALU (s, OP_ADDCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, OV, "OverflowException");

	}
	return;
/* reg: OP_LADD_OVF_UN(reg, reg) */
	case 272:
	{

	MONO_EMIT_NEW_BIALU (s, OP_ADDCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, C, "OverflowException");

	}
	return;
/* reg: OP_LSUB_OVF(reg, reg) */
	case 273:
	{

	MONO_EMIT_NEW_BIALU (s, OP_SUBCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, OV, "OverflowException");

	}
	return;
/* reg: OP_LSUB_OVF_UN(reg, reg) */
	case 274:
	{

	MONO_EMIT_NEW_BIALU (s, OP_SUBCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, C, "OverflowException");

	}
	return;
/* reg: OP_LONG_SHRUN_32(reg) */
	case 275:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_LSHR_UN_IMM, state->reg1, state->left->reg1, 32);

	}
	return;
/* reg: OP_LSHL(reg, OP_ICONST) */
	case 277:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_LSHL_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: OP_LSHR(reg, OP_ICONST) */
	case 279:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_LSHR_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: OP_LSHR_UN(reg, OP_ICONST) */
	case 281:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_LSHR_UN_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_CONV_I8(reg) */
	case 282:
	{

	/* Sign extend the value in the lower word into the upper word */
	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);	

	}
	return;
/* reg: CEE_CONV_U8(reg) */
	case 283:
	{

	/* Clean out the upper word */
	/* Sign extend the value in the lower word into the upper word */
	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* i8con: CEE_CONV_U8(OP_ICONST) */
	case 284:
	{

	/*
	 * This is needed since constant propagation eliminates some 
     * stind.i4/ldind.i4 pairs, along with the conversions done by them.
	 */
	int data = state->left->tree->inst_c0;
	tree->opcode = OP_I8CONST;
	tree->inst_ls_word = data;
	tree->inst_ms_word = 0;

	}
	return;
/* reg: OP_LCONV_TO_I4(i8con) */
	case 287:
	{

     MONO_EMIT_NEW_ICONST (s, state->reg1, state->left->tree->inst_ls_word);

	}
	return;
/* reg: OP_LCONV_TO_I4(reg) */
	case 288:
	{

	/* Sign extend the value in the lower word into the upper word */
	MONO_EMIT_BIALU_IMM (s, tree, OP_LSHR_IMM, state->reg1, state->left->reg1, 0);

	}
	return;
/* reg: OP_LCONV_TO_U4(reg) */
	case 289:
	{

	/* Clean out the upper word */
	MONO_EMIT_BIALU_IMM (s, tree, OP_ISHR_UN_IMM, state->reg1, state->left->reg1, 0);

	}
	return;
/* reg: OP_LCONV_TO_I1(reg) */
	case 294:
	{

	MONO_EMIT_UNALU (s, tree, CEE_CONV_I1, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_U1(reg) */
	case 295:
	{

	MONO_EMIT_UNALU (s, tree, CEE_CONV_U1, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_I2(reg) */
	case 296:
	{

	MONO_EMIT_UNALU (s, tree, CEE_CONV_I2, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_U2(reg) */
	case 297:
	{

	MONO_EMIT_UNALU (s, tree, CEE_CONV_U2, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I1(reg) */
	case 298:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT, state->left->reg1, 127, "OverflowException");
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, -128, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I1, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I1_UN(reg) */
	case 299:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 127, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I1, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I2(reg) */
	case 302:
	{

	/* Probe value to be within -32768 and 32767 */
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT, state->left->reg1, 32767, "OverflowException");
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, -32768, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I2, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I2_UN(reg) */
	case 303:
	{

	/* Probe value to be within 0 and 32767 */
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 32767, "OverflowException");
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I2, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_U2(reg) */
	case 304:
	case 305:
	{

	/* Probe value to be within 0 and 65535 */
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 0xffff, "OverflowException");
	MONO_EMIT_BIALU_IMM (s, tree, OP_AND_IMM, state->reg1, state->left->reg1, 0xffff);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I4(reg) */
	case 306:
	{

	/* The int cast is needed for the VS compiler.  See Compiler Warning (level 2) C4146. */
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT, state->left->reg1, 0x7fffffff, "OverflowException");
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, ((int)-2147483648), "OverflowException");
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I4_UN(reg) */
	case 307:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 0x7fffffff, "OverflowException");
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_U4(reg) */
	case 308:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT, state->left->reg1, 0xffffffffUL, "OverflowException");
	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, 0, "OverflowException");
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_U4_UN(reg) */
	case 309:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, GT_UN, state->left->reg1, 0xffffffff, "OverflowException");
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCONV_TO_OVF_I_UN(reg) */
	case 310:
	case 311:
	case 312:
	case 313:
	case 325:
	case 326:
	{

	MONO_EMIT_NEW_COMPARE_IMM_EXC (s, LT, state->left->reg1, 0, "OverflowException");
	MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_I8(reg) */
	case 319:
	case 320:
	{

	/* Sign extend the value in the lower word into the upper word */
	MONO_EMIT_UNALU (s, tree, CEE_CONV_I8, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_CONV_OVF_I8_UN(reg) */
	case 321:
	case 322:
	case 323:
	case 324:
	{

	/* an unsigned 32 bit num always fits in an (un)signed 64 bit one */
	/* Clean out the upper word */
	MONO_EMIT_UNALU (s, tree, CEE_CONV_U8, state->reg1, state->left->reg1);

	}
	return;
/* reg: OP_LCALLVIRT(reg) */
	case 327:
	{

	mini_emit_virtual_call (s, state, tree, OP_LCALL, OP_LCALL_MEMBASE);

	}
	return;
/* reg: OP_LCALLVIRT_IMT(reg, reg) */
	case 328:
	{

	tree->opcode = OP_LCALLVIRT;
	mini_emit_virtual_imt_call (s, state, tree, OP_LCALL, OP_LCALL_MEMBASE, state->right->reg1);

	}
	return;
/* reg: OP_LCALL_RGCTX(reg) */
	case 330:
	{

	emit_rgctx_argument (s, tree, state->left->reg1, OP_LCALL);

	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_LCALL_REG_RGCTX(reg, reg) */
	case 332:
	{

	emit_rgctx_argument (s, tree, state->right->reg1, OP_LCALL_REG);

	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_LCALL_REG(OP_ICONST) */
	case 333:
	case 334:
	{

	tree->opcode = OP_LCALL;
	((MonoCallInst*)tree)->fptr = state->left->tree->inst_p0;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
	case 335:
	{

	tree->inst_c0 = state->right->tree->inst_c0;
	tree->opcode = OP_ICONST;
	tree->dreg = state->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(reg, reg) */
	case 339:
	{

	tree->sreg1 = state->left->reg1;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), reg) */
	case 340:
	{

	tree->sreg1 = state->left->left->tree->dreg;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
	case 341:
	{

	tree->sreg1 = state->left->left->tree->dreg;
	tree->sreg2 = state->right->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), OP_ICONST) */
	case 342:
	{

	tree->opcode = OP_COMPARE_IMM;
	tree->sreg1 = state->left->left->tree->dreg;
	tree->inst_imm = state->right->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(reg, OP_ICONST) */
	case 343:
	{

	tree->opcode = OP_COMPARE_IMM;
	tree->sreg1 = state->left->reg1;
	tree->inst_imm = state->right->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_LCOMPARE(reg, OP_I8CONST) */
	case 344:
	{

	tree->opcode = OP_COMPARE_IMM;
	tree->sreg1 = state->left->reg1;
	tree->inst_imm = state->right->tree->inst_l;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_AND(reg, reg) */
	case 355:
	{

	MONO_EMIT_BIALU (s, tree, OP_IAND, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_AND(reg, OP_ICONST) */
	case 356:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_IAND_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_OR(reg, reg) */
	case 357:
	{

	MONO_EMIT_BIALU (s, tree, OP_IOR, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_OR(reg, OP_ICONST) */
	case 358:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_IOR_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_XOR(reg, reg) */
	case 359:
	{

	MONO_EMIT_BIALU (s, tree, OP_IXOR, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_XOR(reg, OP_ICONST) */
	case 360:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_IXOR_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_NEG(reg) */
	case 361:
	{

	MONO_EMIT_UNALU (s, tree, OP_INEG, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_NOT(reg) */
	case 362:
	{

	MONO_EMIT_UNALU (s, tree, OP_INOT, state->reg1, state->left->reg1);

	}
	return;
/* reg: CEE_ADD(reg, reg) */
	case 363:
	{

	MONO_EMIT_BIALU (s, tree, OP_IADD, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_ADD(reg, OP_ICONST) */
	case 364:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_IADD_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_ADD_OVF(reg, reg) */
	case 365:
	{

	MONO_EMIT_NEW_BIALU (s, OP_IADDCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, IOV, "OverflowException");

	}
	return;
/* reg: CEE_ADD_OVF_UN(reg, reg) */
	case 366:
	{

	MONO_EMIT_NEW_BIALU (s, OP_IADDCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, IC, "OverflowException");

	}
	return;
/* reg: CEE_SUB(reg, reg) */
	case 367:
	{

	MONO_EMIT_BIALU (s, tree, OP_ISUB, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_SUB(reg, CEE_LDIND_I4(OP_REGVAR)) */
	case 368:
	{

	MONO_EMIT_BIALU (s, tree, OP_ISUB, state->reg1, state->left->reg1, state->right->left->tree->dreg);

	}
	return;
/* reg: CEE_SUB(reg, OP_ICONST) */
	case 369:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ISUB_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_SUB_OVF(reg, reg) */
	case 370:
	{

	MONO_EMIT_NEW_BIALU (s, OP_ISUBCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, IOV, "OverflowException");

	}
	return;
/* reg: CEE_SUB_OVF_UN(reg, reg) */
	case 371:
	{

	MONO_EMIT_NEW_BIALU (s, OP_ISUBCC, state->reg1, state->left->reg1, state->right->reg1);
	MONO_EMIT_NEW_COND_EXC (s, IC, "OverflowException");

	}
	return;
/* reg: CEE_SHL(reg, reg) */
	case 372:
	{

	MONO_EMIT_BIALU (s, tree, OP_ISHL, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_SHL(reg, OP_ICONST) */
	case 373:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ISHL_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_SHR(reg, reg) */
	case 374:
	{

	MONO_EMIT_BIALU (s, tree, OP_ISHR, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_SHR(reg, OP_ICONST) */
	case 375:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ISHR_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_SHR_UN(reg, reg) */
	case 376:
	{

	MONO_EMIT_BIALU (s, tree, OP_ISHR_UN, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_SHR_UN(reg, OP_ICONST) */
	case 377:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_ISHR_UN_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_MUL(reg, reg) */
	case 378:
	{

	MONO_EMIT_BIALU (s, tree, OP_IMUL, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_MUL(reg, OP_ICONST) */
	case 379:
	{

	MONO_EMIT_BIALU_IMM (s, tree, OP_IMUL_IMM, state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: CEE_MUL_OVF(reg, reg) */
	case 380:
	{

	MONO_EMIT_BIALU (s, tree, OP_IMUL_OVF, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_MUL_OVF_UN(reg, reg) */
	case 381:
	{

	MONO_EMIT_BIALU (s, tree, OP_IMUL_OVF_UN, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_DIV(reg, reg) */
	case 382:
	{

	MONO_EMIT_BIALU (s, tree, OP_IDIV, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_DIV_UN(reg, reg) */
	case 383:
	{

	MONO_EMIT_BIALU (s, tree, OP_IDIV_UN, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_REM(reg, reg) */
	case 384:
	{

	MONO_EMIT_BIALU (s, tree, OP_IREM, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* reg: CEE_REM_UN(reg, reg) */
	case 385:
	{

	MONO_EMIT_BIALU (s, tree, OP_IREM_UN, state->reg1, state->left->reg1, state->right->reg1);

	}
	return;
/* c32flags: OP_COMPARE(reg, reg) */
	case 388:
	{

	tree->opcode = OP_ICOMPARE;
	tree->sreg1 = state->left->reg1;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
	case 389:
	{

	tree->opcode = OP_ICOMPARE;
	tree->sreg1 = state->left->left->tree->dreg;
	tree->sreg2 = state->right->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
	case 390:
	case 392:
	{

	tree->opcode = OP_ICOMPARE_IMM;
	tree->sreg1 = state->left->left->tree->dreg;
	tree->inst_imm = state->right->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), reg) */
	case 391:
	{

	tree->opcode = OP_ICOMPARE;
	tree->sreg1 = state->left->left->tree->dreg;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* c32flags: OP_COMPARE(reg, OP_ICONST) */
	case 393:
	{

	tree->opcode = OP_ICOMPARE_IMM;
	tree->sreg1 = state->left->reg1;
	tree->inst_imm = state->right->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BNE_UN(c32flags) */
	case 394:
	{

	tree->opcode = OP_IBNE_UN;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BEQ(c32flags) */
	case 395:
	{

	tree->opcode = OP_IBEQ;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BLT(c32flags) */
	case 396:
	{

	tree->opcode = OP_IBLT;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BLT_UN(c32flags) */
	case 397:
	{

	tree->opcode = OP_IBLT_UN;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BGT(c32flags) */
	case 398:
	{

	tree->opcode = OP_IBGT;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BGT_UN(c32flags) */
	case 399:
	{

	tree->opcode = OP_IBGT_UN;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BGE(c32flags) */
	case 400:
	{

	tree->opcode = OP_IBGE;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BGE_UN(c32flags) */
	case 401:
	{

	tree->opcode = OP_IBGE_UN;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BLE(c32flags) */
	case 402:
	{

	tree->opcode = OP_IBLE;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_BLE_UN(c32flags) */
	case 403:
	{

	tree->opcode = OP_IBLE_UN;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CEQ(c32flags) */
	case 404:
	{
	
	tree->opcode = OP_ICEQ;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CLT(c32flags) */
	case 405:
	{
	
	tree->opcode = OP_ICLT;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CLT_UN(c32flags) */
	case 406:
	{
	
	tree->opcode = OP_ICLT_UN;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CGT(c32flags) */
	case 407:
	{
	
	tree->opcode = OP_ICGT;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CGT_UN(c32flags) */
	case 408:
	{
	
	tree->opcode = OP_ICGT_UN;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_LDIND_I8(OP_REGVAR) */
	case 409:
	{

	state->reg1 = state->left->tree->dreg;

	}
	return;
/* stmt: CEE_STIND_I8(OP_REGVAR, reg) */
	case 410:
	{

	MONO_EMIT_NEW_UNALU (s, OP_MOVE, state->left->tree->dreg, state->right->reg1);

	}
	return;
/* reg: CEE_LDIND_I1(OP_REGVAR) */
	case 411:
	{

	MONO_EMIT_UNALU (s, tree, OP_SEXT_I1, state->reg1, state->left->tree->dreg);
	}
	return;
/* reg: CEE_LDIND_I2(OP_REGVAR) */
	case 412:
	{

	MONO_EMIT_UNALU (s, tree, OP_SEXT_I2, state->reg1, state->left->tree->dreg);
	}
	return;
/* cflags: OP_COMPARE(CEE_LDIND_I4(base), reg) */
	case 419:
	{

	tree->opcode = OP_AMD64_ICOMPARE_MEMBASE_REG;
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	tree->sreg2 = state->right->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_COMPARE(CEE_LDIND_I4(base), OP_ICONST) */
	case 420:
	{

	tree->opcode = OP_AMD64_ICOMPARE_MEMBASE_IMM;
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	tree->inst_imm = state->right->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* cflags: OP_COMPARE(reg, CEE_LDIND_I4(base)) */
	case 421:
	{

	tree->opcode = OP_AMD64_ICOMPARE_REG_MEMBASE;
	tree->sreg2 = state->right->left->tree->inst_basereg;
	tree->inst_offset = state->right->left->tree->inst_offset;
	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I1(base, OP_CEQ(cflags)) */
	case 422:
	{

	tree->opcode = OP_X86_SETEQ_MEMBASE;
	tree->inst_offset = state->left->tree->inst_offset;
	tree->inst_basereg = state->left->tree->inst_basereg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_LOCALLOC(OP_ICONST) */
	case 423:
	{

	if (tree->flags & MONO_INST_INIT) {
		/* microcoded in mini-x86.c */
		tree->sreg1 = mono_regstate_next_int (s->rs);
		tree->dreg = state->reg1;
		MONO_EMIT_NEW_ICONST (s, tree->sreg1, state->left->tree->inst_c0);
		mono_bblock_add_inst (s->cbb, tree);
	} else {
		guint32 size = state->left->tree->inst_c0;
		size = (size + (MONO_ARCH_LOCALLOC_ALIGNMENT - 1)) & ~ (MONO_ARCH_LOCALLOC_ALIGNMENT - 1);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SUB_IMM, X86_ESP, X86_ESP, size);
		MONO_EMIT_UNALU (s, tree, OP_MOVE, state->reg1, X86_ESP);
	}

	}
	return;
/* reg: OP_LOCALLOC(reg) */
	case 424:
	{

	tree->sreg1 = state->left->tree->dreg;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_SETRET(reg) */
	case 425:
	{

	tree->opcode = OP_MOVE;
	tree->sreg1 = state->left->reg1;
	tree->dreg = X86_EAX;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_SETRET(reg) */
	case 426:
	{

	MONO_EMIT_NEW_UNALU (s, OP_MOVE, X86_EDX, state->left->reg2);
	tree->opcode = OP_MOVE;
	tree->sreg1 = state->left->reg1;
	tree->dreg = X86_EAX;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_LDIND_REF(OP_REGVAR) */
	case 427:
	case 428:
	case 429:
	case 430:
	{

	/* This rule might not work on all archs */
	state->reg1 = state->left->tree->dreg;
	tree->dreg = state->reg1;

	}
	return;
/* stmt: OP_SETRET(CEE_LDIND_REF(OP_REGVAR)) */
	case 431:
	case 432:
	case 433:
	case 434:
	{

	tree->opcode = OP_MOVE;
	tree->sreg1 = state->left->left->tree->dreg;
	tree->dreg = X86_EAX;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_SETRET(freg) */
	case 435:
	{

	if (mono_method_signature (s->method)->ret->type == MONO_TYPE_R4)
		tree->opcode = OP_AMD64_SET_XMMREG_R4;
	else
		tree->opcode = OP_AMD64_SET_XMMREG_R8;
	tree->sreg1 = state->left->reg1;
	tree->dreg = 0; /* %xmm0 */
	mono_bblock_add_inst (s->cbb, tree);	
	/* nothing to do */

	}
	return;
/* stmt: OP_SETRET(OP_ICONST) */
	case 436:
	{

	if (state->left->tree->inst_c0 == 0) {
		MONO_EMIT_BIALU (s, tree, CEE_XOR, AMD64_RAX, AMD64_RAX, AMD64_RAX);
	}
	else {
		tree->opcode = OP_ICONST;
		tree->inst_c0 = state->left->tree->inst_c0;
		tree->dreg = X86_EAX;
		mono_bblock_add_inst (s->cbb, tree);
	}

	}
	return;
/* stmt: OP_OUTARG(reg) */
	case 437:
	case 446:
	{

	tree->opcode = OP_X86_PUSH;
	tree->sreg1 = state->left->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_OUTARG_REG(reg) */
	case 438:
	{
	
	MonoCallInst *call = tree->inst_call;

	tree->opcode = OP_MOVE;
	tree->sreg1 = state->left->reg1;
	tree->dreg = mono_regstate_next_int (s->rs);
	mono_bblock_add_inst (s->cbb, tree);

	mono_call_inst_add_outarg_reg (s, call, tree->dreg, tree->backend.reg3, FALSE);

	}
	return;
/* stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR)) */
	case 439:
	case 440:
	case 441:
	case 442:
	case 445:
	{

	tree->opcode = OP_X86_PUSH;
	tree->sreg1 = state->left->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_OUTARG(CEE_LDIND_I(base)) */
	case 443:
	case 444:
	{

	tree->opcode = OP_X86_PUSH_MEMBASE;
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_OUTARG(freg) */
	case 447:
	case 449:
	{

	MONO_EMIT_NEW_BIALU_IMM (s, OP_SUB_IMM, X86_ESP, X86_ESP, 8);
	tree->opcode = OP_STORER8_MEMBASE_REG;
	tree->sreg1 = state->left->reg1;
	tree->inst_destbasereg = X86_ESP;
	tree->inst_offset = 0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_OUTARG_R4(freg) */
	case 448:
	{

	MONO_EMIT_NEW_BIALU_IMM (s, OP_SUB_IMM, X86_ESP, X86_ESP, 8);
	tree->opcode = OP_STORER4_MEMBASE_REG;
	tree->sreg1 = state->left->reg1;
	tree->inst_destbasereg = X86_ESP;
	tree->inst_offset = 0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_AMD64_OUTARG_XMMREG_R4(freg) */
	case 450:
	{

	MonoCallInst *call = tree->inst_call;

	tree->opcode = OP_AMD64_SET_XMMREG_R4;
	tree->sreg1 = state->left->reg1;
	tree->dreg = mono_regstate_next_float (s->rs);
	mono_bblock_add_inst (s->cbb, tree);

	mono_call_inst_add_outarg_reg (s, call, tree->dreg, tree->backend.reg3, TRUE);

	}
	return;
/* stmt: OP_AMD64_OUTARG_XMMREG_R8(freg) */
	case 451:
	{

	MonoCallInst *call = tree->inst_call;

	tree->opcode = OP_AMD64_SET_XMMREG_R8;
	tree->sreg1 = state->left->reg1;
	tree->dreg = mono_regstate_next_float (s->rs);
	mono_bblock_add_inst (s->cbb, tree);

	mono_call_inst_add_outarg_reg (s, call, tree->dreg, tree->backend.reg3, TRUE);

	}
	return;
/* stmt: OP_OUTARG_VT(CEE_LDOBJ(base)) */
	case 452:
	{

	MonoInst *vt = state->left->left->tree;
	//g_print ("vt size: %d at R%d + %d\n", tree->inst_imm, vt->inst_basereg, vt->inst_offset);

	if (!tree->inst_imm)
		return;

	if (tree->inst_imm == 8) {
		/* Can't use this for < 8 since it does an 8 byte memory load */
		tree->opcode = OP_X86_PUSH_MEMBASE;
		tree->inst_basereg = vt->inst_basereg;
		tree->inst_offset = vt->inst_offset;
		mono_bblock_add_inst (s->cbb, tree);
	} else if (tree->inst_imm <= 20) {
		int sz = tree->inst_imm;
		sz += 7;
		sz &= ~7;
		MONO_EMIT_NEW_BIALU_IMM (s, OP_SUB_IMM, X86_ESP, X86_ESP, sz);
		mini_emit_memcpy (s, X86_ESP, 0, vt->inst_basereg, vt->inst_offset, tree->inst_imm, 0);
	} else {
		tree->opcode = OP_X86_PUSH_OBJ;
		tree->inst_basereg = vt->inst_basereg;
		tree->inst_offset = vt->inst_offset;
		mono_bblock_add_inst (s->cbb, tree);
	}

	}
	return;
/* stmt: OP_OUTARG_VT(OP_ICONST) */
	case 453:
	{

	tree->opcode = OP_X86_PUSH_IMM;
	tree->inst_imm = state->left->tree->inst_c0;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_OUTARG_VT(reg) */
	case 454:
	{

	tree->opcode = OP_X86_PUSH;
	tree->sreg1 = state->left->tree->dreg;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_AMD64_OUTARG_ALIGN_STACK */
	case 455:
	{

	MONO_EMIT_NEW_BIALU_IMM (s, OP_SUB_IMM, X86_ESP, X86_ESP, tree->inst_c0);

	}
	return;
/* base: OP_INARG_VT(base) */
	case 457:
	case 503:
	{

	MONO_EMIT_NEW_LOAD_MEMBASE (s, state->reg1, state->left->tree->inst_basereg, 
					state->left->tree->inst_offset);

	}
	return;
/* reg: OP_LDADDR(OP_INARG_VT(base)) */
	case 458:
	case 459:
	case 504:
	case 505:
	{

	MONO_EMIT_NEW_LOAD_MEMBASE (s, state->reg1, state->left->left->tree->inst_basereg, 
					state->left->left->tree->inst_offset);

	}
	return;
/* reg: OP_LDADDR(OP_REGOFFSET) */
	case 460:
	case 461:
	{

	if (state->left->tree->inst_offset) {
		tree->opcode = OP_X86_LEA_MEMBASE;
		tree->sreg1 = state->left->tree->inst_basereg;
		tree->inst_imm = state->left->tree->inst_offset;
		tree->dreg = state->reg1;
	} else {
		tree->opcode = OP_MOVE;
		tree->sreg1 = state->left->tree->inst_basereg;
		tree->dreg = state->reg1;
	}
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_LDELEMA(reg, reg) */
	case 462:
	{

	guint32 size = mono_class_array_element_size (tree->klass);
	
	MONO_EMIT_BOUNDS_CHECK (s, state->left->reg1, MonoArray, max_length, state->right->reg1);

	if (size == 1 || size == 2 || size == 4 || size == 8) {
		static const int fast_log2 [] = { 1, 0, 1, -1, 2, -1, -1, -1, 3 };
		int reg;

		/* The array reg is 64 bits but the index reg is only 32 */
		tree->dreg = mono_regstate_next_float (s->rs);
		reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_UNALU (s, OP_SEXT_I4, reg, state->right->reg1);

		tree->opcode = OP_X86_LEA;
		tree->dreg = state->reg1;
		tree->sreg1 = state->left->reg1;
		tree->sreg2 = reg;
		tree->inst_imm = G_STRUCT_OFFSET (MonoArray, vector);
		tree->backend.shift_amount = fast_log2 [size];
		mono_bblock_add_inst (s->cbb, tree);
	} else {
		int mult_reg = mono_regstate_next_int (s->rs);
		int add_reg = mono_regstate_next_int (s->rs);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_MUL_IMM, mult_reg, state->right->reg1, size);
		MONO_EMIT_NEW_BIALU (s, CEE_ADD, add_reg, mult_reg, state->left->reg1);
		MONO_EMIT_NEW_BIALU_IMM (s, OP_ADD_IMM, state->reg1, add_reg, G_STRUCT_OFFSET (MonoArray, vector));
	}

	}
	return;
/* stmt: CEE_STIND_R8(OP_REGVAR, freg) */
	case 463:
	{

	/* nothing to do: the value is already on the FP stack */

	}
	return;
/* stmt: CEE_STIND_I4(base, CEE_ADD(CEE_LDIND_I4(base), OP_ICONST)) */
	case 464:
	{

	int con = state->right->right->tree->inst_c0;	

	if (con == 1) {
		tree->opcode = OP_X86_INC_MEMBASE;
	} else {
		tree->opcode = OP_X86_ADD_MEMBASE_IMM;
		tree->inst_imm = con;
	}

	tree->inst_basereg = state->left->tree->inst_basereg;
	tree->inst_offset = state->left->tree->inst_offset;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I4(base, CEE_SUB(CEE_LDIND_I4(base), OP_ICONST)) */
	case 465:
	{

	int con = state->right->right->tree->inst_c0;	

	if (con == 1) {
		tree->opcode = OP_X86_DEC_MEMBASE;
	} else {
		tree->opcode = OP_X86_SUB_MEMBASE_IMM;
		tree->inst_imm = con;
	}

	tree->inst_basereg = state->left->tree->inst_basereg;
	tree->inst_offset = state->left->tree->inst_offset;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_SUB(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)) */
	case 466:
	{

	int con = state->right->right->tree->inst_c0;	
	int dreg = state->left->tree->dreg;
	int sreg = state->right->left->left->tree->dreg;

	if (con == 1) {
		if (dreg != sreg)
			MONO_EMIT_NEW_UNALU (s, OP_MOVE, dreg, sreg);
		tree->opcode = OP_X86_DEC_REG;
		tree->dreg = tree->sreg1 = dreg;
	} else if (con == -1) {
		if (dreg != sreg)
			MONO_EMIT_NEW_UNALU (s, OP_MOVE, dreg, sreg);
		tree->opcode = OP_X86_INC_REG;
		tree->dreg = tree->sreg1 = dreg;
	} else {
		tree->opcode = OP_SUB_IMM;
		tree->inst_imm = con;
		tree->sreg1 = sreg;
		tree->dreg = dreg;
	}
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_STIND_I(OP_REGVAR, CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)) */
	case 467:
	case 468:
	{

	int con = state->right->right->tree->inst_c0;
	int dreg = state->left->tree->dreg;
	int sreg = state->right->left->left->tree->dreg;

	if (con == 1) {
		if (dreg != sreg)
			MONO_EMIT_NEW_UNALU (s, OP_MOVE, dreg, sreg);
		tree->opcode = OP_X86_INC_REG;
		tree->dreg = tree->sreg1 = dreg;
	} else if (con == -1) {
		if (dreg != sreg)
			MONO_EMIT_NEW_UNALU (s, OP_MOVE, dreg, sreg);
		tree->opcode = OP_X86_DEC_REG;
		tree->dreg = tree->sreg1 = dreg;
	} else {
		tree->opcode = OP_ADD_IMM;
		tree->inst_imm = con;
		tree->sreg1 = sreg;
		tree->dreg = dreg;
	}
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_LDIND_I2(OP_REGVAR) */
	case 469:
	{

	MONO_EMIT_UNALU (s, tree, OP_SEXT_I2, state->reg1, state->left->tree->dreg);

	}
	return;
/* stmt: CEE_STIND_I8(OP_REGVAR, OP_ICONST) */
	case 470:
	case 471:
	case 472:
	case 473:
	case 474:
	case 475:
	case 476:
	case 477:
	case 478:
	case 479:
	case 480:
	case 481:
	{

	int r = state->left->tree->dreg;
	MONO_EMIT_BIALU (s, tree, CEE_XOR, r, r, r);

	}
	return;
/* reg: OP_CEQ(OP_COMPARE(freg, freg)) */
	case 482:
	case 483:
	case 484:
	case 485:
	case 486:
	{

	MONO_EMIT_BIALU (s, tree, ceq_to_fceq (tree->opcode), state->reg1, state->left->left->reg1,
			 state->left->right->reg1);

	}
	return;
/* stmt: CEE_BNE_UN(fpcflags) */
	case 487:
	case 488:
	case 489:
	case 490:
	case 491:
	case 492:
	case 493:
	case 494:
	case 495:
	case 496:
	{

	tree->opcode = cbranch_to_fcbranch (tree->opcode);
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: CEE_POP(freg) */
	case 497:
	{

	/* we need to pop the value from the x86 FP stack */
	MONO_EMIT_UNALU (s, tree, OP_X86_FPOP, -1, state->left->reg1);	

	}
	return;
/* freg: OP_FCONV_TO_R8(freg) */
	case 498:
	case 499:
	{

	tree->opcode = OP_FMOVE;
	tree->sreg1 = state->left->reg1;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: CEE_ADD(reg, CEE_LDIND_I4(base)) */
	case 500:
	{

	MonoInst *base = state->right->left->tree;

	MONO_EMIT_BIALU_MEMBASE (cfg, tree, OP_X86_ADD_REG_MEMBASE, state->reg1, state->left->reg1, base->inst_basereg, base->inst_offset);

	}
	return;
/* reg: CEE_SUB(reg, CEE_LDIND_I4(base)) */
	case 501:
	{

	MonoInst *base = state->right->left->tree;

	MONO_EMIT_BIALU_MEMBASE (cfg, tree, OP_X86_SUB_REG_MEMBASE, state->reg1, state->left->reg1, base->inst_basereg, base->inst_offset);

	}
	return;
/* reg: CEE_MUL(reg, CEE_LDIND_I4(base)) */
	case 502:
	{

	MonoInst *base = state->right->left->tree;

	MONO_EMIT_BIALU_MEMBASE (cfg, tree, OP_X86_MUL_REG_MEMBASE, state->reg1, state->left->reg1, base->inst_basereg, base->inst_offset);

	}
	return;
/* reg: OP_LMUL(reg, OP_I8CONST) */
	case 524:
	case 525:
	case 526:
	case 527:
	{

	MONO_EMIT_BIALU_IMM (s, tree, bialu_to_bialu_imm (tree->opcode), state->reg1, state->left->reg1, state->right->tree->inst_c0);

	}
	return;
/* reg: OP_ATOMIC_ADD_NEW_I4(base, reg) */
	case 528:
	case 529:
	case 530:
	case 531:
	case 532:
	case 533:
	case 534:
	{

	tree->opcode = tree->opcode;
	tree->dreg = state->reg1;
	tree->sreg2 = state->right->reg1;
	tree->inst_basereg = state->left->tree->inst_basereg; 
	tree->inst_offset = state->left->tree->inst_offset; 
    
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* reg: OP_CALL_REG(CEE_LDIND_I(base)) */
	case 535:
	case 536:
	case 537:
	{

	tree->opcode = call_reg_to_call_membase (tree->opcode);
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VOIDCALL_REG(CEE_LDIND_I(base)) */
	case 538:
	{

	tree->opcode = call_reg_to_call_membase (tree->opcode);
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
/* stmt: OP_VCALL_REG(CEE_LDIND_I(base), reg) */
	case 539:
	{

	mono_arch_emit_this_vret_args (s, (MonoCallInst*)tree, -1, -1, state->right->reg1);
	
	tree->opcode = call_reg_to_call_membase (tree->opcode);
	tree->inst_basereg = state->left->left->tree->inst_basereg;
	tree->inst_offset = state->left->left->tree->inst_offset;
	tree->dreg = state->reg1;
	mono_bblock_add_inst (s->cbb, tree);

	}
	return;
	}
}

const short mono_burg_decode_stmt[] = {
	0,
	12,
	15,
	16,
	17,
	18,
	19,
	20,
	36,
	37,
	38,
	39,
	40,
	41,
	42,
	43,
	44,
	45,
	46,
	47,
	48,
	53,
	54,
	55,
	56,
	57,
	58,
	59,
	60,
	61,
	62,
	63,
	64,
	65,
	66,
	67,
	68,
	69,
	70,
	71,
	72,
	73,
	74,
	75,
	76,
	102,
	103,
	104,
	106,
	107,
	108,
	111,
	113,
	115,
	117,
	119,
	120,
	123,
	124,
	125,
	128,
	129,
	132,
	133,
	134,
	135,
	138,
	139,
	142,
	143,
	144,
	145,
	146,
	147,
	148,
	149,
	150,
	151,
	152,
	153,
	154,
	155,
	156,
	157,
	158,
	159,
	160,
	161,
	162,
	163,
	164,
	165,
	166,
	167,
	168,
	169,
	177,
	204,
	205,
	206,
	209,
	210,
	246,
	249,
	250,
	251,
	252,
	253,
	254,
	255,
	256,
	335,
	336,
	337,
	338,
	345,
	346,
	347,
	348,
	349,
	350,
	351,
	352,
	353,
	354,
	394,
	395,
	396,
	397,
	398,
	399,
	400,
	401,
	402,
	403,
	410,
	413,
	414,
	415,
	422,
	425,
	426,
	431,
	432,
	433,
	434,
	435,
	436,
	437,
	438,
	439,
	440,
	441,
	442,
	443,
	444,
	445,
	446,
	447,
	448,
	449,
	450,
	451,
	452,
	453,
	454,
	455,
	456,
	463,
	464,
	465,
	466,
	467,
	468,
	470,
	471,
	472,
	473,
	474,
	475,
	476,
	477,
	478,
	479,
	480,
	481,
	487,
	488,
	489,
	490,
	491,
	492,
	493,
	494,
	495,
	496,
	497,
	538,
	539,
};

const short mono_burg_decode_reg[] = {
	0,
	9,
	10,
	11,
	13,
	14,
	21,
	22,
	23,
	24,
	25,
	26,
	27,
	28,
	29,
	30,
	31,
	32,
	33,
	34,
	35,
	49,
	50,
	51,
	52,
	77,
	78,
	79,
	80,
	81,
	82,
	83,
	84,
	85,
	86,
	87,
	88,
	89,
	90,
	91,
	92,
	93,
	94,
	95,
	96,
	97,
	98,
	99,
	100,
	101,
	105,
	109,
	110,
	114,
	118,
	126,
	130,
	136,
	137,
	170,
	171,
	172,
	173,
	174,
	175,
	176,
	178,
	179,
	190,
	191,
	192,
	193,
	194,
	195,
	196,
	197,
	198,
	199,
	200,
	201,
	202,
	203,
	226,
	227,
	228,
	229,
	230,
	231,
	232,
	233,
	234,
	235,
	236,
	237,
	238,
	239,
	247,
	248,
	260,
	261,
	262,
	263,
	264,
	265,
	266,
	267,
	268,
	269,
	270,
	271,
	272,
	273,
	274,
	275,
	276,
	277,
	278,
	279,
	280,
	281,
	282,
	283,
	285,
	286,
	287,
	288,
	289,
	290,
	291,
	292,
	293,
	294,
	295,
	296,
	297,
	298,
	299,
	300,
	301,
	302,
	303,
	304,
	305,
	306,
	307,
	308,
	309,
	310,
	311,
	312,
	313,
	314,
	315,
	316,
	317,
	319,
	320,
	321,
	322,
	323,
	324,
	325,
	326,
	327,
	328,
	329,
	330,
	331,
	332,
	333,
	334,
	355,
	356,
	357,
	358,
	359,
	360,
	361,
	362,
	363,
	364,
	365,
	366,
	367,
	368,
	369,
	370,
	371,
	372,
	373,
	374,
	375,
	376,
	377,
	378,
	379,
	380,
	381,
	382,
	383,
	384,
	385,
	386,
	387,
	404,
	405,
	406,
	407,
	408,
	409,
	411,
	412,
	423,
	424,
	427,
	428,
	429,
	430,
	458,
	459,
	460,
	461,
	462,
	469,
	482,
	483,
	484,
	485,
	486,
	500,
	501,
	502,
	503,
	504,
	505,
	506,
	507,
	508,
	509,
	510,
	511,
	512,
	513,
	514,
	515,
	516,
	517,
	518,
	519,
	520,
	521,
	522,
	523,
	524,
	525,
	526,
	527,
	528,
	529,
	530,
	531,
	532,
	533,
	534,
	535,
	537,
};

const short mono_burg_decode_base[] = {
	0,
	1,
	2,
	3,
	4,
	5,
	6,
	7,
	8,
	257,
	258,
	259,
	457,
};

const short mono_burg_decode_cflags[] = {
	0,
	339,
	340,
	341,
	342,
	343,
	344,
	419,
	420,
	421,
};

const short mono_burg_decode_freg[] = {
	0,
	112,
	116,
	121,
	122,
	127,
	131,
	140,
	141,
	207,
	208,
	211,
	212,
	213,
	214,
	215,
	216,
	217,
	218,
	219,
	220,
	221,
	222,
	223,
	224,
	225,
	240,
	241,
	242,
	243,
	244,
	318,
	416,
	417,
	418,
	498,
	499,
	536,
};

const short mono_burg_decode_lreg[] = {
	0,
	180,
	181,
	182,
	183,
	184,
	185,
	186,
	187,
	188,
	189,
};

const short mono_burg_decode_fpcflags[] = {
	0,
	245,
};

const short mono_burg_decode_i8con[] = {
	0,
	284,
};

const short mono_burg_decode_c32flags[] = {
	0,
	388,
	389,
	390,
	391,
	392,
	393,
};

static void closure_reg (MBState *p, int c);

static void
closure_reg (MBState *p, int c)
{
	/* base: reg */
	if (c + 0 < p->cost[MB_NTERM_base]) {
		p->cost[MB_NTERM_base] = c + 0;
		p->rule_base = 1;
	}
}

static void
mono_burg_label_priv (MBTREE_TYPE *tree, MBCOST_DATA *data, MBState *p) {
	int arity;
	int c;
	MBState *left = NULL, *right = NULL;

	switch (mono_burg_arity [MBTREE_OP(tree)]) {
	case 0:
		break;
	case 1:
		left = MBALLOC_STATE;
		mono_burg_label_priv (MBTREE_LEFT(tree), data, left);
		break;
	case 2:
		left = MBALLOC_STATE;
		mono_burg_label_priv (MBTREE_LEFT(tree), data, left);
		right = MBALLOC_STATE;
		mono_burg_label_priv (MBTREE_RIGHT(tree), data, right);
	}

	arity = (left != NULL) + (right != NULL);
	g_assert (arity == mono_burg_arity [MBTREE_OP(tree)]);

	memset (p, 0, sizeof (MBState));
	p->op = MBTREE_OP(tree);
	p->left = left;
	p->right = right;
	p->tree = tree;
	p->cost [1] = 32767;
	p->cost [2] = 32767;
	p->cost [3] = 32767;
	p->cost [4] = 32767;
	p->cost [5] = 32767;
	p->cost [6] = 32767;
	p->cost [7] = 32767;
	p->cost [8] = 32767;
	p->cost [9] = 32767;

	switch (MBTREE_OP(tree)) {
	case OP_REGVAR: /* OP_REGVAR */
		break;
	case CEE_LDIND_I: /* CEE_LDIND_I */
		/* base: CEE_LDIND_I(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 2;
			}
		}
		/* reg: CEE_LDIND_I(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 6;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 22;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 215;
				closure_reg (p, c);
			}
		}
		break;
	case OP_REGOFFSET: /* OP_REGOFFSET */
		/* base: OP_REGOFFSET */
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 3;
			}
		}
		break;
	case OP_LDADDR: /* OP_LDADDR */
		/* base: OP_LDADDR(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 4;
			}
		}
		/* reg: OP_LDADDR(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 14;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LDADDR(OP_INARG_VT(base)) */
		if (
			p->left->op == OP_INARG_VT /* OP_INARG_VT */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 218;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LDADDR(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 220;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LDADDR(OP_VTARG_ADDR(base)) */
		if (
			p->left->op == OP_VTARG_ADDR /* OP_VTARG_ADDR */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 233;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDOBJ: /* CEE_LDOBJ */
		/* base: CEE_LDOBJ(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 5;
			}
		}
		/* reg: CEE_LDOBJ(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 15;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDOBJ(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 16;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDOBJ(OP_INARG_VT(base)) */
		if (
			p->left->op == OP_INARG_VT /* OP_INARG_VT */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 219;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDOBJ(OP_REGOFFSET) */
		if (
			p->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 221;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDOBJ(OP_VTARG_ADDR(base)) */
		if (
			p->left->op == OP_VTARG_ADDR /* OP_VTARG_ADDR */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 234;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ICONST: /* OP_ICONST */
		/* reg: OP_ICONST */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 1;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_ADD: /* CEE_ADD */
		/* base: CEE_ADD(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 6;
			}
		}
		/* base: CEE_ADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 7;
			}
		}
		/* base: CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 8;
			}
		}
		/* reg: CEE_ADD(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 179;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_ADD(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 180;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_ADD(reg, CEE_LDIND_I4(base)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 229;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_REF: /* CEE_LDIND_REF */
		/* reg: CEE_LDIND_REF(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 7;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_REF(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 21;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_REF(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 214;
				closure_reg (p, c);
			}
		}
		break;
	case OP_AOTCONST: /* OP_AOTCONST */
		/* reg: OP_AOTCONST */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 2;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LOAD_GOTADDR: /* OP_LOAD_GOTADDR */
		/* reg: OP_LOAD_GOTADDR */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 3;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_STIND_I: /* CEE_STIND_I */
		/* stmt: CEE_STIND_I(OP_REGVAR, OP_LOAD_GOTADDR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_LOAD_GOTADDR /* OP_LOAD_GOTADDR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 1;
			}
		}
		/* stmt: CEE_STIND_I(base, CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 10;
			}
		}
		/* stmt: CEE_STIND_I(base, CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 11;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 14;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 15;
			}
		}
		/* stmt: CEE_STIND_I(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 17;
			}
		}
		/* stmt: CEE_STIND_I(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 19;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 39;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 42;
			}
		}
		/* stmt: CEE_STIND_I(base, OP_I8CONST) */
		if (
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 105;
			}
		}
		/* stmt: CEE_STIND_I(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 108;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, CEE_ADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_ADD /* CEE_ADD */ &&
			p->right->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 172;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_475 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 179;
			}
		}
		/* stmt: CEE_STIND_I(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_481 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 185;
			}
		}
		break;
	case OP_PATCH_INFO: /* OP_PATCH_INFO */
		break;
	case OP_GOT_ENTRY: /* OP_GOT_ENTRY */
		/* reg: OP_GOT_ENTRY(reg, OP_PATCH_INFO) */
		if (
			p->right->op == OP_PATCH_INFO /* OP_PATCH_INFO */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 4;
				closure_reg (p, c);
			}
		}
		/* reg: OP_GOT_ENTRY(CEE_LDIND_I(OP_REGVAR), OP_PATCH_INFO) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_PATCH_INFO /* OP_PATCH_INFO */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 5;
				closure_reg (p, c);
			}
		}
		break;
	case OP_DUMMY_USE: /* OP_DUMMY_USE */
		/* stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 2;
			}
		}
		/* stmt: OP_DUMMY_USE(CEE_LDIND_I(OP_REGOFFSET)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 3;
			}
		}
		/* stmt: OP_DUMMY_USE(CEE_LDIND_REF(OP_REGOFFSET)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGOFFSET /* OP_REGOFFSET */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 4;
			}
		}
		break;
	case OP_DUMMY_STORE: /* OP_DUMMY_STORE */
		/* stmt: OP_DUMMY_STORE */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 5;
			}
		}
		break;
	case OP_NOT_REACHED: /* OP_NOT_REACHED */
		/* stmt: OP_NOT_REACHED */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 6;
			}
		}
		break;
	case OP_MEMORY_BARRIER: /* OP_MEMORY_BARRIER */
		/* stmt: OP_MEMORY_BARRIER */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 7;
			}
		}
		break;
	case CEE_LDIND_I1: /* CEE_LDIND_I1 */
		/* reg: CEE_LDIND_I1(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 8;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I1(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 210;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_U1: /* CEE_LDIND_U1 */
		/* reg: CEE_LDIND_U1(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 9;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_U1(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 19;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_I2: /* CEE_LDIND_I2 */
		/* reg: CEE_LDIND_I2(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 10;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I2(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 211;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I2(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 223;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_U2: /* CEE_LDIND_U2 */
		/* reg: CEE_LDIND_U2(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 11;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_U2(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 20;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_I4: /* CEE_LDIND_I4 */
		/* reg: CEE_LDIND_I4(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 12;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I4(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 23;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I4(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 216;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDIND_U4: /* CEE_LDIND_U4 */
		/* reg: CEE_LDIND_U4(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 13;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_U4(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 24;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_U4(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 217;
				closure_reg (p, c);
			}
		}
		break;
	case OP_OBJADDR: /* OP_OBJADDR */
		/* reg: OP_OBJADDR(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 17;
				closure_reg (p, c);
			}
		}
		break;
	case OP_VTADDR: /* OP_VTADDR */
		/* reg: OP_VTADDR(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 18;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_STIND_REF: /* CEE_STIND_REF */
		/* stmt: CEE_STIND_REF(base, CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 8;
			}
		}
		/* stmt: CEE_STIND_REF(base, CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 9;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 12;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 13;
			}
		}
		/* stmt: CEE_STIND_REF(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 18;
			}
		}
		/* stmt: CEE_STIND_REF(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 20;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 38;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 41;
			}
		}
		/* stmt: CEE_STIND_REF(base, OP_I8CONST) */
		if (
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 104;
			}
		}
		/* stmt: CEE_STIND_REF(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 107;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_474 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 178;
			}
		}
		/* stmt: CEE_STIND_REF(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_480 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 184;
			}
		}
		break;
	case CEE_STIND_I4: /* CEE_STIND_I4 */
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 16;
			}
		}
		/* stmt: CEE_STIND_I4(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 21;
			}
		}
		/* stmt: CEE_STIND_I4(base, CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 22;
			}
		}
		/* stmt: CEE_STIND_I4(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 23;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 24;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 35;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 40;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 111;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 112;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 113;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_LDIND_I4(base)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 114;
			}
		}
		/* stmt: CEE_STIND_I4(base, CEE_ADD(CEE_LDIND_I4(base), OP_ICONST)) */
		if (
			p->right->op == CEE_ADD /* CEE_ADD */ &&
			p->right->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + right->left->left->cost[MB_NTERM_base] + mono_burg_cost_464 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 169;
			}
		}
		/* stmt: CEE_STIND_I4(base, CEE_SUB(CEE_LDIND_I4(base), OP_ICONST)) */
		if (
			p->right->op == CEE_SUB /* CEE_SUB */ &&
			p->right->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + right->left->left->cost[MB_NTERM_base] + mono_burg_cost_465 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 170;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_SUB(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_SUB /* CEE_SUB */ &&
			p->right->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 171;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, CEE_ADD(CEE_LDIND_I4(OP_REGVAR), OP_ICONST)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_ADD /* CEE_ADD */ &&
			p->right->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 173;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_471 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 175;
			}
		}
		/* stmt: CEE_STIND_I4(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_477 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 181;
			}
		}
		break;
	case CEE_STIND_I1: /* CEE_STIND_I1 */
		/* stmt: CEE_STIND_I1(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 25;
			}
		}
		/* stmt: CEE_STIND_I1(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 26;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 30;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_U1(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_U1 /* CEE_LDIND_U1 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 31;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, CEE_LDIND_I1(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I1 /* CEE_LDIND_I1 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 32;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 37;
			}
		}
		/* stmt: CEE_STIND_I1(base, OP_CEQ(cflags)) */
		if (
			p->right->op == OP_CEQ /* OP_CEQ */
		)
		{
			c = left->cost[MB_NTERM_base] + right->left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 139;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_473 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 177;
			}
		}
		/* stmt: CEE_STIND_I1(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_479 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 183;
			}
		}
		break;
	case CEE_STIND_I2: /* CEE_STIND_I2 */
		/* stmt: CEE_STIND_I2(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 27;
			}
		}
		/* stmt: CEE_STIND_I2(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 28;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 29;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_U2(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_U2 /* CEE_LDIND_U2 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 33;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, CEE_LDIND_I2(OP_REGVAR)) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I2 /* CEE_LDIND_I2 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 34;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 36;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_472 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 176;
			}
		}
		/* stmt: CEE_STIND_I2(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_478 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 182;
			}
		}
		break;
	case OP_MEMCPY: /* OP_MEMCPY */
		/* stmt: OP_MEMCPY(base, base) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 43;
			}
		}
		break;
	case OP_MEMSET: /* OP_MEMSET */
		/* stmt: OP_MEMSET(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 44;
			}
		}
		break;
	case OP_GROUP: /* OP_GROUP */
		/* reg: OP_GROUP(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 66;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LDELEMA2D: /* OP_LDELEMA2D */
		/* reg: OP_LDELEMA2D(reg, OP_GROUP(reg, reg)) */
		if (
			p->right->op == OP_GROUP /* OP_GROUP */
		)
		{
			c = left->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_reg] + right->right->cost[MB_NTERM_reg] + 20;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 25;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_I1: /* CEE_CONV_I1 */
		/* reg: CEE_CONV_I1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 26;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_I2: /* CEE_CONV_I2 */
		/* reg: CEE_CONV_I2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 27;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_I4: /* CEE_CONV_I4 */
		/* reg: CEE_CONV_I4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 28;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_U1: /* CEE_CONV_U1 */
		/* reg: CEE_CONV_U1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 29;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_U2: /* CEE_CONV_U2 */
		/* reg: CEE_CONV_U2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 30;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_U4: /* CEE_CONV_U4 */
		/* reg: CEE_CONV_U4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 31;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_U: /* CEE_CONV_U */
		/* reg: CEE_CONV_U(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 32;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_I: /* CEE_CONV_I */
		/* reg: CEE_CONV_I(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 33;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I4: /* CEE_CONV_OVF_I4 */
		/* reg: CEE_CONV_OVF_I4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 34;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U4: /* CEE_CONV_OVF_U4 */
		/* reg: CEE_CONV_OVF_U4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 35;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I4_UN: /* CEE_CONV_OVF_I4_UN */
		/* reg: CEE_CONV_OVF_I4_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 36;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I1: /* CEE_CONV_OVF_I1 */
		/* reg: CEE_CONV_OVF_I1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 37;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I1_UN: /* CEE_CONV_OVF_I1_UN */
		/* reg: CEE_CONV_OVF_I1_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 38;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U1: /* CEE_CONV_OVF_U1 */
		/* reg: CEE_CONV_OVF_U1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 39;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U1_UN: /* CEE_CONV_OVF_U1_UN */
		/* reg: CEE_CONV_OVF_U1_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 40;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I2: /* CEE_CONV_OVF_I2 */
		/* reg: CEE_CONV_OVF_I2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 41;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I2_UN: /* CEE_CONV_OVF_I2_UN */
		/* reg: CEE_CONV_OVF_I2_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 42;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U2: /* CEE_CONV_OVF_U2 */
		/* reg: CEE_CONV_OVF_U2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 43;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U2_UN: /* CEE_CONV_OVF_U2_UN */
		/* reg: CEE_CONV_OVF_U2_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 44;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CEQ: /* OP_CEQ */
		/* reg: OP_CEQ(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 45;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CEQ(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 204;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CEQ(OP_COMPARE(freg, freg)) */
		if (
			p->left->op == OP_COMPARE /* OP_COMPARE */
		)
		{
			c = left->left->cost[MB_NTERM_freg] + left->right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 224;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CLT: /* OP_CLT */
		/* reg: OP_CLT(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 46;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CLT(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 205;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CLT(OP_COMPARE(freg, freg)) */
		if (
			p->left->op == OP_COMPARE /* OP_COMPARE */
		)
		{
			c = left->left->cost[MB_NTERM_freg] + left->right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 225;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CLT_UN: /* OP_CLT_UN */
		/* reg: OP_CLT_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 47;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CLT_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 206;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CLT_UN(OP_COMPARE(freg, freg)) */
		if (
			p->left->op == OP_COMPARE /* OP_COMPARE */
		)
		{
			c = left->left->cost[MB_NTERM_freg] + left->right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 227;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CGT: /* OP_CGT */
		/* reg: OP_CGT(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 48;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CGT(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 207;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CGT(OP_COMPARE(freg, freg)) */
		if (
			p->left->op == OP_COMPARE /* OP_COMPARE */
		)
		{
			c = left->left->cost[MB_NTERM_freg] + left->right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 226;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CGT_UN: /* OP_CGT_UN */
		/* reg: OP_CGT_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 2;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 49;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CGT_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 208;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CGT_UN(OP_COMPARE(freg, freg)) */
		if (
			p->left->op == OP_COMPARE /* OP_COMPARE */
		)
		{
			c = left->left->cost[MB_NTERM_freg] + left->right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 228;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LABEL: /* OP_LABEL */
		/* stmt: OP_LABEL */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 45;
			}
		}
		break;
	case OP_NOP: /* OP_NOP */
		/* stmt: OP_NOP */
		{
			c = 0;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 46;
			}
		}
		break;
	case OP_BREAK: /* OP_BREAK */
		/* stmt: OP_BREAK */
		{
			c = 0;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 47;
			}
		}
		break;
	case OP_TLS_GET: /* OP_TLS_GET */
		/* reg: OP_TLS_GET */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 50;
				closure_reg (p, c);
			}
		}
		break;
	case OP_SWITCH: /* OP_SWITCH */
		/* stmt: OP_SWITCH(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 2;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 48;
			}
		}
		break;
	case OP_BR: /* OP_BR */
		/* stmt: OP_BR */
		{
			c = 0;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 49;
			}
		}
		break;
	case OP_ARGLIST: /* OP_ARGLIST */
		/* stmt: OP_ARGLIST(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 50;
			}
		}
		break;
	case OP_CALL: /* OP_CALL */
		/* reg: OP_CALL */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 51;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CALLVIRT: /* OP_CALLVIRT */
		/* reg: OP_CALLVIRT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 52;
				closure_reg (p, c);
			}
		}
		break;
	case OP_VOIDCALLVIRT: /* OP_VOIDCALLVIRT */
		/* stmt: OP_VOIDCALLVIRT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 51;
			}
		}
		break;
	case OP_FCALLVIRT: /* OP_FCALLVIRT */
		/* freg: OP_FCALLVIRT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 1;
			}
		}
		break;
	case OP_VCALLVIRT: /* OP_VCALLVIRT */
		/* stmt: OP_VCALLVIRT(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 52;
			}
		}
		break;
	case OP_CALLVIRT_IMT: /* OP_CALLVIRT_IMT */
		/* reg: OP_CALLVIRT_IMT(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 53;
				closure_reg (p, c);
			}
		}
		break;
	case OP_VOIDCALLVIRT_IMT: /* OP_VOIDCALLVIRT_IMT */
		/* stmt: OP_VOIDCALLVIRT_IMT(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 53;
			}
		}
		break;
	case OP_FCALLVIRT_IMT: /* OP_FCALLVIRT_IMT */
		/* freg: OP_FCALLVIRT_IMT(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 2;
			}
		}
		break;
	case OP_VCALLVIRT_IMT: /* OP_VCALLVIRT_IMT */
		/* stmt: OP_VCALLVIRT_IMT(OP_GROUP(reg, reg), reg) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + left->right->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 54;
			}
		}
		break;
	case OP_CALL_REG: /* OP_CALL_REG */
		/* reg: OP_CALL_REG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 54;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CALL_REG(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 57;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CALL_REG(OP_I8CONST) */
		if (
			p->left->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 58;
				closure_reg (p, c);
			}
		}
		/* reg: OP_CALL_REG(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 264;
				closure_reg (p, c);
			}
		}
		break;
	case OP_VOIDCALL: /* OP_VOIDCALL */
		/* stmt: OP_VOIDCALL */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 55;
			}
		}
		break;
	case OP_VOIDCALL_REG: /* OP_VOIDCALL_REG */
		/* stmt: OP_VOIDCALL_REG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 56;
			}
		}
		/* stmt: OP_VOIDCALL_REG(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 66;
			}
		}
		/* stmt: OP_VOIDCALL_REG(OP_I8CONST) */
		if (
			p->left->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 67;
			}
		}
		/* stmt: OP_VOIDCALL_REG(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 197;
			}
		}
		break;
	case OP_FCALL: /* OP_FCALL */
		/* freg: OP_FCALL */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 3;
			}
		}
		break;
	case OP_FCALL_REG: /* OP_FCALL_REG */
		/* freg: OP_FCALL_REG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 4;
			}
		}
		/* freg: OP_FCALL_REG(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 7;
			}
		}
		/* freg: OP_FCALL_REG(OP_I8CONST) */
		if (
			p->left->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 8;
			}
		}
		/* freg: OP_FCALL_REG(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 37;
			}
		}
		break;
	case OP_VCALL: /* OP_VCALL */
		/* stmt: OP_VCALL(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 57;
			}
		}
		break;
	case OP_VCALL_REG: /* OP_VCALL_REG */
		/* stmt: OP_VCALL_REG(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 58;
			}
		}
		/* stmt: OP_VCALL_REG(CEE_LDIND_I(base), reg) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 198;
			}
		}
		break;
	case OP_TRAMPCALL_VTABLE: /* OP_TRAMPCALL_VTABLE */
		/* stmt: OP_TRAMPCALL_VTABLE(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 59;
			}
		}
		break;
	case OP_CALL_RGCTX: /* OP_CALL_RGCTX */
		/* reg: OP_CALL_RGCTX(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 55;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCALL_RGCTX: /* OP_FCALL_RGCTX */
		/* freg: OP_FCALL_RGCTX(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 5;
			}
		}
		break;
	case OP_VOIDCALL_RGCTX: /* OP_VOIDCALL_RGCTX */
		/* stmt: OP_VOIDCALL_RGCTX(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 60;
			}
		}
		break;
	case OP_VCALL_RGCTX: /* OP_VCALL_RGCTX */
		/* stmt: OP_VCALL_RGCTX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 61;
			}
		}
		break;
	case OP_CALL_REG_RGCTX: /* OP_CALL_REG_RGCTX */
		/* reg: OP_CALL_REG_RGCTX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 56;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCALL_REG_RGCTX: /* OP_FCALL_REG_RGCTX */
		/* freg: OP_FCALL_REG_RGCTX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 6;
			}
		}
		break;
	case OP_VOIDCALL_REG_RGCTX: /* OP_VOIDCALL_REG_RGCTX */
		/* stmt: OP_VOIDCALL_REG_RGCTX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 62;
			}
		}
		break;
	case OP_VCALL_REG_RGCTX: /* OP_VCALL_REG_RGCTX */
		/* stmt: OP_VCALL_REG_RGCTX(OP_GROUP(reg, reg), reg) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + left->right->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 63;
			}
		}
		break;
	case OP_SAVE_LMF: /* OP_SAVE_LMF */
		/* stmt: OP_SAVE_LMF */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 64;
			}
		}
		break;
	case OP_RESTORE_LMF: /* OP_RESTORE_LMF */
		/* stmt: OP_RESTORE_LMF */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 65;
			}
		}
		break;
	case OP_I8CONST: /* OP_I8CONST */
		/* reg: OP_I8CONST */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 96;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_POP: /* CEE_POP */
		/* stmt: CEE_POP(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 68;
			}
		}
		/* stmt: CEE_POP(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 69;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I1(base)) */
		if (
			p->left->op == CEE_LDIND_I1 /* CEE_LDIND_I1 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 70;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U1(base)) */
		if (
			p->left->op == CEE_LDIND_U1 /* CEE_LDIND_U1 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 71;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I2(base)) */
		if (
			p->left->op == CEE_LDIND_I2 /* CEE_LDIND_I2 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 72;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U2(base)) */
		if (
			p->left->op == CEE_LDIND_U2 /* CEE_LDIND_U2 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 73;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I4(base)) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 74;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U4(base)) */
		if (
			p->left->op == CEE_LDIND_U4 /* CEE_LDIND_U4 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 75;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 76;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_REF(base)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 77;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I1(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I1 /* CEE_LDIND_I1 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 78;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U1(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_U1 /* CEE_LDIND_U1 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 79;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I2(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I2 /* CEE_LDIND_I2 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 80;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U2(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_U2 /* CEE_LDIND_U2 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 81;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 82;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_U4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_U4 /* CEE_LDIND_U4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 83;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 84;
			}
		}
		/* stmt: CEE_POP(CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 85;
			}
		}
		/* stmt: CEE_POP(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 102;
			}
		}
		/* stmt: CEE_POP(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 0;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 196;
			}
		}
		break;
	case OP_JMP: /* OP_JMP */
		/* stmt: OP_JMP */
		{
			c = 0;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 86;
			}
		}
		break;
	case OP_THROW: /* OP_THROW */
		/* stmt: OP_THROW(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 87;
			}
		}
		/* stmt: OP_THROW(CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 90;
			}
		}
		break;
	case OP_RETHROW: /* OP_RETHROW */
		/* stmt: OP_RETHROW(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 88;
			}
		}
		break;
	case OP_THROW_OR_NULL: /* OP_THROW_OR_NULL */
		/* stmt: OP_THROW_OR_NULL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 89;
			}
		}
		break;
	case OP_CALL_HANDLER: /* OP_CALL_HANDLER */
		/* stmt: OP_CALL_HANDLER */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 91;
			}
		}
		break;
	case OP_START_HANDLER: /* OP_START_HANDLER */
		/* stmt: OP_START_HANDLER */
		{
			c = 2;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 92;
			}
		}
		/* stmt: OP_START_HANDLER */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 136;
			}
		}
		break;
	case OP_ENDFINALLY: /* OP_ENDFINALLY */
		/* stmt: OP_ENDFINALLY */
		{
			c = 2;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 93;
			}
		}
		/* stmt: OP_ENDFINALLY */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 137;
			}
		}
		break;
	case OP_ENDFILTER: /* OP_ENDFILTER */
		/* stmt: OP_ENDFILTER(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 2;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 94;
			}
		}
		/* stmt: OP_ENDFILTER(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 138;
			}
		}
		break;
	case OP_CHECK_THIS: /* OP_CHECK_THIS */
		/* stmt: OP_CHECK_THIS(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 95;
			}
		}
		break;
	case OP_CHECK_THIS_PASSTHROUGH: /* OP_CHECK_THIS_PASSTHROUGH */
		/* reg: OP_CHECK_THIS_PASSTHROUGH(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 59;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_ISINST: /* CEE_ISINST */
		/* reg: CEE_ISINST(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 60;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CISINST: /* OP_CISINST */
		/* reg: OP_CISINST(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 61;
				closure_reg (p, c);
			}
		}
		break;
	case OP_UNBOXCAST: /* OP_UNBOXCAST */
		/* reg: OP_UNBOXCAST(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 62;
				closure_reg (p, c);
			}
		}
		break;
	case OP_UNBOXCAST_REG: /* OP_UNBOXCAST_REG */
		/* reg: OP_UNBOXCAST_REG(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 63;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CASTCLASS: /* CEE_CASTCLASS */
		/* reg: CEE_CASTCLASS(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 64;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CCASTCLASS: /* OP_CCASTCLASS */
		/* reg: OP_CCASTCLASS(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 65;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_STELEM_REF: /* CEE_STELEM_REF */
		/* stmt: CEE_STELEM_REF(OP_GROUP(reg, reg), reg) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + left->right->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 96;
			}
		}
		break;
	case CEE_NEWARR: /* CEE_NEWARR */
		/* reg: CEE_NEWARR(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 67;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMUL: /* OP_LMUL */
		/* lreg: OP_LMUL(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 1;
			}
		}
		/* reg: OP_LMUL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 238;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LMUL(reg, OP_I8CONST) */
		if (
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_524 (p, data);
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 253;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMUL_OVF: /* OP_LMUL_OVF */
		/* lreg: OP_LMUL_OVF(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 2;
			}
		}
		/* reg: OP_LMUL_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 243;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMUL_OVF_UN: /* OP_LMUL_OVF_UN */
		/* lreg: OP_LMUL_OVF_UN(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 3;
			}
		}
		/* reg: OP_LMUL_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 244;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LDIV: /* OP_LDIV */
		/* lreg: OP_LDIV(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 4;
			}
		}
		/* reg: OP_LDIV(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 239;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LDIV_UN: /* OP_LDIV_UN */
		/* lreg: OP_LDIV_UN(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 5;
			}
		}
		/* reg: OP_LDIV_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 240;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LREM: /* OP_LREM */
		/* lreg: OP_LREM(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 6;
			}
		}
		/* reg: OP_LREM(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 241;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LREM_UN: /* OP_LREM_UN */
		/* lreg: OP_LREM_UN(lreg, lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 7;
			}
		}
		/* reg: OP_LREM_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 242;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSHL: /* OP_LSHL */
		/* lreg: OP_LSHL(lreg, reg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 8;
			}
		}
		/* reg: OP_LSHL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 114;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHL(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 115;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 235;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHL(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_525 (p, data);
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 254;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSHR: /* OP_LSHR */
		/* lreg: OP_LSHR(lreg, reg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 9;
			}
		}
		/* reg: OP_LSHR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 116;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 117;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 236;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_526 (p, data);
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 255;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSHR_UN: /* OP_LSHR_UN */
		/* lreg: OP_LSHR_UN(lreg, reg) */
		{
			c = left->cost[MB_NTERM_lreg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_lreg]) {
				p->cost[MB_NTERM_lreg] = c;
				p->rule_lreg = 10;
			}
		}
		/* reg: OP_LSHR_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 118;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR_UN(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 119;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 237;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LSHR_UN(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_527 (p, data);
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 256;
				closure_reg (p, c);
			}
		}
		break;
	case OP_GETCHR: /* OP_GETCHR */
		/* reg: OP_GETCHR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 4;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 68;
				closure_reg (p, c);
			}
		}
		/* reg: OP_GETCHR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 69;
				closure_reg (p, c);
			}
		}
		break;
	case OP_STR_CHAR_ADDR: /* OP_STR_CHAR_ADDR */
		/* reg: OP_STR_CHAR_ADDR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 4;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 70;
				closure_reg (p, c);
			}
		}
		break;
	case OP_STRLEN: /* OP_STRLEN */
		/* reg: OP_STRLEN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 71;
				closure_reg (p, c);
			}
		}
		break;
	case OP_GETTYPE: /* OP_GETTYPE */
		/* reg: OP_GETTYPE(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 72;
				closure_reg (p, c);
			}
		}
		break;
	case OP_GETHASHCODE: /* OP_GETHASHCODE */
		/* reg: OP_GETHASHCODE(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 73;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDLEN: /* CEE_LDLEN */
		/* reg: CEE_LDLEN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 74;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ARRAY_RANK: /* OP_ARRAY_RANK */
		/* reg: OP_ARRAY_RANK(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 75;
				closure_reg (p, c);
			}
		}
		break;
	case OP_CHECK_ARRAY_TYPE: /* OP_CHECK_ARRAY_TYPE */
		/* reg: OP_CHECK_ARRAY_TYPE(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 76;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_LDELEMA: /* CEE_LDELEMA */
		/* reg: CEE_LDELEMA(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 15;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 77;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDELEMA(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 20;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 78;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDELEMA(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 15;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 222;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_REFANYVAL: /* CEE_REFANYVAL */
		/* reg: CEE_REFANYVAL(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 79;
				closure_reg (p, c);
			}
		}
		break;
	case OP_REFANYVAL_REG: /* OP_REFANYVAL_REG */
		/* reg: OP_REFANYVAL_REG(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 80;
				closure_reg (p, c);
			}
		}
		break;
	case OP_REFANYTYPE: /* OP_REFANYTYPE */
		/* reg: OP_REFANYTYPE(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 81;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_MKREFANY: /* CEE_MKREFANY */
		/* stmt: CEE_MKREFANY(OP_GROUP(reg, OP_ICONST), reg) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */ &&
			p->left->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 97;
			}
		}
		/* stmt: CEE_MKREFANY(OP_GROUP(reg, OP_I8CONST), reg) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */ &&
			p->left->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 98;
			}
		}
		break;
	case OP_MKREFANY_REGS: /* OP_MKREFANY_REGS */
		/* stmt: OP_MKREFANY_REGS(OP_GROUP(reg, reg), OP_GROUP(reg, reg)) */
		if (
			p->left->op == OP_GROUP /* OP_GROUP */ &&
			p->right->op == OP_GROUP /* OP_GROUP */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + left->right->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_reg] + right->right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 99;
			}
		}
		break;
	case CEE_LDIND_R4: /* CEE_LDIND_R4 */
		/* freg: CEE_LDIND_R4(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 9;
			}
		}
		break;
	case CEE_LDIND_R8: /* CEE_LDIND_R8 */
		/* freg: CEE_LDIND_R8(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 10;
			}
		}
		break;
	case CEE_STIND_R4: /* CEE_STIND_R4 */
		/* stmt: CEE_STIND_R4(base, freg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 100;
			}
		}
		break;
	case CEE_STIND_R8: /* CEE_STIND_R8 */
		/* stmt: CEE_STIND_R8(base, freg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 101;
			}
		}
		/* stmt: CEE_STIND_R8(OP_REGVAR, freg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 168;
			}
		}
		break;
	case OP_R4CONST: /* OP_R4CONST */
		/* freg: OP_R4CONST */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 11;
			}
		}
		break;
	case OP_R8CONST: /* OP_R8CONST */
		/* freg: OP_R8CONST */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 12;
			}
		}
		break;
	case OP_FADD: /* OP_FADD */
		/* freg: OP_FADD(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 13;
			}
		}
		break;
	case OP_FSUB: /* OP_FSUB */
		/* freg: OP_FSUB(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 14;
			}
		}
		break;
	case OP_FMUL: /* OP_FMUL */
		/* freg: OP_FMUL(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 15;
			}
		}
		break;
	case OP_FDIV: /* OP_FDIV */
		/* freg: OP_FDIV(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 16;
			}
		}
		break;
	case OP_FREM: /* OP_FREM */
		/* freg: OP_FREM(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 17;
			}
		}
		break;
	case OP_FNEG: /* OP_FNEG */
		/* freg: OP_FNEG(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 18;
			}
		}
		break;
	case OP_CKFINITE: /* OP_CKFINITE */
		/* freg: OP_CKFINITE(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 19;
			}
		}
		break;
	case OP_SIN: /* OP_SIN */
		/* freg: OP_SIN(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 20;
			}
		}
		break;
	case OP_COS: /* OP_COS */
		/* freg: OP_COS(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 21;
			}
		}
		break;
	case OP_ABS: /* OP_ABS */
		/* freg: OP_ABS(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 22;
			}
		}
		break;
	case OP_TAN: /* OP_TAN */
		/* freg: OP_TAN(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 23;
			}
		}
		break;
	case OP_ATAN: /* OP_ATAN */
		/* freg: OP_ATAN(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 24;
			}
		}
		break;
	case OP_SQRT: /* OP_SQRT */
		/* freg: OP_SQRT(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 25;
			}
		}
		break;
	case OP_FCONV_TO_I4: /* OP_FCONV_TO_I4 */
		/* reg: OP_FCONV_TO_I4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 82;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_U4: /* OP_FCONV_TO_U4 */
		/* reg: OP_FCONV_TO_U4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 83;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_I4: /* OP_FCONV_TO_OVF_I4 */
		/* reg: OP_FCONV_TO_OVF_I4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 84;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_U4: /* OP_FCONV_TO_OVF_U4 */
		/* reg: OP_FCONV_TO_OVF_U4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 85;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_I8: /* OP_FCONV_TO_OVF_I8 */
		/* reg: OP_FCONV_TO_OVF_I8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 86;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_U8: /* OP_FCONV_TO_OVF_U8 */
		/* reg: OP_FCONV_TO_OVF_U8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 87;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_I: /* OP_FCONV_TO_OVF_I */
		/* reg: OP_FCONV_TO_OVF_I(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 88;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_OVF_U: /* OP_FCONV_TO_OVF_U */
		/* reg: OP_FCONV_TO_OVF_U(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 89;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_I: /* OP_FCONV_TO_I */
		/* reg: OP_FCONV_TO_I(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 90;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_U: /* OP_FCONV_TO_U */
		/* reg: OP_FCONV_TO_U(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 91;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_I2: /* OP_FCONV_TO_I2 */
		/* reg: OP_FCONV_TO_I2(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 92;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_U2: /* OP_FCONV_TO_U2 */
		/* reg: OP_FCONV_TO_U2(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 93;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_I1: /* OP_FCONV_TO_I1 */
		/* reg: OP_FCONV_TO_I1(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 94;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_U1: /* OP_FCONV_TO_U1 */
		/* reg: OP_FCONV_TO_U1(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 95;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_R8: /* OP_FCONV_TO_R8 */
		/* freg: OP_FCONV_TO_R8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 26;
			}
		}
		/* freg: OP_FCONV_TO_R8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 0;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 35;
			}
		}
		break;
	case OP_FCONV_TO_R4: /* OP_FCONV_TO_R4 */
		/* freg: OP_FCONV_TO_R4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 27;
			}
		}
		/* freg: OP_FCONV_TO_R4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 0;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 36;
			}
		}
		break;
	case CEE_CONV_R4: /* CEE_CONV_R4 */
		/* freg: CEE_CONV_R4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 2;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 28;
			}
		}
		break;
	case CEE_CONV_R8: /* CEE_CONV_R8 */
		/* freg: CEE_CONV_R8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 2;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 29;
			}
		}
		break;
	case CEE_CONV_R_UN: /* CEE_CONV_R_UN */
		/* freg: CEE_CONV_R_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 2;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 30;
			}
		}
		break;
	case OP_COMPARE: /* OP_COMPARE */
		/* fpcflags: OP_COMPARE(freg, freg) */
		{
			c = left->cost[MB_NTERM_freg] + right->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_fpcflags]) {
				p->cost[MB_NTERM_fpcflags] = c;
				p->rule_fpcflags = 1;
			}
		}
		/* c32flags: OP_COMPARE(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 1;
			}
		}
		/* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 2;
			}
		}
		/* c32flags: OP_COMPARE(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 3;
			}
		}
		/* c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), reg) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 4;
			}
		}
		/* c32flags: OP_COMPARE(CEE_LDIND_I4(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 5;
			}
		}
		/* c32flags: OP_COMPARE(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_c32flags]) {
				p->cost[MB_NTERM_c32flags] = c;
				p->rule_c32flags = 6;
			}
		}
		/* cflags: OP_COMPARE(CEE_LDIND_I4(base), reg) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 7;
			}
		}
		/* cflags: OP_COMPARE(CEE_LDIND_I4(base), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 8;
			}
		}
		/* cflags: OP_COMPARE(reg, CEE_LDIND_I4(base)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 9;
			}
		}
		break;
	case CEE_LDIND_I8: /* CEE_LDIND_I8 */
		/* reg: CEE_LDIND_I8(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 97;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_LDIND_I8(OP_REGVAR) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 209;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_STIND_I8: /* CEE_STIND_I8 */
		/* stmt: CEE_STIND_I8(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 103;
			}
		}
		/* stmt: CEE_STIND_I8(base, OP_I8CONST) */
		if (
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 106;
			}
		}
		/* stmt: CEE_STIND_I8(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 109;
			}
		}
		/* stmt: CEE_STIND_I8(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 110;
			}
		}
		/* stmt: CEE_STIND_I8(OP_REGVAR, reg) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 135;
			}
		}
		/* stmt: CEE_STIND_I8(OP_REGVAR, OP_ICONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_470 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 174;
			}
		}
		/* stmt: CEE_STIND_I8(OP_REGVAR, OP_I8CONST) */
		if (
			p->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = mono_burg_cost_476 (p, data);
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 180;
			}
		}
		break;
	case OP_LADD: /* OP_LADD */
		/* base: OP_LADD(base, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_base] + 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 9;
			}
		}
		/* base: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 10;
			}
		}
		/* base: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 11;
			}
		}
		/* reg: OP_LADD(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 98;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LADD(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 99;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LADD(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 100;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LADD(CEE_LDIND_I(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 101;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LADD(CEE_LDIND_REF(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 102;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSUB: /* OP_LSUB */
		/* reg: OP_LSUB(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 103;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LNEG: /* OP_LNEG */
		/* reg: OP_LNEG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 104;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LNOT: /* OP_LNOT */
		/* reg: OP_LNOT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 105;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LAND: /* OP_LAND */
		/* reg: OP_LAND(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 106;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LOR: /* OP_LOR */
		/* reg: OP_LOR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 107;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LXOR: /* OP_LXOR */
		/* reg: OP_LXOR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 108;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LADD_OVF: /* OP_LADD_OVF */
		/* reg: OP_LADD_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 109;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LADD_OVF_UN: /* OP_LADD_OVF_UN */
		/* reg: OP_LADD_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 110;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSUB_OVF: /* OP_LSUB_OVF */
		/* reg: OP_LSUB_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 111;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LSUB_OVF_UN: /* OP_LSUB_OVF_UN */
		/* reg: OP_LSUB_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 112;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LONG_SHRUN_32: /* OP_LONG_SHRUN_32 */
		/* reg: OP_LONG_SHRUN_32(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 113;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_I8: /* CEE_CONV_I8 */
		/* reg: CEE_CONV_I8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 120;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_U8: /* CEE_CONV_U8 */
		/* reg: CEE_CONV_U8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 121;
				closure_reg (p, c);
			}
		}
		/* i8con: CEE_CONV_U8(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 0;
			if (c < p->cost[MB_NTERM_i8con]) {
				p->cost[MB_NTERM_i8con] = c;
				p->rule_i8con = 1;
			}
		}
		break;
	case OP_FCONV_TO_I8: /* OP_FCONV_TO_I8 */
		/* reg: OP_FCONV_TO_I8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 122;
				closure_reg (p, c);
			}
		}
		break;
	case OP_FCONV_TO_U8: /* OP_FCONV_TO_U8 */
		/* reg: OP_FCONV_TO_U8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 123;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_I4: /* OP_LCONV_TO_I4 */
		/* reg: OP_LCONV_TO_I4(i8con) */
		{
			c = left->cost[MB_NTERM_i8con] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 124;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LCONV_TO_I4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 125;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_U4: /* OP_LCONV_TO_U4 */
		/* reg: OP_LCONV_TO_U4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 126;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_U8: /* OP_LCONV_TO_U8 */
		/* reg: OP_LCONV_TO_U8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 127;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_I8: /* OP_LCONV_TO_I8 */
		/* reg: OP_LCONV_TO_I8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 128;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_U: /* OP_LCONV_TO_U */
		/* reg: OP_LCONV_TO_U(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 129;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_I: /* OP_LCONV_TO_I */
		/* reg: OP_LCONV_TO_I(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 130;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_I1: /* OP_LCONV_TO_I1 */
		/* reg: OP_LCONV_TO_I1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 131;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_U1: /* OP_LCONV_TO_U1 */
		/* reg: OP_LCONV_TO_U1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 132;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_I2: /* OP_LCONV_TO_I2 */
		/* reg: OP_LCONV_TO_I2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 133;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_U2: /* OP_LCONV_TO_U2 */
		/* reg: OP_LCONV_TO_U2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 134;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I1: /* OP_LCONV_TO_OVF_I1 */
		/* reg: OP_LCONV_TO_OVF_I1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 135;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I1_UN: /* OP_LCONV_TO_OVF_I1_UN */
		/* reg: OP_LCONV_TO_OVF_I1_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 136;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U1: /* OP_LCONV_TO_OVF_U1 */
		/* reg: OP_LCONV_TO_OVF_U1(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 137;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U1_UN: /* OP_LCONV_TO_OVF_U1_UN */
		/* reg: OP_LCONV_TO_OVF_U1_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 138;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I2: /* OP_LCONV_TO_OVF_I2 */
		/* reg: OP_LCONV_TO_OVF_I2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 139;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I2_UN: /* OP_LCONV_TO_OVF_I2_UN */
		/* reg: OP_LCONV_TO_OVF_I2_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 140;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U2: /* OP_LCONV_TO_OVF_U2 */
		/* reg: OP_LCONV_TO_OVF_U2(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 141;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U2_UN: /* OP_LCONV_TO_OVF_U2_UN */
		/* reg: OP_LCONV_TO_OVF_U2_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 142;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I4: /* OP_LCONV_TO_OVF_I4 */
		/* reg: OP_LCONV_TO_OVF_I4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 143;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I4_UN: /* OP_LCONV_TO_OVF_I4_UN */
		/* reg: OP_LCONV_TO_OVF_I4_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 144;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U4: /* OP_LCONV_TO_OVF_U4 */
		/* reg: OP_LCONV_TO_OVF_U4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 145;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U4_UN: /* OP_LCONV_TO_OVF_U4_UN */
		/* reg: OP_LCONV_TO_OVF_U4_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 146;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I_UN: /* OP_LCONV_TO_OVF_I_UN */
		/* reg: OP_LCONV_TO_OVF_I_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 147;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I8_UN: /* OP_LCONV_TO_OVF_I8_UN */
		/* reg: OP_LCONV_TO_OVF_I8_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 148;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U8: /* OP_LCONV_TO_OVF_U8 */
		/* reg: OP_LCONV_TO_OVF_U8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 149;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U: /* OP_LCONV_TO_OVF_U */
		/* reg: OP_LCONV_TO_OVF_U(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 150;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I8: /* OP_LCONV_TO_OVF_I8 */
		/* reg: OP_LCONV_TO_OVF_I8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 151;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U_UN: /* OP_LCONV_TO_OVF_U_UN */
		/* reg: OP_LCONV_TO_OVF_U_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 152;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_U8_UN: /* OP_LCONV_TO_OVF_U8_UN */
		/* reg: OP_LCONV_TO_OVF_U8_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 153;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_OVF_I: /* OP_LCONV_TO_OVF_I */
		/* reg: OP_LCONV_TO_OVF_I(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 154;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_R_UN: /* OP_LCONV_TO_R_UN */
		/* freg: OP_LCONV_TO_R_UN(lreg) */
		{
			c = left->cost[MB_NTERM_lreg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 31;
			}
		}
		/* freg: OP_LCONV_TO_R_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 32;
			}
		}
		break;
	case CEE_CONV_OVF_I8: /* CEE_CONV_OVF_I8 */
		/* reg: CEE_CONV_OVF_I8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 155;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I: /* CEE_CONV_OVF_I */
		/* reg: CEE_CONV_OVF_I(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 156;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I8_UN: /* CEE_CONV_OVF_I8_UN */
		/* reg: CEE_CONV_OVF_I8_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 157;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U8_UN: /* CEE_CONV_OVF_U8_UN */
		/* reg: CEE_CONV_OVF_U8_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 158;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_I_UN: /* CEE_CONV_OVF_I_UN */
		/* reg: CEE_CONV_OVF_I_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 159;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U_UN: /* CEE_CONV_OVF_U_UN */
		/* reg: CEE_CONV_OVF_U_UN(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 160;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U8: /* CEE_CONV_OVF_U8 */
		/* reg: CEE_CONV_OVF_U8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 161;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_CONV_OVF_U: /* CEE_CONV_OVF_U */
		/* reg: CEE_CONV_OVF_U(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 162;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALLVIRT: /* OP_LCALLVIRT */
		/* reg: OP_LCALLVIRT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 163;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALLVIRT_IMT: /* OP_LCALLVIRT_IMT */
		/* reg: OP_LCALLVIRT_IMT(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 164;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALL: /* OP_LCALL */
		/* reg: OP_LCALL */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 165;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALL_RGCTX: /* OP_LCALL_RGCTX */
		/* reg: OP_LCALL_RGCTX(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 166;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALL_REG: /* OP_LCALL_REG */
		/* reg: OP_LCALL_REG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 167;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LCALL_REG(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 169;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LCALL_REG(OP_I8CONST) */
		if (
			p->left->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 170;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LCALL_REG(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 265;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCALL_REG_RGCTX: /* OP_LCALL_REG_RGCTX */
		/* reg: OP_LCALL_REG_RGCTX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 168;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCOMPARE: /* OP_LCOMPARE */
		/* cflags: OP_LCOMPARE(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 1;
			}
		}
		/* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), reg) */
		if (
			p->left->op == CEE_LDIND_I8 /* CEE_LDIND_I8 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 2;
			}
		}
		/* cflags: OP_LCOMPARE(CEE_LDIND_I(OP_REGVAR), CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 3;
			}
		}
		/* cflags: OP_LCOMPARE(CEE_LDIND_I8(OP_REGVAR), OP_ICONST) */
		if (
			p->left->op == CEE_LDIND_I8 /* CEE_LDIND_I8 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */ &&
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = mono_burg_cost_342 (p, data);
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 4;
			}
		}
		/* cflags: OP_LCOMPARE(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_343 (p, data);
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 5;
			}
		}
		/* cflags: OP_LCOMPARE(reg, OP_I8CONST) */
		if (
			p->right->op == OP_I8CONST /* OP_I8CONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + mono_burg_cost_344 (p, data);
			if (c < p->cost[MB_NTERM_cflags]) {
				p->cost[MB_NTERM_cflags] = c;
				p->rule_cflags = 6;
			}
		}
		break;
	case CEE_BNE_UN: /* CEE_BNE_UN */
		/* stmt: CEE_BNE_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 115;
			}
		}
		/* stmt: CEE_BNE_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 125;
			}
		}
		/* stmt: CEE_BNE_UN(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 186;
			}
		}
		break;
	case CEE_BEQ: /* CEE_BEQ */
		/* stmt: CEE_BEQ(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 116;
			}
		}
		/* stmt: CEE_BEQ(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 126;
			}
		}
		/* stmt: CEE_BEQ(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 187;
			}
		}
		break;
	case CEE_BLT: /* CEE_BLT */
		/* stmt: CEE_BLT(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 117;
			}
		}
		/* stmt: CEE_BLT(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 127;
			}
		}
		/* stmt: CEE_BLT(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 188;
			}
		}
		break;
	case CEE_BLT_UN: /* CEE_BLT_UN */
		/* stmt: CEE_BLT_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 118;
			}
		}
		/* stmt: CEE_BLT_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 128;
			}
		}
		/* stmt: CEE_BLT_UN(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 189;
			}
		}
		break;
	case CEE_BGT: /* CEE_BGT */
		/* stmt: CEE_BGT(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 119;
			}
		}
		/* stmt: CEE_BGT(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 129;
			}
		}
		/* stmt: CEE_BGT(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 190;
			}
		}
		break;
	case CEE_BGT_UN: /* CEE_BGT_UN */
		/* stmt: CEE_BGT_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 120;
			}
		}
		/* stmt: CEE_BGT_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 130;
			}
		}
		/* stmt: CEE_BGT_UN(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 191;
			}
		}
		break;
	case CEE_BGE: /* CEE_BGE */
		/* stmt: CEE_BGE(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 121;
			}
		}
		/* stmt: CEE_BGE(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 131;
			}
		}
		/* stmt: CEE_BGE(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 192;
			}
		}
		break;
	case CEE_BGE_UN: /* CEE_BGE_UN */
		/* stmt: CEE_BGE_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 122;
			}
		}
		/* stmt: CEE_BGE_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 132;
			}
		}
		/* stmt: CEE_BGE_UN(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 193;
			}
		}
		break;
	case CEE_BLE: /* CEE_BLE */
		/* stmt: CEE_BLE(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 123;
			}
		}
		/* stmt: CEE_BLE(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 133;
			}
		}
		/* stmt: CEE_BLE(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 194;
			}
		}
		break;
	case CEE_BLE_UN: /* CEE_BLE_UN */
		/* stmt: CEE_BLE_UN(cflags) */
		{
			c = left->cost[MB_NTERM_cflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 124;
			}
		}
		/* stmt: CEE_BLE_UN(c32flags) */
		{
			c = left->cost[MB_NTERM_c32flags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 134;
			}
		}
		/* stmt: CEE_BLE_UN(fpcflags) */
		{
			c = left->cost[MB_NTERM_fpcflags] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 195;
			}
		}
		break;
	case CEE_AND: /* CEE_AND */
		/* reg: CEE_AND(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 171;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_AND(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 172;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_OR: /* CEE_OR */
		/* reg: CEE_OR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 173;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_OR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 174;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_XOR: /* CEE_XOR */
		/* reg: CEE_XOR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 175;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_XOR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 176;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_NEG: /* CEE_NEG */
		/* reg: CEE_NEG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 177;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_NOT: /* CEE_NOT */
		/* reg: CEE_NOT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 178;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_ADD_OVF: /* CEE_ADD_OVF */
		/* reg: CEE_ADD_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 181;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_ADD_OVF_UN: /* CEE_ADD_OVF_UN */
		/* reg: CEE_ADD_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 182;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SUB: /* CEE_SUB */
		/* reg: CEE_SUB(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 183;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SUB(reg, CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->right->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 184;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SUB(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 185;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SUB(reg, CEE_LDIND_I4(base)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 230;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SUB_OVF: /* CEE_SUB_OVF */
		/* reg: CEE_SUB_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 186;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SUB_OVF_UN: /* CEE_SUB_OVF_UN */
		/* reg: CEE_SUB_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 187;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SHL: /* CEE_SHL */
		/* reg: CEE_SHL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 188;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SHL(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 189;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SHR: /* CEE_SHR */
		/* reg: CEE_SHR(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 190;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SHR(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 191;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_SHR_UN: /* CEE_SHR_UN */
		/* reg: CEE_SHR_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 192;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_SHR_UN(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 193;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_MUL: /* CEE_MUL */
		/* reg: CEE_MUL(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 194;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_MUL(reg, OP_ICONST) */
		if (
			p->right->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 195;
				closure_reg (p, c);
			}
		}
		/* reg: CEE_MUL(reg, CEE_LDIND_I4(base)) */
		if (
			p->right->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */
		)
		{
			c = left->cost[MB_NTERM_reg] + right->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 231;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_MUL_OVF: /* CEE_MUL_OVF */
		/* reg: CEE_MUL_OVF(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 196;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_MUL_OVF_UN: /* CEE_MUL_OVF_UN */
		/* reg: CEE_MUL_OVF_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 197;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_DIV: /* CEE_DIV */
		/* reg: CEE_DIV(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 198;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_DIV_UN: /* CEE_DIV_UN */
		/* reg: CEE_DIV_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 199;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_REM: /* CEE_REM */
		/* reg: CEE_REM(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 200;
				closure_reg (p, c);
			}
		}
		break;
	case CEE_REM_UN: /* CEE_REM_UN */
		/* reg: CEE_REM_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 201;
				closure_reg (p, c);
			}
		}
		break;
	case OP_IMUL_IMM: /* OP_IMUL_IMM */
		/* reg: OP_IMUL_IMM(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 202;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMUL_IMM: /* OP_LMUL_IMM */
		/* reg: OP_LMUL_IMM(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 203;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LCONV_TO_R8: /* OP_LCONV_TO_R8 */
		/* freg: OP_LCONV_TO_R8(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 33;
			}
		}
		break;
	case OP_LCONV_TO_R4: /* OP_LCONV_TO_R4 */
		/* freg: OP_LCONV_TO_R4(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_freg]) {
				p->cost[MB_NTERM_freg] = c;
				p->rule_freg = 34;
			}
		}
		break;
	case OP_LOCALLOC: /* OP_LOCALLOC */
		/* reg: OP_LOCALLOC(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 212;
				closure_reg (p, c);
			}
		}
		/* reg: OP_LOCALLOC(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 213;
				closure_reg (p, c);
			}
		}
		break;
	case OP_SETRET: /* OP_SETRET */
		/* stmt: OP_SETRET(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 140;
			}
		}
		/* stmt: OP_SETRET(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 141;
			}
		}
		/* stmt: OP_SETRET(CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 142;
			}
		}
		/* stmt: OP_SETRET(CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 143;
			}
		}
		/* stmt: OP_SETRET(CEE_LDIND_U4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_U4 /* CEE_LDIND_U4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 144;
			}
		}
		/* stmt: OP_SETRET(CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 145;
			}
		}
		/* stmt: OP_SETRET(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 146;
			}
		}
		/* stmt: OP_SETRET(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 147;
			}
		}
		break;
	case OP_OUTARG: /* OP_OUTARG */
		/* stmt: OP_OUTARG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 148;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 150;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_I4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I4 /* CEE_LDIND_I4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 151;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_U4(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_U4 /* CEE_LDIND_U4 */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 152;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_I(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 153;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_I(base)) */
		if (
			p->left->op == CEE_LDIND_I /* CEE_LDIND_I */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 154;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_REF(base)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 155;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDIND_REF(OP_REGVAR)) */
		if (
			p->left->op == CEE_LDIND_REF /* CEE_LDIND_REF */ &&
			p->left->left->op == OP_REGVAR /* OP_REGVAR */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 156;
			}
		}
		/* stmt: OP_OUTARG(CEE_LDOBJ(reg)) */
		if (
			p->left->op == CEE_LDOBJ /* CEE_LDOBJ */
		)
		{
			c = left->left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 157;
			}
		}
		/* stmt: OP_OUTARG(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 158;
			}
		}
		break;
	case OP_OUTARG_REG: /* OP_OUTARG_REG */
		/* stmt: OP_OUTARG_REG(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 149;
			}
		}
		break;
	case OP_OUTARG_R4: /* OP_OUTARG_R4 */
		/* stmt: OP_OUTARG_R4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 159;
			}
		}
		break;
	case OP_OUTARG_R8: /* OP_OUTARG_R8 */
		/* stmt: OP_OUTARG_R8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 160;
			}
		}
		break;
	case OP_AMD64_OUTARG_XMMREG_R4: /* OP_AMD64_OUTARG_XMMREG_R4 */
		/* stmt: OP_AMD64_OUTARG_XMMREG_R4(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 161;
			}
		}
		break;
	case OP_AMD64_OUTARG_XMMREG_R8: /* OP_AMD64_OUTARG_XMMREG_R8 */
		/* stmt: OP_AMD64_OUTARG_XMMREG_R8(freg) */
		{
			c = left->cost[MB_NTERM_freg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 162;
			}
		}
		break;
	case OP_OUTARG_VT: /* OP_OUTARG_VT */
		/* stmt: OP_OUTARG_VT(CEE_LDOBJ(base)) */
		if (
			p->left->op == CEE_LDOBJ /* CEE_LDOBJ */
		)
		{
			c = left->left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 163;
			}
		}
		/* stmt: OP_OUTARG_VT(OP_ICONST) */
		if (
			p->left->op == OP_ICONST /* OP_ICONST */
		)
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 164;
			}
		}
		/* stmt: OP_OUTARG_VT(reg) */
		{
			c = left->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 165;
			}
		}
		break;
	case OP_AMD64_OUTARG_ALIGN_STACK: /* OP_AMD64_OUTARG_ALIGN_STACK */
		/* stmt: OP_AMD64_OUTARG_ALIGN_STACK */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 166;
			}
		}
		break;
	case OP_AMD64_SAVE_SP_TO_LMF: /* OP_AMD64_SAVE_SP_TO_LMF */
		/* stmt: OP_AMD64_SAVE_SP_TO_LMF */
		{
			c = 1;
			if (c < p->cost[MB_NTERM_stmt]) {
				p->cost[MB_NTERM_stmt] = c;
				p->rule_stmt = 167;
			}
		}
		break;
	case OP_INARG_VT: /* OP_INARG_VT */
		/* base: OP_INARG_VT(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_base]) {
				p->cost[MB_NTERM_base] = c;
				p->rule_base = 12;
			}
		}
		break;
	case OP_VTARG_ADDR: /* OP_VTARG_ADDR */
		/* reg: OP_VTARG_ADDR(base) */
		{
			c = left->cost[MB_NTERM_base] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 232;
				closure_reg (p, c);
			}
		}
		break;
	case OP_IMIN: /* OP_IMIN */
		/* reg: OP_IMIN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 245;
				closure_reg (p, c);
			}
		}
		break;
	case OP_IMIN_UN: /* OP_IMIN_UN */
		/* reg: OP_IMIN_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 246;
				closure_reg (p, c);
			}
		}
		break;
	case OP_IMAX: /* OP_IMAX */
		/* reg: OP_IMAX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 247;
				closure_reg (p, c);
			}
		}
		break;
	case OP_IMAX_UN: /* OP_IMAX_UN */
		/* reg: OP_IMAX_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 248;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMIN: /* OP_LMIN */
		/* reg: OP_LMIN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 249;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMIN_UN: /* OP_LMIN_UN */
		/* reg: OP_LMIN_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 250;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMAX: /* OP_LMAX */
		/* reg: OP_LMAX(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 251;
				closure_reg (p, c);
			}
		}
		break;
	case OP_LMAX_UN: /* OP_LMAX_UN */
		/* reg: OP_LMAX_UN(reg, reg) */
		{
			c = left->cost[MB_NTERM_reg] + right->cost[MB_NTERM_reg] + 0;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 252;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_ADD_NEW_I4: /* OP_ATOMIC_ADD_NEW_I4 */
		/* reg: OP_ATOMIC_ADD_NEW_I4(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 257;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_ADD_NEW_I8: /* OP_ATOMIC_ADD_NEW_I8 */
		/* reg: OP_ATOMIC_ADD_NEW_I8(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 258;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_ADD_I4: /* OP_ATOMIC_ADD_I4 */
		/* reg: OP_ATOMIC_ADD_I4(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 259;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_ADD_I8: /* OP_ATOMIC_ADD_I8 */
		/* reg: OP_ATOMIC_ADD_I8(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 260;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_EXCHANGE_I4: /* OP_ATOMIC_EXCHANGE_I4 */
		/* reg: OP_ATOMIC_EXCHANGE_I4(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 261;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_EXCHANGE_I8: /* OP_ATOMIC_EXCHANGE_I8 */
		/* reg: OP_ATOMIC_EXCHANGE_I8(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 262;
				closure_reg (p, c);
			}
		}
		break;
	case OP_ATOMIC_CAS_IMM_I4: /* OP_ATOMIC_CAS_IMM_I4 */
		/* reg: OP_ATOMIC_CAS_IMM_I4(base, reg) */
		{
			c = left->cost[MB_NTERM_base] + right->cost[MB_NTERM_reg] + 1;
			if (c < p->cost[MB_NTERM_reg]) {
				p->cost[MB_NTERM_reg] = c;
				p->rule_reg = 263;
				closure_reg (p, c);
			}
		}
		break;
	default:
#ifdef MBGET_OP_NAME
		g_error ("unknown operator: %s", MBGET_OP_NAME(MBTREE_OP(tree)));
#else
		g_error ("unknown operator: 0x%04x", MBTREE_OP(tree));
#endif
	}

}

MBState *
mono_burg_label (MBTREE_TYPE *tree, MBCOST_DATA *data)
{
	MBState *p = MBALLOC_STATE;
	mono_burg_label_priv (tree, data, p);
	return p->rule_stmt ? p : NULL;
}

int
mono_burg_rule (MBState *state, int goal)
{
	g_return_val_if_fail (state != NULL, 0);
	g_return_val_if_fail (goal > 0, 0);

	switch (goal) {
	case MB_NTERM_stmt:
		return mono_burg_decode_stmt [state->rule_stmt];
	case MB_NTERM_reg:
		return mono_burg_decode_reg [state->rule_reg];
	case MB_NTERM_base:
		return mono_burg_decode_base [state->rule_base];
	case MB_NTERM_cflags:
		return mono_burg_decode_cflags [state->rule_cflags];
	case MB_NTERM_freg:
		return mono_burg_decode_freg [state->rule_freg];
	case MB_NTERM_lreg:
		return mono_burg_decode_lreg [state->rule_lreg];
	case MB_NTERM_fpcflags:
		return mono_burg_decode_fpcflags [state->rule_fpcflags];
	case MB_NTERM_i8con:
		return mono_burg_decode_i8con [state->rule_i8con];
	case MB_NTERM_c32flags:
		return mono_burg_decode_c32flags [state->rule_c32flags];
	default: g_assert_not_reached ();
	}
	return 0;
}

MBState **
mono_burg_kids (MBState *state, int rulenr, MBState *kids [])
{
	g_return_val_if_fail (state != NULL, NULL);
	g_return_val_if_fail (kids != NULL, NULL);

	switch (rulenr) {
	case 1:
		kids[0] = state;
		break;
	case 2:
	case 3:
	case 4:
	case 5:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 14:
	case 15:
	case 16:
	case 17:
	case 18:
	case 19:
	case 20:
	case 29:
	case 30:
	case 34:
	case 35:
	case 40:
	case 41:
	case 42:
	case 43:
	case 44:
	case 49:
	case 50:
	case 51:
	case 52:
	case 63:
	case 64:
	case 65:
	case 66:
	case 67:
	case 68:
	case 69:
	case 70:
	case 71:
	case 102:
	case 103:
	case 104:
	case 105:
	case 107:
	case 109:
	case 119:
	case 121:
	case 134:
	case 135:
	case 136:
	case 137:
	case 138:
	case 139:
	case 140:
	case 141:
	case 143:
	case 152:
	case 153:
	case 154:
	case 155:
	case 156:
	case 157:
	case 158:
	case 159:
	case 160:
	case 164:
	case 165:
	case 166:
	case 167:
	case 211:
	case 212:
	case 247:
	case 258:
	case 259:
	case 263:
	case 264:
	case 284:
	case 329:
	case 333:
	case 334:
	case 335:
	case 337:
	case 341:
	case 342:
	case 389:
	case 390:
	case 392:
	case 409:
	case 411:
	case 412:
	case 413:
	case 414:
	case 423:
	case 427:
	case 428:
	case 429:
	case 430:
	case 431:
	case 432:
	case 433:
	case 434:
	case 436:
	case 439:
	case 440:
	case 441:
	case 442:
	case 445:
	case 453:
	case 455:
	case 456:
	case 460:
	case 461:
	case 466:
	case 467:
	case 468:
	case 469:
	case 470:
	case 471:
	case 472:
	case 473:
	case 474:
	case 475:
	case 476:
	case 477:
	case 478:
	case 479:
	case 480:
	case 481:
		break;
	case 6:
	case 13:
	case 21:
	case 22:
	case 23:
	case 24:
	case 25:
	case 26:
	case 27:
	case 28:
	case 31:
	case 32:
	case 33:
	case 36:
	case 37:
	case 38:
	case 39:
	case 47:
	case 48:
	case 54:
	case 55:
	case 58:
	case 60:
	case 76:
	case 78:
	case 79:
	case 80:
	case 81:
	case 82:
	case 83:
	case 84:
	case 85:
	case 86:
	case 87:
	case 88:
	case 89:
	case 90:
	case 91:
	case 92:
	case 93:
	case 94:
	case 95:
	case 96:
	case 97:
	case 98:
	case 99:
	case 100:
	case 101:
	case 106:
	case 108:
	case 110:
	case 111:
	case 112:
	case 118:
	case 120:
	case 122:
	case 123:
	case 125:
	case 126:
	case 127:
	case 128:
	case 142:
	case 161:
	case 162:
	case 168:
	case 169:
	case 170:
	case 171:
	case 172:
	case 173:
	case 175:
	case 176:
	case 179:
	case 191:
	case 193:
	case 194:
	case 195:
	case 196:
	case 197:
	case 198:
	case 199:
	case 201:
	case 203:
	case 207:
	case 208:
	case 218:
	case 219:
	case 220:
	case 221:
	case 222:
	case 223:
	case 224:
	case 225:
	case 226:
	case 227:
	case 228:
	case 229:
	case 230:
	case 231:
	case 232:
	case 233:
	case 234:
	case 235:
	case 236:
	case 237:
	case 238:
	case 239:
	case 240:
	case 241:
	case 242:
	case 243:
	case 244:
	case 246:
	case 248:
	case 250:
	case 251:
	case 252:
	case 253:
	case 254:
	case 255:
	case 257:
	case 261:
	case 262:
	case 266:
	case 267:
	case 275:
	case 277:
	case 279:
	case 281:
	case 282:
	case 283:
	case 285:
	case 286:
	case 287:
	case 288:
	case 289:
	case 290:
	case 291:
	case 292:
	case 293:
	case 294:
	case 295:
	case 296:
	case 297:
	case 298:
	case 299:
	case 300:
	case 301:
	case 302:
	case 303:
	case 304:
	case 305:
	case 306:
	case 307:
	case 308:
	case 309:
	case 310:
	case 311:
	case 312:
	case 313:
	case 314:
	case 315:
	case 316:
	case 317:
	case 318:
	case 319:
	case 320:
	case 321:
	case 322:
	case 323:
	case 324:
	case 325:
	case 326:
	case 327:
	case 330:
	case 331:
	case 343:
	case 344:
	case 345:
	case 346:
	case 347:
	case 348:
	case 349:
	case 350:
	case 351:
	case 352:
	case 353:
	case 354:
	case 356:
	case 358:
	case 360:
	case 361:
	case 362:
	case 364:
	case 368:
	case 369:
	case 373:
	case 375:
	case 377:
	case 379:
	case 386:
	case 387:
	case 393:
	case 394:
	case 395:
	case 396:
	case 397:
	case 398:
	case 399:
	case 400:
	case 401:
	case 402:
	case 403:
	case 404:
	case 405:
	case 406:
	case 407:
	case 408:
	case 415:
	case 416:
	case 417:
	case 418:
	case 424:
	case 425:
	case 426:
	case 435:
	case 437:
	case 438:
	case 447:
	case 448:
	case 449:
	case 450:
	case 451:
	case 454:
	case 457:
	case 487:
	case 488:
	case 489:
	case 490:
	case 491:
	case 492:
	case 493:
	case 494:
	case 495:
	case 496:
	case 497:
	case 498:
	case 499:
	case 503:
	case 524:
	case 525:
	case 526:
	case 527:
		kids[0] = state->left;
		break;
	case 45:
	case 46:
	case 53:
	case 57:
	case 59:
	case 75:
	case 113:
	case 114:
	case 115:
	case 116:
	case 124:
	case 129:
	case 130:
	case 131:
	case 132:
	case 163:
	case 174:
	case 178:
	case 180:
	case 181:
	case 182:
	case 183:
	case 184:
	case 185:
	case 186:
	case 187:
	case 188:
	case 189:
	case 190:
	case 192:
	case 200:
	case 202:
	case 209:
	case 210:
	case 213:
	case 214:
	case 215:
	case 216:
	case 217:
	case 245:
	case 249:
	case 256:
	case 260:
	case 265:
	case 268:
	case 269:
	case 270:
	case 271:
	case 272:
	case 273:
	case 274:
	case 276:
	case 278:
	case 280:
	case 328:
	case 332:
	case 339:
	case 355:
	case 357:
	case 359:
	case 363:
	case 365:
	case 366:
	case 367:
	case 370:
	case 371:
	case 372:
	case 374:
	case 376:
	case 378:
	case 380:
	case 381:
	case 382:
	case 383:
	case 384:
	case 385:
	case 388:
	case 462:
	case 506:
	case 507:
	case 508:
	case 509:
	case 510:
	case 511:
	case 512:
	case 513:
	case 514:
	case 515:
	case 516:
	case 517:
	case 518:
	case 519:
	case 520:
	case 521:
	case 522:
	case 523:
	case 528:
	case 529:
	case 530:
	case 531:
	case 532:
	case 533:
	case 534:
		kids[0] = state->left;
		kids[1] = state->right;
		break;
	case 56:
	case 338:
		kids[0] = state->right->left;
		break;
	case 61:
	case 62:
	case 72:
	case 73:
	case 74:
	case 336:
	case 340:
	case 391:
	case 410:
	case 463:
		kids[0] = state->right;
		break;
	case 77:
		kids[0] = state->left;
		kids[1] = state->right->left;
		kids[2] = state->right->right;
		break;
	case 117:
	case 133:
	case 177:
		kids[0] = state->left->left;
		kids[1] = state->left->right;
		kids[2] = state->right;
		break;
	case 144:
	case 145:
	case 146:
	case 147:
	case 148:
	case 149:
	case 150:
	case 151:
	case 420:
	case 443:
	case 444:
	case 446:
	case 452:
	case 458:
	case 459:
	case 504:
	case 505:
	case 535:
	case 536:
	case 537:
	case 538:
		kids[0] = state->left->left;
		break;
	case 204:
	case 205:
	case 419:
	case 539:
		kids[0] = state->left->left;
		kids[1] = state->right;
		break;
	case 206:
		kids[0] = state->left->left;
		kids[1] = state->left->right;
		kids[2] = state->right->left;
		kids[3] = state->right->right;
		break;
	case 421:
	case 422:
	case 500:
	case 501:
	case 502:
		kids[0] = state->left;
		kids[1] = state->right->left;
		break;
	case 464:
	case 465:
		kids[0] = state->left;
		kids[1] = state->right->left->left;
		break;
	case 482:
	case 483:
	case 484:
	case 485:
	case 486:
		kids[0] = state->left->left;
		kids[1] = state->left->right;
		break;
	default:
		g_assert_not_reached ();
	}
	return kids;
}

